

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include <adrcore/features/featurefilter.h>

#define DBGFILTER DBGALL

namespace adore
{

	namespace core
	{

		void filterFeatures( FeatureGrid &candidates, float minDistance, float minimumFraction, int maxPoints )
		{
			if ( candidates.getSize() == 0 ) return;

			//filter points too near to strongest corners
			vector<FeaturePoint* > near;
			FeatureGrid::Iterator it = candidates.getBegin();
			FeatureGrid::Iterator endIt = candidates.getEnd();
			FeaturePoint *pt;

			pt = *it;
			float maxStrength = pt->getStrength();
			float minStrength = maxStrength * minimumFraction;

			int numPts = 0;

			int sizebefore = candidates.getSize();
			ADORE_UNUSED( sizebefore );
			int nearremoved = 0;

			while ( it != endIt && numPts < maxPoints )
			{
				pt = *it;
				++it;

				DBGOUT( DBGFILTER, "\nstrength : %f\n", pt->getStrength() );

				if ( ( minimumFraction > 0 ) && ( pt->getStrength() < minStrength ) )
				{
//					printf( "breaking\n" );
					break;
				}

				if ( minDistance > 0 )
				{
#ifndef NDEBUG
					FeatureGrid::Iterator itti = candidates.pointSet.find( pt );
					assert( itti != candidates.pointSet.end() );
#endif
					candidates.pickNear( ( *pt )( 0 ), ( *pt )( 1 ), minDistance, near );
					DBGOUT( DBGFILTER, "found %zd points near %p(%f, %f)\n", near.size(), pt, ( *pt )( 0 ), ( *pt )( 1 ) );
					DBGOUT( DBGALL, "first point was %p\n", near[0] );

					//this is a crude way do eliminate ambiguities thar arise from
					//features close together sharing the same strength
					//since these features are mostly of low strength this may not be too harmful
					bool removeAllInVicinity = false;
					for ( uint j = 0; j < near.size(); ++j )
					{
						if ( ( pt != near[j] ) && ( pt->getStrength() == near[j]->getStrength() ) )
						{
							removeAllInVicinity = true;
							DBGOUT( DBGNORMAL, "Featurefilter: Strength ambiguity detected, removing point itself, too. Strength has been %f\n", pt->getStrength() );
							break;
						}
					}

					for ( uint j = 0; j < near.size(); ++j )
					{
						if ( ( pt != near[j] ) || removeAllInVicinity )
						{
							if ( *it == near[j] ) ++it;//save iterator
							FeaturePoint *npt = near[j];
							DBGOUT( DBGFILTER, "removing (%f, %f), str: %f\n", ( *npt )( 0 ), ( *npt )( 1 ), npt->getStrength() );
							FeaturePoint *rpt = candidates.remove( ( *npt )( 0 ), ( *npt )( 1 ) );
							assert( rpt == npt );
							ADORE_UNUSED( rpt );
							delete npt;
							++nearremoved;
						}
					}

					near.clear();
#ifndef NDEBUG
//					candidates.pickNear( ( *pt )( 0 ), ( *pt )( 1 ), minDistance, near );
//					DBGOUT( DBGALL, "check found %zd points near: %p\n\n", near.size(), near[0] );
//					assert( near.size() == 1 );//only the point itself shall remain
//					assert( near[0] == pt );
//					near.clear();
#endif
				}
				++numPts;
			}

			int fracremoved = 0;
			if ( minimumFraction > 0.0 || numPts != candidates.getSize() )
			{
				while ( it != endIt )
				{
					pt = *it;
					++it;
					//         printf("removing (to weak %f [%f, %f]) (%f, %f)\n", (*pd)[0], minStrength, maxStrength, (*pt)[0], (*pt)[1]);
					bool removed = candidates.remove( ( *pt )( 0 ), ( *pt )( 1 ) );
					assert( removed );
					ADORE_UNUSED( removed );
					++fracremoved;
					delete pt;
				}
			}
			assert_msg( candidates.getSize() + fracremoved + nearremoved == sizebefore, "%d + %d + %d != %d\n", candidates.getSize(), fracremoved, nearremoved, sizebefore );

//			DBGOUT( DBGNORMAL, "near-removals: %d, size/fraction-removals: %d\n", nearRemoved, sizeFracRemoved );
		}

	}
}
