

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_FEATURESFEATUREFILTER_H
#define ADORE_FEATURESFEATUREFILTER_H

#include <adrcore/search/grid2d.h>
#include <adrcore/types/feature.h>
#include <adrcore/types/featurepoint.h>
#include <adrcore/imgprocessing/channel.h>

namespace adore
{

	namespace core
	{

		/**
		 * a feature grid. use this to store your candidate features in order to filter them with filterfeatures(...).
		 */
		typedef Grid2DInhomo<FeaturePoint *, FeatureStrengthComparator<> > FeatureGrid;

		/**
		 * filters features
		 * @param candidates a FeatureGrid that contains candidate features
		 * @param minDistance the minimum distance features shall have to each other
		 * @param minimumFraction the minimum fraction of strength relative to the strongest feature every feature must have
		 * @param maxPoints the maximum number of points which shall survive
		 *  
		*/
		void filterFeatures ( FeatureGrid &candidates, float minDistance, float minimumFraction = 0.0, int maxPoints = INT_MAX );

		/**
		 * adjusts features to assume the closest sub-pixel extremum in its 3x3 neighborhood
		 * if such an extremum does not exist, the feature is removed.
		 *  
		 * @tparam ImgT the image type
		 * @param candidates a FeatureGrid containing the feature points
		 * @param img the image used to compute the sub-pixel positions
		 */
		template<class ImgT>
		void adjustFeatures ( FeatureGrid &candidates, Channel<ImgT> &img )
		{
			const float dxMax = 0.5F, dyMax = 0.5F;

			FeatureGrid::Iterator it = candidates.getBegin();
			FeatureGrid::Iterator endIt = candidates.getEnd();

			UsingROI ur ( &img );//destructor restores ROI

			while ( it != endIt )
			{
				FeaturePoint *pt = *it;
				float dx, dy;

				++it;//has to be incremented before a removal happens
//				cout << *pt << endl;;

//				printChannelRegion( img, int( ( *pt )( 0 ) ) - 1, int( ( *pt )( 1 ) ) - 1, 3, 3 );

				img.setROI ( int ( ( *pt ) ( 0 ) ) - 1, int ( ( *pt ) ( 1 ) ) - 1, 3, 3 );
				bool success = subCoorQuadratic ( img.getIteratorBegin(), dx, dy );

				if ( success && fabs ( dx ) < dxMax && fabs ( dy ) < dyMax )
				{
					( *pt ) ( 0 ) += dx;
					( *pt ) ( 1 ) += dy;
//					printf( "pushed back: (%f, %f)\n", ( *pt )( 0 ), ( *pt )( 1 ) );
				}
				else
				{
//					printf( "dx: %f < %f\n", dx, dxMax );
//					printf( "dy: %f < %f\n", dy, dyMax );
//					printf( "quadratic: %s\n", success ? "sucess" : "no success" );

					bool removed = candidates.remove ( ( *pt ) ( 0 ), ( *pt ) ( 1 ) );
					assert ( removed ); ADORE_UNUSED ( removed );

					/*if ( pt )*/ delete pt;
				}
//				pressKey();
			}
		}
	}

}

#endif
