

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGBILINEARINTERPOLATOR_H
#define ADORE_IMGPROCESSINGBILINEARINTERPOLATOR_H

#include <adrcore/util/calctype.h>
#include <adrcore/imgprocessing/channel.h>

namespace adore
{

	namespace core
	{

		/**
		 * Interpolates pixel values
		 * @tparam T the image type
		 *  
		*/
		template<class T>
		class BilinearInterpolator
		{
			private:
				typedef typename core::calcT<T>::type resultT;

			public:
				/**
				 * Constructor
				 */
				BilinearInterpolator()
				{
					init( 0, 0 );
				}

				/**
				 * inits the Interpolator. Pre-computes values to speed up interpolation at the same image coordinates, e.g. in a multi-channel image.
				 * @param x the sub-pixel x coordinate
				 * @param y the sub-pixel y coordinate
				 */
				void init( const float x, const float y )
				{
//					assert_msg( x >=0 && x <= getWidth() - 1, "x = %f, width = %d", x, getWidth() );
//					assert_msg( y >=0 && y <= getHeight() - 1, "y = %f, height = %d", y, getHeight() );

					lx = ( int )floor( x );
					ux = ( int )ceil( x );
					ly = ( int )floor( y );
					uy = ( int )ceil( y );
					vx = x - float( lx );
					vy = y - float( ly );
				}

				/**
				 * @param ch the channel whose values shall be interpolated
				 * @return the value of the channel at the point 
				 */
				resultT valueOf( const Channel<T> &ch ) const
				{
					float gll = *( ch.getBase() + ch.getOffset( lx, ly ) );
					float glu = *( ch.getBase() + ch.getOffset( lx, uy ) );
					float gul = *( ch.getBase() + ch.getOffset( ux, ly ) );
					float guu = *( ch.getBase() + ch.getOffset( ux, uy ) );

					resultT result = resultT( gll + vx * ( gul - gll ) + vy * ( glu - gll ) + vx * vy * ( gll + guu - glu - gul ) );
					return result;
				}

				/**
				 * @return 0, the nonexistent image border necessary to use this interpolator
				 */
				int border() const
				{
					return 0;
				}

			private:

				int lx, ly;	///lower reference position
				int ux, uy;	///upper reference position
				float vx, vy;	///weights
		};

	}

}

#endif
