

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORECOLORNORMALIZER_H
#define ADORE_CORECOLORNORMALIZER_H

#include <adrcore/imgprocessing/channelprocessor.h>
#include <adrcore/imgprocessing/channelfunctions.h>
#include <adrcore/util/linearnormalization.h>

namespace adore
{
	namespace core
	{

		template<class T> T getColorNormalizerRange()
		{
			assert( false ); return 0;
		}
		template<> inline uint8_t getColorNormalizerRange()
		{
			return 255;
		}
		template<> inline float getColorNormalizerRange()
		{
			return 1.0;
		}

		/**
		     * normalized the color of an image region globally
		     
		*/
		template<class inT, class ouT = inT>
		class ColorNormalizer : public ChannelProcessor<1, 1, inT, ouT>
		{
			public:
				typedef inT img_out_type;
				typedef ouT img_in_type;
			public:
				ColorNormalizer( ouT _range = 0 ):range( _range )
				{
					;
				}

				void process( Channel<img_in_type> *in, Channel<img_out_type> *out )
				{
					typename Channel<img_in_type>::Iterator2D srcIt = in->getIteratorBegin();
					typename Channel<img_out_type>::Iterator2D dstIt = out->getIteratorBegin();
					typename Channel<img_in_type>::Iterator2D srcEnd = in->getIteratorEnd();
					normalizeLinear( srcIt, srcEnd, dstIt, range );
				}

			private:
				ouT range;
		};

		/**
		 * normalized the color of an image region locally
		 
		*/
		template<class inT, class ouT = inT>
		class ColorNormalizerLocal : public ChannelProcessor<1, 1, inT, ouT>
		{
			public:
				typedef inT img_out_type;
				typedef ouT img_in_type;
			public:
				ColorNormalizerLocal( ouT _range = 0, uint _hood = 3 ):range( _range ), hood( _hood )
				{
					;
				}

				void process( Channel<img_in_type> *in, Channel<img_out_type> *out )
				{
					UsingROI ur( in );
					typename Channel<img_in_type>::Iterator2D srcIt = in->getIteratorBegin();
					typename Channel<img_out_type>::Iterator2D dstIt = out->getIteratorBegin();
					typename Channel<img_in_type>::Iterator2D srcEnd = in->getIteratorEnd();
					for ( ; srcIt != srcEnd; ++srcIt, ++dstIt )
					{
						*dstIt = processLocal( in, srcIt.getPointer() );
					}
				}

			private:

				img_out_type processLocal( Channel<img_in_type> *in, img_in_type *ptr )
				{
					float factor = 1.0;
					img_in_type min;

					uint x, y;
					in->getCoors( ptr, x, y );

					setROISavely<inT>( *in, x-hood, y-hood, hood<<1, hood<<1 );

					typename Channel<img_in_type>::Iterator2D srcIt = in->getIteratorBegin();
					typename Channel<img_in_type>::Iterator2D srcEnd = in->getIteratorEnd();
					normalizeLinearCoeffs<typename Channel<img_in_type>::Iterator2D>( srcIt, srcEnd, range, factor, min );

					return img_out_type( factor * float( *ptr - min ) );
				}

			private:

				ouT range;
				uint hood;
		};

	}
}

#endif
