

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORECORONAINDICES_H
#define ADORE_CORECORONAINDICES_H

#include "roi.h"

namespace adore
{

	namespace core
	{

		/**
			This class helps to identify the pixel that belong to the corona of another.
			it is useful to handle image borders
			 
				321
				480
				567
			 */
		class CoronaIndices
		{
			public:
				typedef vector<int>::const_iterator iterator;

				CoronaIndices( CONNECTIVITY con )
				{
					if ( con == EIGHT )
					{
						indices[0].resize( 5 );
						indices[0][0] = 2;
						indices[0][1] = 3;
						indices[0][2] = 4;
						indices[0][3] = 5;
						indices[0][4] = 6;
						indices[1].resize( 3 );
						indices[1][0] = 4;
						indices[1][1] = 5;
						indices[1][2] = 6;
						indices[2].resize( 5 );
						indices[2][0] = 4;
						indices[2][1] = 5;
						indices[2][2] = 6;
						indices[2][3] = 7;
						indices[2][4] = 0;
						indices[3].resize( 3 );
						indices[3][0] = 6;
						indices[3][1] = 7;
						indices[3][2] = 0;
						indices[4].resize( 5 );
						indices[4][0] = 6;
						indices[4][1] = 7;
						indices[4][2] = 0;
						indices[4][3] = 1;
						indices[4][4] = 2;
						indices[5].resize( 3 );
						indices[5][0] = 0;
						indices[5][1] = 1;
						indices[5][2] = 2;
						indices[6].resize( 5 );
						indices[6][0] = 0;
						indices[6][1] = 1;
						indices[6][2] = 2;
						indices[6][3] = 3;
						indices[6][4] = 4;
						indices[7].resize( 3 );
						indices[7][0] = 2;
						indices[7][1] = 3;
						indices[7][2] = 4;
						indices[8].resize( 8 );
						indices[8][0] = 0;
						indices[8][1] = 1;
						indices[8][2] = 2;
						indices[8][3] = 3;
						indices[8][4] = 4;
						indices[8][5] = 5;
						indices[8][6] = 6;
						indices[8][7] = 7;
					}
					else//FOUR
					{
						indices[0].resize( 3 );
						indices[0][0] = 2;
						indices[0][1] = 4;
						indices[0][2] = 6;
						indices[1].resize( 2 );
						indices[1][0] = 4;
						indices[1][1] = 6;
						indices[2].resize( 3 );
						indices[2][0] = 4;
						indices[2][1] = 6;
						indices[2][2] = 0;
						indices[3].resize( 2 );
						indices[3][0] = 6;
						indices[3][1] = 0;
						indices[4].resize( 3 );
						indices[4][0] = 6;
						indices[4][1] = 0;
						indices[4][2] = 2;
						indices[5].resize( 2 );
						indices[5][0] = 0;
						indices[5][1] = 2;
						indices[6].resize( 3 );
						indices[6][0] = 0;
						indices[6][1] = 2;
						indices[6][2] = 4;
						indices[7].resize( 2 );
						indices[7][0] = 2;
						indices[7][1] = 4;
						indices[8].resize( 4 );
						indices[8][0] = 0;
						indices[8][1] = 2;
						indices[8][2] = 4;
						indices[8][3] = 6;
					}
				}

				void init( uint x, uint y, const ROI &r )
				{
					int idxmap[3][3] = {{3, 2, 1}, {4, 8, 0}, {5, 6, 7}};

					int xIdx = 1;
					if ( x >= r.getX() + r.getWidth() - 1 ) xIdx = 2;
					else if ( x <= r.getX() ) xIdx = 0;

					int yIdx = 1;
					if ( y >= r.getY() + r.getHeight() - 1 ) yIdx = 2;
					else if ( y <= r.getY() ) yIdx = 0;

					index = idxmap[yIdx][xIdx];
				}

				iterator begin() const
				{
					return indices[index].begin();
				}

				iterator end() const
				{
					return indices[index].end();
				}

				int size() const
				{
					return indices[index].size();
				}

				int operator[]( uint idx ) const
				{
					return indices[index][idx];
				}

			private:
				uint index;
				vector<int> indices[9];
		};

	}
}

#endif
