

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREDERIVATIVES_H
#define ADORE_COREDERIVATIVES_H

#include <adrcore/imgprocessing/channel.h>
#include <adrcore/util/calctype.h>
#include <cassert>

namespace adore
{

	namespace core
	{

		template<class imgT>
		void get2ndDerivates( typename Channel<imgT>::Iterator2D &it,
		                      typename sCalcT<imgT>::type &dxdx,
		                      typename sCalcT<imgT>::type &dydy,
		                      typename sCalcT<imgT>::type &dxdy )
		{
			typedef typename sCalcT<imgT>::type calcT;

			calcT center2 = 2 * calcT( *it );
			dxdx = calcT( *it.getPointer( -1, 0 ) + *it.getPointer( 1, 0 ) - center2 );
			dydy = calcT( *it.getPointer( 0, -1 ) + *it.getPointer( 0, 1 ) - center2 );
			dxdy = calcT( 0.25 * ( ( *it.getPointer( 1, 1 ) - *it.getPointer( -1, 1 ) ) -
			                       ( *it.getPointer( 1, -1 ) - *it.getPointer( -1, -1 ) ) ) );
		}

		template<class imgT>
		void get2ndDerivates( Channel<imgT> *chan, uint x, uint y,
		                      typename sCalcT<imgT>::type &dxdx,
		                      typename sCalcT<imgT>::type &dydy,
		                      typename sCalcT<imgT>::type &dxdy )
		{
			typedef typename sCalcT<imgT>::type calcT;

			calcT center2 = 2 * calcT( *chan->getPointer( x, y ) );
			dxdx = calcT( *chan->getPointer( x-1, y ) + *chan->getPointer( x+1, y ) - center2 );
			dydy = calcT( *chan->getPointer( x, y-1 ) + *chan->getPointer( x, y+1 ) - center2 );
			dxdy = calcT( 0.25 * ( ( *chan->getPointer( x+1, y+1 ) - *chan->getPointer( x-1, y+1 ) ) -
			                       ( *chan->getPointer( x+1, y-1 ) - *chan->getPointer( x-1, y-1 ) ) ) );
		}

		template<class imgT>
		void get1stDerivates( typename Channel<imgT>::Iterator2D &it,
		                      typename sCalcT<imgT>::type &gx,
		                      typename sCalcT<imgT>::type &gy )
		{
			typedef typename sCalcT<imgT>::type calcT;

			gx = calcT( *it.getPointer( 1, 0 ) - *it.getPointer( -1, 0 ) );
			gy = calcT( *it.getPointer( 0, 1 ) - *it.getPointer( 0, -1 ) );
		}

		template<class imgT>
		void get1stDerivates( Channel<imgT> *chan, uint x, uint y,
		                      typename sCalcT<imgT>::type &gx,
		                      typename sCalcT<imgT>::type &gy )
		{
			typedef typename sCalcT<imgT>::type calcT;

			gx = calcT( *chan->getPointer( x+1, y ) - *chan->getPointer( x-1, y ) );
			gy = calcT( *chan->getPointer( x, y+1 ) - *chan->getPointer( x, y-1 ) );
		}

		template<class imgT>
		void get1stDerivatesSobel( Channel<imgT> *chan, uint x, uint y,
		                           typename sCalcT<imgT>::type &gx,
		                           typename sCalcT<imgT>::type &gy )
		{
			typedef typename sCalcT<imgT>::type calcT;

			gx = calcT( *chan->getPointer( x+1, y ) - *chan->getPointer( x-1, y ) );
			gx <<= 1;
			gx += calcT( *chan->getPointer( x+1, y-1 ) - *chan->getPointer( x-1, y-1 ) );
			gx += calcT( *chan->getPointer( x+1, y+1 ) - *chan->getPointer( x-1, y+1 ) );
			gx >>= 2;

			gy = calcT( *chan->getPointer( x, y+1 ) - *chan->getPointer( x, y-1 ) );
			gy <<= 1;
			gy += calcT( *chan->getPointer( x-1, y+1 ) - *chan->getPointer( x-1, y-1 ) );
			gy += calcT( *chan->getPointer( x+1, y+1 ) - *chan->getPointer( x+1, y-1 ) );
			gy >>= 2;
		}

		template<>
		inline void get1stDerivatesSobel<float>( Channel<float> *chan, uint x, uint y,
		                sCalcT<float>::type &gx,
		                sCalcT<float>::type &gy )
		{
			ADORE_UNUSED( chan );
			ADORE_UNUSED( x );
			ADORE_UNUSED( y );
			ADORE_UNUSED( gx );
			ADORE_UNUSED( gy );
			std::abort();
		}
		template<>
		inline void get1stDerivatesSobel<double>( Channel<double> *chan, uint x, uint y,
		                sCalcT<double>::type &gx,
		                sCalcT<double>::type &gy )
		{
			ADORE_UNUSED( chan );
			ADORE_UNUSED( x );
			ADORE_UNUSED( y );
			ADORE_UNUSED( gx );
			ADORE_UNUSED( gy );
			std::abort();
		}

		template<class imgT>
		void get1stDerivatesSobel( typename Channel<imgT>::Iterator2D &it,
		                           uint x, uint y,
		                           typename sCalcT<imgT>::type &gx,
		                           typename sCalcT<imgT>::type &gy )
		{
			typedef typename sCalcT<imgT>::type calcT;

			gx = calcT( *it.getPointer( 1, 0 ) - *it.getPointer( -1, 0 ) );
			gx <<= 1;
			gx += calcT( *it.getPointer( 1, -1 ) - *it.getPointer( -1, -1 ) );
			gx += calcT( *it.getPointer( 1, 1 ) - *it.getPointer( -1, 1 ) );
			gx >>= 2;

			gy = calcT( *it.getPointer( 0, 1 ) - *it.getPointer( 0, -1 ) );
			gy <<= 1;
			gy += calcT( *it.getPointer( -1, 1 ) - *it.getPointer( -1, -1 ) );
			gy += calcT( *it.getPointer( 1, 1 ) - *it.getPointer( 1, -1 ) );
			gy >>= 2;
		}

		template<class imgT>
		void get1stDerivatesColor( ChannelSet<imgT> &chan, uint x, uint y,
		                           typename sCalcT<imgT>::type &dxx,
		                           typename sCalcT<imgT>::type &dyy,
		                           typename sCalcT<imgT>::type &dxy )
		{
			typedef typename sCalcT<imgT>::type calcT;

			for ( int i = 0; i < chan.size(); ++i )
			{
//				get1stDerivates( chan[i], x, y, gx, gy );

			}
		}

		///////
		///////  2nd moment like in harrisfast
		///////

		inline void addG( float a, float b, float c, float d, double weight, double &gxx, double &gyy, double &gxy )
		{
			double xx = b - a + d - c;
			double yy = c - a + d - b;
			gxy += weight * xx * yy;
			gxx += weight * xx * xx;
			gyy += weight * yy * yy;
		}

		template<class imgT>
		void calc2ndMomentMat( Channel<imgT> *in, const AMatrixf &weights, double scale, AMatrixf &result )
		{
			assert( isEven( weights.rows() ) );

			int x = in->getROI().getX();
			int y = in->getROI().getY();
			int width = in->getROI().getWidth();
			int height = in->getROI().getHeight();

                        assert( width == (int) weights.cols() + 1 );
                        assert( height == (int) weights.rows() + 1 );

			typename Channel<imgT>::Iterator1D it1    = in->getXIterator( y, x );
			typename Channel<imgT>::Iterator1D it2    = in->getXIterator( y, x + 1 );
			typename Channel<imgT>::Iterator1D it3    = in->getXIterator( y + 1, x );
			typename Channel<imgT>::Iterator1D it4    = in->getXIterator( y + 1, x + 1 );
			typename AMatrixf::const_iterator wit = weights.begin();

			double gxx = 0, gyy = 0, gxy = 0;
//			uint X, Y;
			for ( int j = 0; j < height - 1; ++j )
			{
				for ( int i = 0; i < width - 1; ++i, ++wit, ++it1, ++it2, ++it3, ++it4 )
				{
//					in->getCoors( it1.getPointer(), X, Y );
//					printf( "%d, %d\n", X, Y );
					addG( *it1, *it2, *it3, *it4, *wit, gxx, gyy, gxy );
				}
				it1.nextLine( in->getWStep() );
				it2.nextLine( in->getWStep() );
				it3.nextLine( in->getWStep() );
				it4.nextLine( in->getWStep() );
			}

			const double shift = 1.0; //to stay in a sane range
			gxx = shift * scale * gxx;
			gyy = shift * scale * gyy;
			gxy = shift * scale * gxy;

			result = gxx, gxy, gxy, gyy;
		}

	}
}

#endif
