

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGDIZENZOGRADIENTS_H
#define ADORE_IMGPROCESSINGDIZENZOGRADIENTS_H

#include <adrcore/util/calctype.h>
#include <adrcore/imgprocessing/channelprocessor.h>
#include <adrcore/imgprocessing/channel.h>
#include <adrcore/imgprocessing/channelfunctions.h>
#include <adrcore/imgprocessing/imageplane.h>
#include <adrcore/imgprocessing/sobeloperator2pass.h>
#include <adrcore/imgprocessing/borderfiller.h>

namespace adore
{

	namespace core
	{

		/**
		 
		*/
		template <class inT, class ouT>
		class DiZenzoGradients : public ChannelProcessor<0, 3, inT, ouT>
		{
				typedef typename signedT<typename imgCalcT<inT>::type>::type gradT;

			public:
				DiZenzoGradients():gradientX( NULL ), gradientY( NULL ), bf( 1 )
				{
					;
				}

				virtual ~DiZenzoGradients()
				{
					;
				}

				void process( ChannelSet<inT> &in, ChannelSet<ouT> &out );

				void setGradientXOperator( ChannelProcessor< 1, 1, inT, gradT>* theValue )
				{
					gradientX = theValue;
				}

				ChannelProcessor< 1, 1, inT, gradT >* getGradientXOperator() const
				{
					return gradientX;
				}

				void setGradientYOperator( ChannelProcessor< 1, 1, inT, gradT>* theValue )
				{
					gradientY = theValue;
				}

				ChannelProcessor< 1, 1, inT, gradT >* getGradientYOperator() const
				{
					return gradientY;
				}

			protected:

				ChannelProcessor<1, 1, inT, gradT> *gradientX;
				ChannelProcessor<1, 1, inT, gradT> *gradientY;
				ImagePlane<gradT> gradX;
				ImagePlane<gradT> gradY;
				BorderFiller<gradT> bf;
		};

		template <class inT, class ouT>
		void DiZenzoGradients<inT, ouT>::process( ChannelSet<inT> &in,
		                ChannelSet<ouT> &out )
		{
			assert( out.size() >= 3 );
			uint numIn = in.size();

			ChannelProcessor<1, 1, inT, gradT> *xGrad = gradientX;
			ChannelProcessor<1, 1, inT, gradT> *yGrad = gradientY;
			ImagePlane<inT> *tmpImg = NULL;
			if ( !gradientX || !gradientY )
			{
				tmpImg = new ImagePlane<inT>( in[0]->getWidth(), in[0]->getHeight(), 1 );
				if ( !gradientX )
				{
					xGrad = new SobelOperator2Pass<inT, DIR_X>( tmpImg );
				}
				if ( !gradientY )
				{
					yGrad = new SobelOperator2Pass<inT, DIR_Y>( tmpImg );
				}
			}

			out[0]->fill( 0 );
			out[1]->fill( 0 );
			out[2]->fill( 0 );

			gradX.init( in[0]->getWidth(), in[0]->getHeight(), 1 );
			gradY.init( in[0]->getWidth(), in[0]->getHeight(), 1 );
			bf.process( &gradX, &gradX );
			bf.process( &gradY, &gradY );

//			ouT min, max, *minL, *maxL;
			for ( uint i = 0; i < numIn; ++i )
			{
				Channel<inT> *plane = in[i];
//				printf( "plane\n" );
//				printChannelRegion( *plane, 0, 0, 20, 20 );
				xGrad->process( plane, &gradX );
				yGrad->process( plane, &gradY );
//				printf( "gradX\n" );
//				printChannelRegion( gradX, 0, 0, 20, 20 );
//				printf( "gradY\n" );
//				printChannelRegion( gradY, 0, 0, 20, 20 );

				typename Channel<gradT>::Iterator2D itGx = gradX.getIteratorBegin();
				typename Channel<gradT>::Iterator2D itGy = gradY.getIteratorBegin();

				typename Channel<ouT>::Iterator2D itGxx = out[0]->getIteratorBegin();
				typename Channel<ouT>::Iterator2D itGyy = out[1]->getIteratorBegin();
				typename Channel<ouT>::Iterator2D itGxy = out[2]->getIteratorBegin();

				typename Channel<gradT>::Iterator2D stop = gradX.getIteratorEnd();

				for ( ; itGx != stop; ++itGx, ++itGy, ++itGxx, ++itGyy, ++itGxy )
				{
//					cout << ": " << *itGx << "  " << *itGy << " " << endl;
//					*itGxx = updateAvg( *itGxx, *itGx * *itGx, int( i ) + 1 );
//					*itGyy = updateAvg( *itGyy, *itGy * *itGy, int( i ) + 1 );
//					*itGxy = updateAvg( *itGxy, *itGx * *itGy, int( i ) + 1 );
					*itGxx += *itGx * *itGx;
					*itGyy += *itGy * *itGy;
					*itGxy += *itGx * *itGy;
//					cout << ": " << *itGxx << "  " << *itGyy << "  " << *itGxy << " " << endl;
//					pressKey();
				}

//				minMaxLoc( *out[0], min, max, minL, maxL );
//				cout << min << " " << max << endl;
//				minMaxLoc( *out[1], min, max, minL, maxL );
//				cout << min << " " << max << endl;
//				minMaxLoc( *out[2], min, max, minL, maxL );
//				cout << min << " " << max << endl;
			}

			//normalize: devide through the number of channels

			typename Channel<ouT>::Iterator2D itGxx = out[0]->getIteratorBegin();
			typename Channel<ouT>::Iterator2D itGyy = out[1]->getIteratorBegin();
			typename Channel<ouT>::Iterator2D itGxy = out[2]->getIteratorBegin();
			ouT divisor = ouT( numIn );
			for ( ; itGxx != out[0]->getIteratorEnd(); ++itGxx, ++itGyy, ++itGxy )
			{
				*itGxx /= divisor;
				*itGyy /= divisor;
				*itGxy /= divisor;
			}

//			minMaxLoc( *out[0], min, max, minL, maxL );
//			cout << min << " " << max << endl;
//			minMaxLoc( *out[1], min, max, minL, maxL );
//			cout << min << " " << max << endl;
//			minMaxLoc( *out[2], min, max, minL, maxL );
//			cout << min << " " << max << endl;

			if ( !gradientX || !gradientY )
			{
				if ( !gradientX )
				{
					DBGOUT(DBGALL, "DizenzoGradients: deleting xGrad\n");
					delete xGrad;
					xGrad = NULL;
				}
				if ( !gradientY )
				{
					DBGOUT(DBGALL, "DizenzoGradients: deleting yGrad\n");
					delete yGrad;
					yGrad = NULL;
				}
				DBGOUT(DBGALL, "DizenzoGradients: deleting tmpImg\n");
				delete tmpImg;
				tmpImg = NULL;
				DBGOUT(DBGALL, "DizenzoGradients: everything cleaned up\n");
			}

		}

	}
}

#endif
