

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGIMGCONSTANTS_H
#define ADORE_IMGPROCESSINGIMGCONSTANTS_H

#include <adrcore/util/debug.h>

namespace adore
{

	namespace core
	{

		/**
		 * Constants used by image plane to indicate how to handle the channels of the image
		 * affects interators, for example.
		 */
		enum
		{
			CHANNEL_COMBINED = -2,
            CHANNEL_SPLIT = -1
		};

		/**
		 * direction inside an image
		 * used by convolvers, for example
		 */
		enum DIRECTION
		{
			DIR_X,
			DIR_Y,
			DIR_UNDEF
		};

		/**
		 * Determin, how the border of an image should be treated
		 * 
		 */
		enum BORDER_MODE
		{
			/**
			 * expand the image with copies of the last pixel.
			 */
			BORDER_COPY,
			/**
			 * set the values outside the imge to zero
			 */
			BORDER_ZERO
		};

		template <int T>
		struct orthoDir
		{
			static const int DIR = DIR_UNDEF;
		};

		template <>
		struct orthoDir<DIR_Y>
		{
			static const int DIR = DIR_X;
		};

		template <>
		struct orthoDir<DIR_X>
		{
			static const int DIR = DIR_Y;
		};

		template <int T> inline int getOrthoDir()
		{
			error( "implement me!\n" );
		}
		template <> inline int getOrthoDir<DIR_X>()
		{
			return DIR_Y;
		}
		template <> inline int getOrthoDir<DIR_Y>()
		{
			return DIR_X;
		}

		/**
		 * connectivity of pixel grids. The values are are also used
		 * as increments in for-loop, therefore DO NOT ALTER!
		 */
		enum CONNECTIVITY
		{
			EIGHT = 1,
			FOUR = 2
		};

		/**
		 * the relative coordinates of the neighbors of a pixel
		 */
		const char CORONA[8][2] = {{1, 0}, {1, 1}, {0, 1}, {-1, 1}, {-1, 0}, {-1, -1}, {0, -1}, {1, -1}};

		/**
		 * maps sectors to corona indices
		 * The sectors are enumerated as follows
		 * 3 2 1
		 * 0 X 0
		 * 1 2 3
		 * therefore, sector 3 is represented by coronas CORONA[7] and CORONA[3]
		 */
		const char SECTOR_CORONA[4][2] = {{4, 0}, {5, 1}, {6, 2}, {7, 3}};

		template <class T> inline T imgMax()
		{
			error( "implement me!\n" );
		}
		template <> inline unsigned char  imgMax<unsigned char>()
		{
			return 255u;
		}
		template <> inline unsigned short imgMax<unsigned short>()
		{
			return 65025u;
		}
		template <> inline float  imgMax<float>()
		{
			return 1.0;
		}

	}
}

#endif

