

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGITERATORGETTER_H
#define ADORE_IMGPROCESSINGITERATORGETTER_H

#include <adrcore/imgprocessing/channel.h>

namespace adore
{

	namespace core
	{

		/**
		 * creates an Iterator for an Image depending on the direction supplied
		 * It is used internally, e.g. in the convolvers.
		 *  
		 */
		template<int dirT>
		class IteratorGetter
		{
			public:
				template <class T> Image2DIterator<T> getBegin2D( Channel<T> &chan, int xStep = 1, int yStep = 1 ) const
				{
					error( "choose either DIR_H or DIR_V" );
				}

				template <class T> Image2DIterator<T> getEnd2D( Channel<T> &chan, int xStep = 1, int yStep = 1 ) const
				{
					error( "choose either DIR_H or DIR_V" );
				}

				template <class T> Image2DIterator<T> get2D( Channel<T> &chan, int row, int col, int range, int xStep = 1, int yStep = 1 ) const
				{
					error( "choose either DIR_H or DIR_V" );
				}

				/**
				 * same as above, but swaps row and column, if DIR_Y
				 */
				template <class T> Image2DIterator<T> get2DSwap( Channel<T> &chan, int row, int col, int range, int xStep = 1, int yStep = 1 ) const
				{
					error( "choose either DIR_H or DIR_V" );
				}

				template <class T> Image1DIterator<T> getBegin1D( Channel<T> &chan, int coor, int step ) const
				{
					ADORE_UNUSED( step );
					error( "choose either DIR_H or DIR_V" );
				}

				template <class T> Image1DIterator<T> getEnd1D( Channel<T> &chan, int coor, int step ) const
				{
					error( "choose either DIR_H or DIR_V" );
				}

				template <class T> Image1DIterator<T> get1D( Channel<T> &chan, int row, int col, int step ) const
				{
					error( "choose either DIR_H or DIR_V" );
				}

				template <class T> Image1DIterator<T> get1DSwap( Channel<T> &chan, int row, int col, int step ) const
				{
					error( "choose either DIR_H or DIR_V" );
				}

				/**
				 * Dir_X = row major => returns width
				 */
				template <class T> int getMajorDim( Channel<T> &chan ) const
				{
					error( "choose either DIR_H or DIR_V" );
				}

				template <class T> int getMinorDim( Channel<T> &chan ) const
				{
					error( "choose either DIR_H or DIR_V" );
				}
		};

		template<>
		class IteratorGetter<DIR_X>
		{
			public:
				template <class T> Image2DIterator<T> getBegin2D( Channel<T> &chan, int xStep = 1, int yStep = 1 ) const
				{
					return chan.getIteratorBeginH( xStep, yStep );
				}

				template <class T> Image2DIterator<T> getEnd2D( Channel<T> &chan, int xStep = 1, int yStep = 1 ) const
				{
					return chan.getIteratorEndH( xStep, yStep );
				}

				template <class T> Image2DIterator<T> get2D( Channel<T> &chan, int row, int col, int range, int xStep = 1, int yStep = 1 ) const
				{
					return chan.getIterator( row, col, range, xStep, yStep );
				}

				template <class T> Image2DIterator<T> get2DSwap( Channel<T> &chan, int row, int col, int range, int xStep = 1, int yStep = 1 ) const
				{
					return get2D( chan, row, col, range, xStep, yStep );
				}

				template <class T> Image1DIterator<T> getBegin1D( Channel<T> &chan, int coor, int step = 1 ) const
				{
					return chan.getXIteratorBegin( step );
				}

				template <class T> Image1DIterator<T> getEnd1D( Channel<T> &chan, int coor, int step = 1 ) const
				{
					return chan.getXIteratorEnd( step );
				}

				template <class T> Image1DIterator<T> get1D( Channel<T> &chan, int row, int col, int step = 1 ) const
				{
					return chan.getXIterator( row, col, step );
				}

				template <class T> Image1DIterator<T> get1DSwap( Channel<T> &chan, int row, int col, int step = 1 ) const
				{
					return get1D( chan, row, col, step );
				}

				template <class T> int getMajorDim( Channel<T> &chan ) const
				{
// 					printf("DIR_X: returning major dim: %d\n", chan.getROI().getWidth());
					return chan.getROI().getWidth();
				}

				template <class T> int getMinorDim( Channel<T> &chan ) const
				{
// 					printf("DIR_X: returning major dim: %d\n", chan.getROI().getHeight());
					return chan.getROI().getHeight();
				}
		};

		template<>
		class IteratorGetter<DIR_Y>
		{
			public:
				template <class T> Image2DIterator<T> getBegin2D( Channel<T> &chan, int xStep = 1, int yStep = 1 ) const
				{
					return chan.getIteratorBeginV( xStep, yStep );
				}

				template <class T> Image2DIterator<T> getEnd2D( Channel<T> &chan, int xStep = 1, int yStep = 1 ) const
				{
					return chan.getIteratorEndV( xStep, yStep );
				}

				template <class T> Image2DIterator<T> get2D( Channel<T> &chan, int row, int col, int range, int xStep = 1, int yStep = 1 ) const
				{
					return chan.getIteratorV( row, col, range, xStep, yStep );
				}

				template <class T> Image2DIterator<T> get2DSwap( Channel<T> &chan, int row, int col, int range, int xStep = 1, int yStep = 1 ) const
				{
					return get2D( chan, col, row, range, xStep, yStep );
				}

				template <class T> Image1DIterator<T> getBegin1D( Channel<T> &chan, int coor, int step = 1 ) const
				{
					return chan.getYIteratorBegin( step );
				}

				template <class T> Image1DIterator<T> getEnd1D( Channel<T> &chan, int coor, int step = 1 ) const
				{
					return chan.getYIteratorEnd( step );
				}

				template <class T> Image1DIterator<T> get1D( Channel<T> &chan, int row, int col, int step = 1 ) const
				{
					return chan.getYIterator( row, col, step );
				}

				template <class T> Image1DIterator<T> get1DSwap( Channel<T> &chan, int row, int col, int step = 1 ) const
				{
					return get1D( chan, col, row, step );
				}

				template <class T> int getMajorDim( Channel<T> &chan ) const
				{
// 					printf("DIR_Y: returning major dim: %d\n", chan.getROI().getHeight());
					return chan.getROI().getHeight();
				}

				template <class T> int getMinorDim( Channel<T> &chan ) const
				{
// 					printf("DIR_Y: returning minor dim: %d\n", chan.getROI().getWidth());
					return chan.getROI().getWidth();
				}
		};

	}

}

#endif
