

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREKERNELANISO_3_3_H
#define ADORE_COREKERNELANISO_3_3_H

#include <adrcore/util/calctype.h>
#include <adrcore/util/templatelimits.h>
#include <adrcore/imgprocessing/kerneloperatorbase.h>

namespace adore
{

	namespace core
	{

		/**
			 
			replaces the center pixel with the value of the closest average of
			three adjacent neigbour pixel.
			This resamples a simple anisotropic filter
		*/
		template<class inT>
		class KernelAniso_3_3 : public KernelOperatorBase
		{
			private:
				typedef typename signedT<typename calcT<inT>::type>::type cT;
			public:
				typedef inT img_in_type;
				typedef inT img_out_type;

			public:
				KernelAniso_3_3()
				{
					setTopHood( 1 );
					setLeftHood( 1 );
					setBottomHood( 1 );
					setRightHood( 1 );
				}

				/**
					takes an iterator of lineIterators
				*/
				template<class IteratorT>
				img_out_type convolve( IteratorT it )
				{
					typedef typename IteratorT::value_type lineIterator;

					inT corona[10];//two overlap

					//top 3 values
					lineIterator lit = *it;
					for ( int i = 0; i < 3; ++i, ++lit )
					{
						corona[i] = *lit;
					}
					//left and right
					++it; lit    = *it;
					corona[7]    = *lit; ++lit;
					cT center = *lit * 3; ++lit;//3 is important, because we take the "average" of 3 values without division (i.e. sum only)
					corona[3]    = *lit;
					//bottom 3 values
					++it; lit = *it;
					for ( int i = 4; i < 7; ++i, ++lit )
					{
						corona[i] = *lit;
					}
					//overlap
					corona[8] = corona[0];
					corona[9] = corona[1];

					cT bestVal = 0;
					cT bestDiff = maxLimit<cT>();
					for ( int i = 0; i < 8; ++i )
					{
						cT avg = corona[i] + corona[i + 1] + corona[i + 2];
						cT candDiff = tabs( avg - center );
						if ( candDiff < bestDiff )
						{
							bestVal = avg;
							bestDiff = candDiff;
						}
					}

					return inT( float( bestVal ) / 3.0F );
				}

		};

	}

}

#endif
