

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGKERNELEXTREMUM_3_3_H
#define ADORE_IMGPROCESSINGKERNELEXTREMUM_3_3_H

#include <adrcore/imgprocessing/kerneloperatorbase.h>
#include <algorithm>
#include <functional>
#include <adrcore/types/commontypes.h>

namespace adore
{

	namespace core
	{

		/**
		     * returns comp(a, b) ? a : b;
		     * This can be used to model a dilate operator
			  
		*/
		template <class inT, class ComparatorT = std::less<inT> >
		class KernelExtremum_3_3 : public KernelOperatorBase
		{
				typedef inT calcT;
			public:
				typedef inT img_in_type;
				typedef inT img_out_type;

			public:
				KernelExtremum_3_3()
				{
					setTopHood( 1 );
					setLeftHood( 1 );
					setBottomHood( 1 );
					setRightHood( 1 );
				}

				~KernelExtremum_3_3()
				{
					;
				}

				/**
				 * Convolves.
				 * The iterator has to iterate through iterators that iterate lines :)
				 * That means:
				 * LineIterator = *IteratorT
				 * value = *LineIterator
				 */
				template<class IteratorT>
				img_out_type convolve( IteratorT it )
				{
					typedef typename IteratorT::value_type lineIterator;
					lineIterator lit1 = *it++;
					lineIterator lit2 = *it++;
					lineIterator lit3 = *it++;

					calcT a = *lit1; ++lit1;
					calcT b = *lit1; ++lit1;
					calcT c = *lit1; ++lit1;
					calcT d = *lit2; ++lit2;
					calcT e = *lit2; ++lit2;
					calcT f = *lit2; ++lit2;
					calcT g = *lit3; ++lit3;
					calcT h = *lit3; ++lit3;
					calcT i = *lit3; ++lit3;

					return std::min<inT, ComparatorT>( a,
					                                   std::min<inT, ComparatorT>( b,
					                                                               std::min<inT, ComparatorT>( c,
					                                                                                           std::min<inT, ComparatorT>( d,
					                                                                                                                       std::min<inT, ComparatorT>( e,
					                                                                                                                                                   std::min<inT, ComparatorT>( f,
					                                                                                                                                                                               std::min<inT, ComparatorT>( g,
					                                                                                                                                                                                                           std::min<inT, ComparatorT>( h, i, comp ), comp ), comp ), comp ), comp ), comp ), comp ), comp );
				}

			private:
				ComparatorT comp;
		};

	}
}

#endif
