

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGPOLARRESAMPLER_H
#define ADORE_IMGPROCESSINGPOLARRESAMPLER_H

#include <adrcore/imgprocessing/channelprocessor.h>
#include <adrcore/imgprocessing/bilinearinterpolator.h>

namespace adore
{

	namespace core
	{

		/**
		 
		*/
		template <class img_in_typeT, class img_out_typeT = img_in_typeT>
		class PolarResampler : public ChannelProcessor<1, 1, img_in_typeT, img_in_typeT>
		{
			public:
				typedef img_in_typeT img_in_type;
				typedef img_out_typeT img_out_type;

			public:
				PolarResampler(): ChannelProcessor<1, 1, img_in_type, img_in_type>()
				{
				}

				void process( Channel<img_in_type> *in, Channel<img_out_type> *out )
				{
					assert( in != out );

					UsingROI r1( in );
					in->minROIDistance( 1 );//for bilinearinterpolator

					float rw = 0.5F * min( in->getROI().getWidth(), in->getROI().getHeight() );
					float rx = 0.5F * in->getROI().getWidth() + in->getROI().getX();
					float ry = 0.5F * in->getROI().getHeight() + in->getROI().getY();
					uint iphi, irad;

					BilinearInterpolator<img_in_type> inter;

					typename Channel<img_out_type>::Iterator2D it = out->getIteratorBegin();
					typename Channel<img_out_type>::Iterator2D stop = out->getIteratorEnd();
					for ( ; it != stop; ++it )
					{
						out->getCoors( it.getPointer(), iphi, irad );
						float phi = 2.0F * M_PI * float( iphi ) / float( out->getROI().getWidth() );
						float rad = float( irad ) / float( out->getROI().getHeight() );

						float x = rx + cosf( phi ) * rad * rw;
						float y = ry + sinf( phi ) * rad * rw;
//						printf( "coors: %f, %f, %f, %f\n", phi, rad, x, y );
						inter.init( x, y );
						*it = inter.valueOf( *in );
					}

				}

		};

	}

}

#endif
