

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORESCALEHELPERS_H
#define ADORE_CORESCALEHELPERS_H

#include <adrcore/types/featurepoint.h>
#include <adrcore/types/view.h>


namespace adore
{
	namespace core
	{

		/**
			assuming, you are on the scale that belongs to
			the given spacing, the pixel position on scale 1
			is returned.
			This is only true, if the ScaleSpacer uses SubsamplerSmooth!
		 */
		inline float getPixelPosition( float coor, int spacing )
		{
			return coor * float( spacing ) + 0.5 * float( spacing - 1 );
		}


		template<class imgT>
		class ScaleMatchHelper
		{
			public:
				typedef ImagePlane<imgT> ImageT;
				typedef View<imgT> ViewT;

			public:
				ScaleMatchHelper( FeaturePoint *fp1, FeaturePoint *fp2 ):
						minScale( fp1->getScale() ),
						maxScale( fp2->getScale() ),
						minFp( fp1 ),
						maxFp( fp2 ),
						minImg( static_cast<ViewT *>( fp1->getView() )->getImage() ),
						maxImg( static_cast<ViewT *>( fp2->getView() )->getImage() ),
						scaleRatio( 1.0 )
				{
					if ( minScale > maxScale )
					{
						std::swap( minScale, maxScale );
						std::swap( minFp, maxFp );
						std::swap( minImg, maxImg );
					}
					scaleRatio = maxScale / minScale;
				}

				virtual ~ScaleMatchHelper() {}

			public:

				float minScale;
				float maxScale;
				FeaturePoint *minFp;
				FeaturePoint *maxFp;
				ImageT *minImg;
				ImageT *maxImg;
				float scaleRatio; // max/min
		};

		template<class imgT>
		class ScalePatchHelper : public ScaleMatchHelper<imgT>
		{
			public:
				typedef ImagePlane<imgT> ImageT;
				typedef View<imgT> ViewT;

			public:
				ScalePatchHelper( FeaturePoint *fp1, FeaturePoint *fp2, int _hood ):
						ScaleMatchHelper<imgT>( fp1, fp2 ),
						hood( _hood ),
						size( hood + hood + 1 ),
						maxPatch( size, size, this->maxImg->getNumChans() ),
						minPatch( size, size, this->minImg->getNumChans() )
				{
				}

				void fillPatches()
				{
					assert( this->maxImg->getNumChans() == this->minImg->getNumChans() );

					for ( uint i = 0; i < this->maxImg->getNumChans(); ++i )
					{
						this->minImg->setChannelMode( i );
						this->minPatch.setChannelMode( i );
						AArray<float, 2> invScaler( 2, 2 );
						invScaler = 1.0f/this->scaleRatio, 0.0F, 0.0F, 1.0f/this->scaleRatio;
						warpAffine( *this->minImg, ( *this->minFp )( 0 ), ( *this->minFp )( 1 ),
						            invScaler,
						            minPatch.getIteratorBegin(), hood, hood );

						this->maxImg->setChannelMode( i );
						maxPatch.setChannelMode( i );
						getPatch( *this->maxImg, ( *this->maxFp )( 0 ), ( *this->maxFp )( 1 ),
						          maxPatch.getIteratorBegin(), hood, hood );
					}
				}

			public:
				int hood, size;
				ImageT maxPatch, minPatch;
		};

	}
}

#endif
