

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGSOBELOPERATOR2PASS_H
#define ADORE_IMGPROCESSINGSOBELOPERATOR2PASS_H

#include <adrcore/imgprocessing/channelprocessor.h>
#include <adrcore/imgprocessing/imageplane.h>
#include <adrcore/imgprocessing/convolver1d3.h>
#include <adrcore/imgprocessing/convolver1d3notch.h>
#include <adrcore/imgprocessing/kernelgauss_3_1.h>
#include <adrcore/imgprocessing/kernelprewitt_3_1.h>
#include <adrcore/util/calctype.h>


namespace adore
{

	namespace core
	{

		/**
			 
		*/

		template<class typeIn>
		class SobelOperator2PassBase
		{
			public:

				SobelOperator2PassBase( Channel<typeIn> * tmp = NULL ):
						tmpPlane( tmp )
				{
					;
				}

				virtual ~SobelOperator2PassBase()
				{
					;
				}

				void setTmpPlane( Channel<typeIn> * tmp )
				{
					tmpPlane = tmp;
				}

				Channel<typeIn> *getTmpPlane() const
				{
					return tmpPlane;
				}

			private:

				Channel<typeIn> *tmpPlane;

		};

		template<class typeIn, int dirT>
		class SobelOperator2Pass : public ChannelProcessor< 1, 1, typeIn, typename signedT< typename imgCalcT<typeIn>::type >::type >, public SobelOperator2PassBase<typeIn>
		{
			public:

				typedef typename KernelPrewitt_3_1<typeIn>::img_out_type img_out_type;
				typedef typeIn img_in_type;

				SobelOperator2Pass( Channel<typeIn> * tmp = NULL ):
						SobelOperator2PassBase<typeIn>( tmp )
				{
					;
				}


				void process( Channel<typeIn> *in, Channel <img_out_type> *out )
				{
					Channel<typeIn> *tmpPlane = this->getTmpPlane();
					if ( !tmpPlane )
					{
//						printf( "sobelop2pass vreates temporary plane\n" );
						tmpPlane = new ImagePlane<typeIn>( in->getWidth(), in->getHeight(), 1 );
					}

					gauss.process( in, tmpPlane );
					prewitt.process( tmpPlane, out );

					if ( !this->getTmpPlane() )
					{
						delete tmpPlane; tmpPlane = NULL;
					}
				}

			private:

				Convolver1D3<KernelGauss_3_1<typeIn> , dirT > gauss;//in Y-Richtung Weichzeichnen
				Convolver1D3Notch<KernelPrewitt_3_1<typeIn>, orthoDir<dirT>::DIR > prewitt;//in X-Richtung Ableiten
		};

	}

}

#endif
