

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "filterransac3viewhelper.h"
#include "errorfunctors3views.h"

#define DBGFILTER DBGALL
namespace adore
{

	namespace core
	{

		template<class Solution, template<typename> class ErrorFn>
		bool FilterRANSAC3ViewHelper<Solution, ErrorFn>::
		updateSolution ( Solution &sol, const vector<int> &base, float thresh, bool opt )
		{
			double err = computeErrors ( sol, thresh, candInliers, corrs, client->getNumCandidates(),
			                             cam, optimize3D );
			DBGErr = err;

			int in = ( int ) candInliers.size();
			bool okay = err < client->getError();
			if ( okay )
			{
				DBGOUT ( DBGFILTER, "good enough (in: %d/%d, err: %.10f/%.10f)\n", in, client->getNumIn(), err, client->getError() );
				bestSolution = sol;
				client->setNumIn ( in );
				client->setError ( err );
				inliers.resize ( candInliers.size() );
				copy ( candInliers.begin(), candInliers.end(), inliers.begin() );
				return true;
			}
			else
			{
				DBGOUT ( DBGFILTER, "NOT good enough (in: %d/%d, err: %.10f/%.10f)\n", in, client->getNumIn(), err, client->getError() );
			}
			return false;
		}

		//instantiation
		template class FilterRANSAC3ViewHelper<TTensorSolution, TTWithCorrector>;
		template class FilterRANSAC3ViewHelper<TTensorSolution, TTSampson>;

	}//ns
}
