

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREFILTERRANSAC3VIEWHELPER_H
#define ADORE_COREFILTERRANSAC3VIEWHELPER_H

#include <adrcore/types/pointcorr.h>
#include <adrcore/types/featurepoint.h>
#include <adrcore/util/mathutil.h>
#include <adrcore/util/debug.h>
#include <adrcore/util/c++0x.h>
#include <adrcore/random/nviransacclient.h>
#include <adrcore/util/divisiondistortion.h>
#include <adrcore/nview/ttensorcalculatorppp.h>
#include "filterransaccommon.h"

namespace adore
{

	namespace core
	{
		class TTensorSolution
		{
			private:
				ATensor3 tten;
			public:
				TTensorSolution() :tten ( 3, 3, 3 ) {}
				TTensorSolution( const ATensor3& f ) : tten ( f ) {}
				inline const ATensor3 & getTTensor() const { return tten; }
				inline AVector getCorrectedPoint ( AVector &vec ) const { return vec; }
				inline AVector getUnCorrectedPoint ( AVector &vec ) const { return vec; }
				NViewInfo *createNViewInfo()
				{
					NViewInfo *info = new NViewInfoTT;
					info->setTTensor ( tten );
					return info;
				}

		};

		template<> class SolutionCalculator<TTensorSolution>
		{
			private:
				TTensorCalculatorPPP tcalc;

			public:
				SolutionCalculator()
				{
//					tcalc.setBundleAdjust ( false );
					tcalc.setMinAlgebraic ( true );
				}

				template<class CorrIterator, class ViewIterator>
				int calcSloppy ( CorrIterator cit, int num, ViewIterator vit )
				{
					return calc ( cit, num, vit );
				}

				template<class CorrIterator, class ViewIterator>
				int calc ( CorrIterator cit, int num, ViewIterator vit )
				{
					return tcalc.calculateTrifocalTensor ( cit, num, vit );
				}

				TTensorSolution getSolutionSloppy ( int i )
				{
					return tcalc.getSolution ( i );
				}

				TTensorSolution getSolution ( int i )
				{
					return tcalc.getSolution ( i );
				}
		};

		template<> inline int getOuterSampleSize<TTensorSolution>() {return 6;}

		template<> inline int getInnerSampleSize<TTensorSolution>() {return 12;}

		/**
			 
		*/
		template<class SolutionT, template<typename> class ErrorFn>
		class FilterRANSAC3ViewHelper
		{
			public:
				typedef SolutionT Solution;
				typedef SolutionCalculator<Solution> Calculator;

				typedef NVIRANSACClient::InlierCollT InlierCollT;
				typedef NVIRANSACClient::iterator iterator;
				typedef tuple<PointCorr*, FeaturePoint *, FeaturePoint *, FeaturePoint *> CorrT;
				typedef vector< CorrT > CorrsT;
				typedef CorrsT::iterator corr_iterator;

				class CorrTComp
				{
					public:
						bool operator()( const CorrT &c1, const CorrT &c2 )
						{
							return ( c1.get<0>()->getBadness() < c2.get<0>()->getBadness() );
						}
				};

			private:
				RANSACClient *client;
				vector<Camera *> cam;
				vector<AVector *>points;
				Camera cam1;
				Camera cam2;
				Camera cam3;

				CorrsT corrs;

			public:
				const int numSamples;
				InlierCollT candInliers;
				InlierCollT inliers;
				Solution bestSolution;
				const bool optimize3D;

				double DBGErr;

			public:
				FilterRANSAC3ViewHelper ( RANSACClient *c, int num ) :client ( c ), cam( 3 ), points( 3 ), numSamples ( num ), optimize3D( true )
				{
					cam[0] = &cam1;
					cam[1] = &cam2;
					cam[2] = &cam3;
					cam1.getProjMat().setIdentity();
				}

				template<class CorrIterator, class ViewIterator>
				void init ( CorrIterator it, int size, ViewIterator vit )
				{
					corrs.clear();
					corrs.reserve ( size );
					inliers.clear();
					inliers.reserve ( size );
					candInliers.clear();
					candInliers.reserve ( size );
					ViewBase *v1 = *vit; ++vit;
					ViewBase *v2 = *vit; ++vit;
					ViewBase *v3 = *vit;
					for ( int i = 0; i < size; ++i, ++it )
					{
						PointCorr *corr = *it;
						if ( !corr ) continue;
						FeaturePoint *fp1 = corr->getFeature ( v1 );
						if ( !fp1 ) continue;
						FeaturePoint *fp2 = corr->getFeature ( v2 );
						if ( !fp2 ) continue;
						FeaturePoint *fp3 = corr->getFeature ( v3 );
						if ( !fp3 ) continue;
						corrs.push_back ( make_tuple ( corr, fp1, fp2, fp3 ) );
					}
					client->setOkay ( false );
					if ( numSamples <= ( int ) corrs.size() ) client->setOkay ( true );
					client->setNumCandidates ( corrs.size() );
					client->setNumIn ( -1 );
					client->setError ( FLT_MAX );

					CorrTComp comp;
					sort( corrs.begin(), corrs.end(), comp );
				}

				PointCorr *getSamplePointCorr ( int i ) {return getPointCorr ( i );}
				PointCorr *getPointCorr ( int i ) {return corrs[i].get<0>();}

				bool updateSolution ( Solution &sol, const vector<int> &base, float thresh, bool opt );

				iterator begin() {return inliers.begin();}

				iterator end() {return inliers.end();}

				MODEL_TYPE getModelType() const  { assert( false ); return MODEL_TYPE_H; }

			private:
				//test all of the indexed points and compute a score (majority vote)
				int isOriented ( Solution &sol, AVector &epiPoint, const vector<int> &in );

				ErrorFn<SolutionT> computeErrors;

		};

	}

}

#endif
