

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "filterransaccp.h"

#include <cassert>

namespace adore {
	namespace core {


        template<class ErrorFn>
        bool RANSACCPHelperPointsAndCams<ErrorFn>::calcSolution(const vector<int> &sample, AMatrix& H)
        {
            assert ( ( int ) sample.size() == sampleSize );

            vector< AVector* > pts1( sample.size() );
            vector< AVector* > pts2( sample.size() );

            //copy sample
            for ( uint i = 0; i < sample.size(); ++i )
            {
                pts1[i] = points3d[sample[i]].first;
                pts2[i] = points3d[sample[i]].second;
            }

            return calc.calculateAssumingMetricUseAll( cam1, cam2, pts1, pts2, H );
        }

        template<class ErrorFn>
        bool RANSACCPHelperPointsAndCams<ErrorFn>::updateSolution(const AMatrix &H, double thresh)
        {
            vector<int> candidates; candidates.reserve( points3d.size() );
            double error = errfn(H, thresh, candidates, points3d);

            int in = ( int ) candidates.size();
            bool okay = error < client->getError();

            if ( okay )
            {
                DBGOUT ( DBGNORMAL, "good enough (in: %d/%d, err: %.10f/%.10f)\n", in, client->getNumIn(), error, client->getError() );
                Hsolution = H;
                client->setNumIn ( in );
                client->setError ( error );
                inliers.resize ( candidates.size() );
                copy ( candidates.begin(), candidates.end(), inliers.begin() );
                return true;
            }
            else
            {
                DBGOUT ( DBGNORMAL, "NOT good enough (in: %d/%d, err: %.10f/%.10f)\n", in, client->getNumIn(), error, client->getError() );
            }

            return false;
        }

        template<class Helper>
        bool FilterRANSACCP<Helper>::applySample( const vector<int> & sample )
        {
                AMatrix H(4, 4);
                bool have_solution = m_helper.calcSolution(sample, H);
                if(!have_solution) return false;

                return m_helper.updateSolution(H, m_fractionThresh);
        }

        template class FilterRANSACCP< RANSACCPHelperPointsAndCams<CPErrorsSimpleDist> >;

    } // core
} // adore
