

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "baprojective.h"
#include <adrcore/util/aarrayutil.h>
#include <adrcore/parameterization/simplecamparam.h>
#include <adrcore/parameterization/zfixedpointparam.h>
#include <adrcore/minimization/sparsedefaulterrorfunc.h>

namespace adore
{

	namespace core
	{

		void BAProjective::modify ( NView &nview )
		{
			vector<Camera *> cams;
			vector<AVector3D *> point3D;
			vector<AVector> goalPts2D;
			vector<AVector2D *> pts;
			nview.getAll<AVector2D> ( cams, point3D, pts );
			reallocPts ( goalPts2D, pts.size(), 2 );
			for ( uint i = 0; i < pts.size(); ++i )
			{
				AVector2D *pt = pts[i];
				if ( pt )
				{
					double z = ( *pt ) ( 2 );
					goalPts2D[i] ( 0 ) = ( *pt ) ( 0 ) / z;
					goalPts2D[i] ( 1 ) = ( *pt ) ( 1 ) / z;
				}
				else
				{
					setNullArray( goalPts2D[i] );
				}
			}

			SimpleCamParam cP;
			ZFixedPointParam pP;
			const double huberVal = HUBER_VALUE;//TODO choose appropriately for view
			SparseHuberVectorErrorFunc errFunc ( goalPts2D, huberVal );
			BAMinFunction<SimpleCamParam, ZFixedPointParam> minFunc ( cP, pP );

			setLastCamIsIdentity ( false );
			BABase::modify ( cams, point3D, minFunc, errFunc, cP, pP, thresh );
		}


		template <typename ErrorFuncT, class MinFunctionT>
		void BAProjectiveSimpleT<ErrorFuncT, MinFunctionT>::modify ( NView &nview )
		{
			vector<Camera *> cams;
			vector<AVector3D *> point3D;
			vector<AVector> goalPts2D;
			vector<AVector2D *> pts;
			nview.getAll<AVector2D> ( cams, point3D, pts );
			reallocPts ( goalPts2D, pts.size(), 2 );
			missing.reserve( int( 0.2 * pts.size() ) );
			for ( uint i = 0; i < pts.size(); ++i )
			{
				AVector2D *pt = pts[i];
				if ( pt )
				{
					double z = ( *pt ) ( 2 );
					goalPts2D[i] ( 0 ) = ( *pt ) ( 0 ) / z;
					goalPts2D[i] ( 1 ) = ( *pt ) ( 1 ) / z;
				}
				else
				{
					setNullArray( goalPts2D[i] );
					missing.push_back( i );
				}
			}


			const double huberVal = HUBER_VALUE;//TODO choose appropriately for view
			ErrorFuncT errFunc ( goalPts2D, huberVal );
			BundleAdjusterSimple<ErrorFuncT, MinFunctionT>::modify ( cams, point3D, errFunc, thresh );
		}

		template class BAProjectiveSimpleT<SparseHuberVectorErrorFunc>;
		template class BAProjectiveSimpleT<SparseHuberVectorErrorFunc, BAMinFunctionSimple_RTF>;
		template class BAProjectiveSimpleT<SparseHuberVectorErrorFunc, BAMinFunctionSimple_Quat_Full>;
		template class BAProjectiveSimpleT<SparseHuberVectorErrorFunc, BAMinFunctionSimple_Quat_F>;
		template class BAProjectiveSimpleT<SparseBlakeZissVectorErrorFunc>;
		template class BAProjectiveSimpleT<SparseBlakeZissVectorErrorFunc, BAMinFunctionSimple_RTF>;

	}

}
