

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_MINIMIZATIONBAPROJECTIVE_H
#define ADORE_MINIMIZATIONBAPROJECTIVE_H

#include <adrcore/nview/nviewoperator.h>
#include <adrcore/minimization/bundleadjuster.h>
#include <adrcore/minimization/baminfunction.h>
#include <adrcore/minimization/bundleadjustersimple.h>

namespace adore
{

	namespace core
	{

		typedef BundleAdjuster<BAMinFunction<SimpleCamParam, ZFixedPointParam>,
		SparseHuberVectorErrorFunc, SimpleCamParam, ZFixedPointParam> BABase;
		/**
			 
		*/
		class BAProjective: public NViewOperator, public BABase
		{
			private:
				double thresh;
			public:
				BAProjective ( double t = FLT_EPSILON ) :
					BABase(10000),
					thresh ( t )
				{
					;
				}
				void modify ( NView &nview );

				void setThresh ( double theValue )
				{
					thresh = theValue;
				}


				double getThresh() const
				{
					return thresh;
				}

		};

		/**
		 
		 */
		template <typename ErrorFuncT, class MinFunctionT = BAMinFunctionSimple>
		class BAProjectiveSimpleT: public NViewOperator, public BundleAdjusterSimple<ErrorFuncT, MinFunctionT>
		{
			private:
				double thresh;
				using BundleAdjusterSimple<ErrorFuncT, MinFunctionT>::missing;

			public:
				BAProjectiveSimpleT ( double t = FLT_EPSILON ) :thresh ( t )
				{
					;
				}

				BAProjectiveSimpleT( const BAProjectiveSimpleT &ba ) : BundleAdjusterSimple<ErrorFuncT, MinFunctionT>( ba ), thresh( ba.thresh )
				{
					;
				}

				NViewOperator *clone()
				{
					return new BAProjectiveSimpleT( *this );
				}

				void modify ( NView &nview );

				void setThresh ( double theValue )
				{
					thresh = theValue;
				}


				double getThresh() const
				{
					return thresh;
				}

		};

		typedef BAProjectiveSimpleT<SparseHuberVectorErrorFunc> BAProjectiveSimple;
		typedef BAProjectiveSimpleT<SparseHuberVectorErrorFunc, BAMinFunctionSimple_RTF> BAProjectiveSimpleRTF;
		typedef BAProjectiveSimpleT<SparseHuberVectorErrorFunc, BAMinFunctionSimple_Quat_Full> BAProjectiveSimpleQuatFull;
		typedef BAProjectiveSimpleT<SparseHuberVectorErrorFunc, BAMinFunctionSimple_Quat_F> BAProjectiveSimpleQuatF;
		typedef BAProjectiveSimpleT<SparseBlakeZissVectorErrorFunc> BAProjectiveSimpleBlakeZiss;
		typedef BAProjectiveSimpleT<SparseBlakeZissVectorErrorFunc, BAMinFunctionSimple_RTF> BAProjectiveSimpleBlakeZissRTF;

	}

}

#endif
