

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "cameraminjacobi.h"
#include <adrcore/util/matmul2.h>
#include <adrcore/util/printmat.h>

namespace adore
{
	namespace core
	{

		CameraMinJacobi::CameraMinJacobi():
				pts3D( NULL )
		{
		}


		CameraMinJacobi::~CameraMinJacobi()
		{
		}

		void CameraMinJacobi::operator()( const AVector &camera, AMatrix& diff ) const
		{
			assert( pts3D );
//			AMatrix diff( 2 * pts3D->size(), 12 );

			int idx = 0, idxN = 1;
			for ( uint i = 0; i < pts3D->size(); ++i )
			{
				AVector &pt3D = *( *pts3D )[i];

				double denom = camera( 8 ) * pt3D( 0 ) + camera( 9 ) * pt3D( 1 ) + camera( 10 ) * pt3D( 2 ) + camera( 11 ) * pt3D( 3 );
				double denomSqr =  denom * denom;
				double numer1 = camera( 0 ) * pt3D( 0 ) + camera( 1 ) * pt3D( 1 ) + camera( 2 ) * pt3D( 2 ) + camera( 3 ) * pt3D( 3 );
				double numer2 = camera( 4 ) * pt3D( 0 ) + camera( 5 ) * pt3D( 1 ) + camera( 6 ) * pt3D( 2 ) + camera( 7 ) * pt3D( 3 );
				double factor1 = numer1 / denomSqr;
				double factor2 = numer2 / denomSqr;

				diff( idx, 0 ) = diff( idxN, 4 ) = pt3D( 0 ) / denomSqr;
				diff( idx, 1 ) = diff( idxN, 5 ) = pt3D( 1 ) / denomSqr;
				diff( idx, 2 ) = diff( idxN, 6 ) = pt3D( 2 ) / denomSqr;
				diff( idx, 3 ) = diff( idxN, 7 ) = pt3D( 3 ) / denomSqr;
				diff( idx, 4 ) = diff( idxN, 0 ) = 0.0;
				diff( idx, 5 ) = diff( idxN, 1 ) = 0.0;
				diff( idx, 6 ) = diff( idxN, 2 ) = 0.0;
				diff( idx, 7 ) = diff( idxN, 3 ) = 0.0;
				diff( idx, 8 ) = -pt3D( 0 ) * factor1;
				diff( idx, 9 ) = -pt3D( 1 ) * factor1;
				diff( idx,10 ) = -pt3D( 2 ) * factor1;
				diff( idx,11 ) = -pt3D( 3 ) * factor1;
				diff( idxN, 8 ) = -pt3D( 0 ) * factor2;
				diff( idxN, 9 ) = -pt3D( 1 ) * factor2;
				diff( idxN,10 ) = -pt3D( 2 ) * factor2;
				diff( idxN,11 ) = -pt3D( 3 ) * factor2;
				idx += 2;
				idxN += 2;
			}

//			return diff;
		}


	}
}
