

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "lineartriangulator.h"
#include <adrcore/util/printmat.h>

namespace adore
{

	namespace core
	{

		LinearTriangulator::LinearTriangulator() :numCams ( 0 ), A(),store4 ( 0, 4, SVD_VT )
		{
		}

		LinearTriangulator::~LinearTriangulator()
		{
		}

		bool LinearTriangulator::operator() ( const vector <AMatrix *> &cams,
		                                      const vector < AVector * > &points,
		                                      AVector &result )
		{
			assert_msg ( cams.size() == points.size(), "You need a Camera for each point and vice versa\n" );
			if ( cams.size() < 1 ) return false;
			if ( numCams != cams.size() )
			{
				numCams = cams.size();
				A.resize ( 2 * numCams, 4 );
			}

//			printf( "cams:" );
//			for ( uint i = 0 ; i < cams.size(); ++i )
//			{
//				printf( "cam %d:\n", i );
//				util::whereUnInited( cams[i]->getProjMat() );
//			}
//			pressKey();
//			printf( "points:" );
//			util::whereUnInited( points );
//			pressKey();
			fillA ( cams, points );
//			printMat(A, 25); cout << endl;
//			printf( "A:" );
//			util::whereUnInited( A );
//			pressKey();

			store4.changeRowsWorkspaceOnly( A.rows() );
			store4.repairD();
			svdVT ( A, store4 );

			if ( store4.D ( 2 ) < FLT_EPSILON )
			{
				DBGOUT ( DBGNORMAL, "Triangulation failed. Information not sufficient or point close to epipole\n" );
				return false;
			}

			result = store4.VT ( 3, Range::all() );

			//normalizeExponents(result);//OPT

			return true;
		}

		void LinearTriangulator::fillA ( const vector <AMatrix *> &cams, const vector < AVector* > &points )
		{
			vector <AMatrix *>::const_iterator ci = cams.begin();
			vector < AVector * >::const_iterator pi = points.begin();
			for ( uint i = 0; i < cams.size(); ++i, ++ci, ++pi )
			{
				assert( *ci );
				assert( *pi );

				int row = 2 * i;
				AMatrix &cam = **ci;

				A ( row, 0 ) = ( **pi ) ( 0 ) * cam ( 2, 0 ) - cam ( 0, 0 );
				A ( row, 1 ) = ( **pi ) ( 0 ) * cam ( 2, 1 ) - cam ( 0, 1 );
				A ( row, 2 ) = ( **pi ) ( 0 ) * cam ( 2, 2 ) - cam ( 0, 2 );
				A ( row, 3 ) = ( **pi ) ( 0 ) * cam ( 2, 3 ) - cam ( 0, 3 );

				++row;
				A ( row, 0 ) = ( **pi ) ( 1 ) * cam ( 2, 0 ) - cam ( 1, 0 );
				A ( row, 1 ) = ( **pi ) ( 1 ) * cam ( 2, 1 ) - cam ( 1, 1 );
				A ( row, 2 ) = ( **pi ) ( 1 ) * cam ( 2, 2 ) - cam ( 1, 2 );
				A ( row, 3 ) = ( **pi ) ( 1 ) * cam ( 2, 3 ) - cam ( 1, 3 );
			}
		}

	}
}
