

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWNORMALIZER_H
#define ADORE_NVIEWNORMALIZER_H

#include <vector>
#include <adrcore/util/mathutil.h>
#include <adrcore/util/unptr.h>
#include <adrcore/types/view.h>
#include <adrcore/types/nviewmap.h>
#include <adrcore/nview/nviewoperator.h>

namespace adore
{
	namespace core
	{

		using namespace std;

		/**
		 * Some useful routines for pre-nomalization of data.
		 * E.g. usually you want to normalize your data before stuffing it into a DLT-Algo.
		 *
		 * Often, two versions of the same function are availiable. The suffix may be "Ptr" or "Value", indicating
		 * an Iterator to pointer data or to value data, respectivly.
		 *
		 *  
		*/
		class Normalizer:public NViewOperator
		{
			public:
				/**
				 * Constructor.
				 */
				Normalizer( bool normalize = true ):norm( normalize )
				{
					;
				}

				/**
				 * Destructor.
				 */
				~Normalizer()
				{
					;
				}

				void normalize( NViewMap &map ) const
				{
					NViewMap::NViewIterator it = map.getEntriesBegin();
					for ( ; it != map.getEntriesEnd(); ++it )
					{
						DBGOUT( DBGALL, "Normalizer normalizes NView %p\n", *it );
						( *it )->normalize();
					}
				}

				void denormalize( NViewMap &map ) const
				{
					NViewMap::NViewIterator it = map.getEntriesBegin();
					for ( ; it != map.getEntriesEnd(); ++it )
					{
						DBGOUT( DBGALL, "Normalizer denormalizes NView %p\n", *it );
						( *it )->denormalize();
					}
				}


				void normalize( NView &nview ) const
				{
					nview.normalize();
				}

				void denormalize( NView &nview ) const
				{
					nview.denormalize();
				}

				void modify( NView &nview )
				{
					if ( norm ) normalize( nview );
					else denormalize( nview );
				}

				bool getNormalize() const
				{
					return norm;
				}

				void setNormalize( const bool n )
				{
					norm = n;
				}

			private:

				bool norm;

		};

	}

}

#endif
