

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "point3dcorrector.h"
#include <adrcore/util/printmat.h>

namespace adore
{
	namespace core
	{

		Point3DCorrector::Point3DCorrector( const int numCams ):
				func( NULL ),
				jacobi( NULL ),
				goalVec( 2 * numCams ),
				inputVec( 3 )
		{
		}


		void Point3DCorrector::operator()( vector< Camera *> &cams, const vector< AVector2D *> &pts2D, AVector3D &pt3D )
		{
			func.setCams( &cams );
			jacobi.setCams( &cams );

			//printf("vor lm anpassung: (%f, %f, %f, %f)\n", pt3D[0], pt3D[1], pt3D[2], pt3D[3]);

			double z = pt3D( 2 );
			inputVec( 0 ) =  pt3D( 0 ) / z;
			inputVec( 1 ) =  pt3D( 1 ) / z;
			inputVec( 2 ) =  pt3D( 3 ) / z;

			int idx = 0;
			for ( uint i = 0; i < pts2D.size(); ++i )
			{
				goalVec( idx++ ) =  ( *pts2D[i] )( 0 ) / ( *pts2D[i] )( 2 );
				goalVec( idx++ ) =  ( *pts2D[i] )( 1 ) / ( *pts2D[i] )( 2 );
				//printf("in Point3DCorrector: w-coordinate of 2D-Point is: %f\n", pts2D[i][2]);
			}

			//printf("vor lm mini: \n");
			//printCvMat(inputVec);
			DefaultErrorFunc<double> errF( goalVec );
			lm.solve( inputVec, func, goalVec.size(), jacobi, errF );
			//printf("nach lm mini: \n");
			//printCvMat(inputVec);

			pt3D( 0 ) = inputVec( 0 );
			pt3D( 1 ) = inputVec( 1 );
			pt3D( 2 ) = 1.0;
			pt3D( 3 ) = inputVec( 2 );
		}

	}
}
