

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_MINIMIZATIONPOINT3DCORRECTOR_H
#define ADORE_MINIMIZATIONPOINT3DCORRECTOR_H

#include <adrcore/minimization/lm2.h>
#include <adrcore/minimization/defaulterrorfunc.h>
#include <adrcore/types/camera.h>
#include <adrcore/nview/point3dcorrfunc.h>
#include <adrcore/nview/point3dcorrjacobi.h>

namespace adore
{
	namespace core
	{

		/**
		 * This class minimizes the reprojection error by adjusting the 3D-point. So it should be used, if the
		 * Cameras are known.
		 * This class is used by the Point3DCalculator.
		 *  
		*/
		class Point3DCorrector
		{

			protected:
				Point3DCorrFunc func;

				Point3DCorrJacobi jacobi;
				LM2<Point3DCorrFunc, Point3DCorrJacobi, DefaultErrorFunc<double>, InvCovarUnused<double>::type > lm;
				AVector goalVec;
				AVector inputVec;

			public:
				/**
				 * Constructor.
				 * @param numCams the number of cameras
				    advised).
				 */
				//homogen means: homogenous 2D-Points should be used in minimization
				Point3DCorrector( const int numCams );

				/**
				 * Destructor.
				 */
				~Point3DCorrector()
				{
					;
				}
				
				void setNumCams(const int numCams)
				{
					goalVec.resize(2 * numCams);
				}

				/**
				 * The core function. Carries out the minimization.
				 * @param cams the cameras
				 * @param pts2D the 2D-points that correspond to the 3D-point.
				 * @param pt3D the 3D-point to be adjusted.
				 */
				void operator()( vector< Camera *> &cams, const vector< AVector2D * > &pts2D, AVector3D &pt3D );

		};

	}

}

#endif
