

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "quasiaffinator.h"

namespace adore
{

	namespace core
	{

		QuasiAffinator::QuasiAffinator()
		{
		}


		QuasiAffinator::~QuasiAffinator()
		{
		}

		void QuasiAffinator::modify ( NView &nview )
		{

			NView::CameraIterator cit = nview.getCamerasBegin();
			for ( ; cit != nview.getCamerasEnd(); ++cit )
			{
				AMatrix &cam = ( *cit )->getProjMat();
				AVector tmpCol = cam ( Range::all(), 2 );
				cam ( Range::all(), 2 ) = cam ( Range::all(), 3 );
				cam ( Range::all(), 3 ) = tmpCol;
			}

			NView::Point3DIterator pit = nview.getPoints3DBegin();
			for ( ; pit != nview.getPoints3DEnd(); ++pit )
			{
				if ( *pit )
				{
					double tmp = ( **pit ) ( 2 );
					( **pit ) ( 2 ) = ( **pit ) ( 3 );
					( **pit ) ( 3 ) = tmp;
				}
			}


		}

		int QuasiAffinator::countInvisible ( NView &nview )
		{
			int invisible = 0;
			NView::CorrIterator cit = nview.getPointCorrsBegin();
			while ( cit != nview.getPointCorrsEnd() )
			{
				PointCorr *pc = *cit; ++cit;
				if ( pc->size() > 0 )
				{
					NView::Point3D *pt3D = nview.getPoint3D ( pc );
					if ( pt3D )
					{
						for ( PointCorr::iterator it = pc->begin(); it != pc->end(); ++it )
						{
							FeaturePoint *fp = *it;
							Camera *cam = nview.getCamera ( fp->getView() );
							if ( !cam )
							{
								DBGOUT ( DBGCRITICAL, "countInvisible: missing Camera - stopping\n" );
								return 0;
							}
							if ( cam->getSign ( *pt3D ) <= 0 )
							{
								++invisible;
								break;
							}
						}
					}
					else
					{
						DBGOUT ( DBGNORMAL, "countInvisible: no 3D point for PointCorr %p\n", pc );
					}
				}
			}
			return invisible;
		}

		int QuasiAffinator::countInvisible ( NView &nview, AMatrix &H )
		{
			int invisible = 0;

			AVector HPt3D( 4 );
			Camera cams[nview.getNumCameras()];
			map<ViewBase *, int> viewToCam;

			int idx = 0;
			for ( NView::ViewIterator vit = nview.getViewsBegin(); vit != nview.getViewsEnd(); ++vit, ++idx )
			{
				ViewBase *v = *vit;
				assert( v );
				Camera *cam = nview.getCamera ( v );
				assert( cam );
				matMul2( cam->getProjMat(), H, cams[idx].getProjMat() );
//				if(!isRightHand(cams[idx].getProjMat()))
//				{
//                                    DBGOUT( DBGNORMAL, "QuasiAffinator: Camera was not right handed.\n" );
//                                  cams[idx].getProjMat() *= -1;
//                                  assert ( isRightHand ( cams[idx].getProjMat() ) );
//				}
				viewToCam[v] = idx;
			}

			invert( H );

			NView::CorrIterator cit = nview.getPointCorrsBegin();
			while ( cit != nview.getPointCorrsEnd() )
			{
				PointCorr *pc = *cit; ++cit;
				if ( pc->size() > 0 )
				{
					NView::Point3D *pt3D = nview.getPoint3D ( pc );
					if ( pt3D )
					{
						matMul2( H, *pt3D, HPt3D );
						for ( PointCorr::iterator it = pc->begin(); it != pc->end(); ++it )
						{
							FeaturePoint *fp = *it;
							Camera &cam = cams[viewToCam[fp->getView()]];
							if ( cam.getSign ( HPt3D ) <= 0 )
							{
								++invisible;
								break;
							}
						}
					}
					else
					{
						DBGOUT ( DBGNORMAL, "countInvisible: no 3D point for PointCorr %p\n", pc );
					}
				}
			}
			return invisible;
		}

	}

}
