

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWRANSACCLIENTMERGERCP_H
#define ADORE_NVIEWRANSACCLIENTMERGERCP_H

#include <adrcore/types/aarray.h>
#include <adrcore/random/ransacinterface.h>

namespace adore
{
	namespace core
	{


		/**
		 * This is a client of the generic RANSAC algo that implements RANCACInterface.
		 * Is is used to remove outliers when 3D-Points are used for the calculation of
		 * the merging homography. The homography is calculated of Cameras und Points, thus the suffix CP.
		 *  
		*/
		class RANSACClientMergerCP : public RANSACInterface
		{
			public:
				RANSACClientMergerCP();

				~RANSACClientMergerCP();

				void computeError( int resultIdx, vector<int> &inlierFlags, vector<double> &distVals );

				bool checkPrerequisitsAndInit( int numInliers, const vector< int > &indices );

				int applySample( const vector<int> &sample );

				void ransacDone();

				void setCameras( vector<AMatrix *> *c1, vector<AMatrix *> *c2 )
				{
					assert( c1->size() == c2->size() );
					cams1 = c1;
					cams2 = c2;
				}

				void setPoints3D( vector< AVector3D* > *p1, vector< AVector3D* > *p2 )
				{
					assert( p1->size() == p2->size() );
					pts3D1 = p1;
					pts3D2 = p2;
				}

				void setFractionThresh( double theValue )
				{
					fractionThresh = theValue;
				}

				double getFractionThresh() const
				{
					return fractionThresh;
				}

				void setOneViewCase( bool theValue )
				{
					oneViewCase = theValue;
				}

				bool getOneViewCase() const
				{
					return oneViewCase;
				}

				bool optimize( int bestIdx );

			private:
				bool oneViewCase;
				double fractionThresh;
				vector<AMatrix *> dummyCams;
				vector<AMatrix *> *cams1;//cams of NView 1
				vector<AMatrix *> *cams2;//cams of NView 2
				vector< AVector3D* > *pts3D1;
				vector< AVector3D* > *pts3D2;
				AMatrix H;
				AVector3D pt22;
		};

	}
}

#endif
