

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "tensorupdater.h"

namespace adore
{
	namespace core
	{

		TensorUpdater::TensorUpdater() :
				minAlgebraic ( true )
		{
		}

		TensorUpdater::TensorUpdater( const TensorUpdater &tu ) :
				minAlgebraic ( tu.minAlgebraic )
		{

		}

		TensorUpdater::~TensorUpdater()
		{
		}

		void TensorUpdater::modify ( NView &nview )
		{
			int numviews = nview.getNumViews();
			switch ( numviews )
			{
				case 2:
					modify2 ( nview );
					return;
				case 3:
					modify3 ( nview );
					return;
				default:
					error ( "wrong nviewset\n" );
					break;
			}
		}

		void TensorUpdater::modify3 ( NView &nview )
		{
			assert ( nview.getNumViews() == 3 );
			DBGOUT ( DBGNORMAL, "TensorUpdater: creating TTensor\n" );

			//gcc < 3.4 tcalc.template init<vector<NvPoint*>::iterator>(feats[0].begin(), feats[1].begin(), feats[2].begin(), feats[0].size(), true); //NviewSet-Points are considered homogeneous

			tcalc.setMinAlgebraic ( minAlgebraic );

			if ( tcalc.calculateTrifocalTensor ( nview.getPointCorrsBegin(), nview.getNumPointCorrs(), nview.getViewsBegin() ) < 1 )
			{
				DBGOUT ( DBGCRITICAL, "TensorUpdater: cannot calculate trifocal tensor - maybe too few correpondences\n" );
				if ( nview.getInfo() ) nview.getInfo()->setOkay ( false );
				return;
			}
			else
			{
				if ( !nview.getInfo() )
				{
					nview.setInfo( new NViewInfoTT );
				}
				nview.getInfo()->setOkay ( true );
				nview.getInfo()->setTTensor ( tcalc.getTensor1() );
			}
		}

		void TensorUpdater::modify2 ( NView &nview )
		{
			assert ( nview.getNumViews() == 2 );

			if ( fcalc.calculateFundamentalMatrix ( nview.getPointCorrsBegin(), nview.getNumPointCorrs(), nview.getViewsBegin() ) < 1 )
			{
				DBGOUT ( DBGCRITICAL, "TensorUpdater: cannot calculate fundamental matrix - maybe too few correpondences\n" );
				if ( nview.getInfo() ) nview.getInfo()->setOkay ( false );
				return;
			}
			else
			{
				if ( !nview.getInfo() )
				{
					nview.setInfo( new NViewInfoFM );
				}
				nview.getInfo()->setOkay ( true );
				nview.getInfo()->setFMatrix ( fcalc.getFMatrix1() );
			}
		}


	}
}
