

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "ttalgebraicfunc.h"
#include "ttensorcalculator.h"
#include <adrcore/util/printmat.h>

namespace adore
{
	namespace core
	{

		TTAlgebraicFunc::TTAlgebraicFunc( TTensorCalculator * p ):
				parent( p ),
				lastLineTensor( 27 )
		{
		}

		TTAlgebraicFunc::~TTAlgebraicFunc()
		{
		}

		AVector TTAlgebraicFunc::operator()( const AVector &epipoles ) const
		{
			AVector lineTensor = parent->lineTensor;
			this->operator()( epipoles, lineTensor );
			return lineTensor;
		}

		void TTAlgebraicFunc::operator()( const AVector &epipoles, AVector &lineTensor ) const
		{
			assert( lineTensor.rows() == 27 );
			assert( epipoles.rows() == 6 );

			AVector &epi1 = parent->epipole1;
			AVector &epi2 = parent->epipole2;
			AMatrix &E    = parent->E;
			AMatrix &linearEq   = parent->linearEq;

			epi1( 0 ) = epipoles( 0 );
			epi1( 1 ) = epipoles( 1 );
			epi1( 2 ) = epipoles( 2 );
			epi2( 0 ) = epipoles( 3 );
			epi2( 1 ) = epipoles( 4 );
			epi2( 2 ) = epipoles( 5 );

			parent->fillMatE( E, epi1, epi2 );
			parent->enforceEpipolesConstraint( linearEq, E, lineTensor, NULL );
		}


	}
}
