

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWTTALGEBRAICFUNC_H
#define ADORE_NVIEWTTALGEBRAICFUNC_H

#include <adrcore/types/aarray.h>

namespace adore
{
	namespace core
	{

		class TTensorCalculator;

		/**
		 * Used internally.
		 * This is the portion of Algo 16.2, p.396, that is used in LM-Minimization
		 * This Functor is a friend class of TTensorCalculator an alters its internal values, such as lineTensor, epipole[1,2], E, camRemains
		 *  
		*/
		class TTAlgebraicFunc
		{

			public:

				typedef double T_numtype;

			public:
				/**
				 * Constructor.
				 * @param parent the TTensorCalculator whose members can be used.
				 */
				TTAlgebraicFunc( TTensorCalculator * parent );

				/**
				 * Destructor.
				 */
				~TTAlgebraicFunc();

				/**
				 * a mapping from epipoles to an error that has to be minimized.
				 * @param epipoles a 6-vector consisting of the epipoles in the second and third View
				 * @param error a 27-vector describing the difference of the DLT-tensor and the algebraic valid tensor corresponding to the epipoles. The epipoles are modified to minimize this difference.
				 */
				AVector operator()( const AVector &epipoles ) const;
				void operator()( const AVector &epipoles, AVector &result ) const;

			protected:

				TTensorCalculator *parent;
				AVector lastLineTensor;
		};

	}

}

#endif
