

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef ADORE_PARAMETERIZATIONANGLEAXISFOCALCAMPARAM_H
#define ADORE_PARAMETERIZATIONANGLEAXISFOCALCAMPARAM_H

#include <adrcore/types/camera.h>
#include <adrcore/parameterization/angleaxis.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/


    /// TODO!
		class AngleAxisFocalCamParam
		{
			mutable AMatrix K, Kinv, R, T;
			mutable AVector t, t2;

			public:
				AngleAxisFocalCamParam() : K(3, 3), Kinv(3, 3), R(3, 3), T(3, 3), t(3), t2(3) {}

				int getSize() { return 7; }

				void fromParam( const vector<AVector> & param, vector<Camera *> & cams ) const
				{
					for ( uint i = 0; i < param.size(); ++i )
					{
						fromParam( param[i], *cams[i] );
					}
				}

				void toParam( const vector<Camera *> & cams, vector<AVector> & param ) const
				{
					for ( uint i = 0; i < param.size(); ++i )
					{
						toParam( *cams[i], param[i] );
					}
				}

				void fromParam( const AVector & param, Camera & cam ) const
				{
					fromParam( param, cam.getProjMat() );
				}

				void fromParam( const AVector & vec, AMatrix & cam ) const
				{
					double ax = vec(0), ay = vec(1), az = vec(2);
					double tx = vec(3), ty = vec(4), tz = vec(5);
					double f  = vec(6);

					// Rotation matrix
					t = ax, ay, az;
					AngleAxis::fromParam(t, R);

					K = f, 0, 0, 0, f, 0, 0, 0, 1;
					matMul2(K, R, T); // "Kalibration matrix"

					t = -tx, -ty, -tz;
					matMul2(K, t, t2);

					cam(Range::all(), Range(0, 2)) = T;
					cam(Range::all(), 3) = t2;

//					cout << "param: " << vec << endl;
//					AVector k(3);
//					double angle = sqrt(tx * tx + ty * ty + tz * tz);
//					k = ax / angle, ay / angle, az / angle;

				}

				void toParam( const Camera & cam, AVector & vec ) const
				{
					toParam( cam.getProjMat(), vec );
				}

				void toParam( const AMatrix & cam, AVector & vec ) const
				{
					T = cam ( Range::all(), Range ( 0, 2 ) );
					rq ( T, K, R );
					makeDiagOfROfRQPosisitve ( K, R );

					AngleAxis anax;
					anax.setIntervalCentre(0.0);
					anax.toParam(R, t);
					vec(0) = t(0);
					vec(1) = t(1);
					vec(2) = t(2);

					Kinv = K;
					invertUpperTriangular3x3Matrix ( Kinv );

					t = cam ( Range::all(), 3 );
					matMul2 ( Kinv, t, t2 );
//					matMul2_AT ( R, t2, t );//trans of R is the inverse of R

					vec(3) = -t2(0), vec(4) = -t2(1), vec(5) = -t2(2);

					double div = K ( 2, 2 );
					assert ( fabs( div ) > FLT_EPSILON );
					if(fabs( div ) > FLT_EPSILON) vec(6) = (0.5 * K(0, 0) + 0.5 * K(1, 1)) / div;
					else vec(6) = 0;
				}

		};

	}

}

#endif
