

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_AUTOCALIBRATIONCALIBRATIONPARAMMIXIN_H
#define ADORE_AUTOCALIBRATIONCALIBRATIONPARAMMIXIN_H

#include <vector>
#include <adrcore/types/camera.h>

namespace adore
{
	namespace core
	{

		using namespace std;
		/**
		 
		*/
		class CalibrationParamMixIn
		{
			public:

				virtual ~CalibrationParamMixIn()
				{
					;
				}

			protected:

				/**
				 * For a symmetric Matrix
				 * a b c d
				 * b e f g
				 * c f 1 h
				 * d g h x
				 * calculate x so that the matrix has rank 3.
				 * @return the value of x
				 */
				double calculate44( const double a, const double b, const double c, const double d, const double e, const double f, const double g, const double h ) const;

				/**
				 * For a symmetric Matrix
				 * a b c d
				 * b e f g
				 * c f i h
				 * d g h x
				 * calculate x so that the matrix has rank 3.
				 * @return the value of x
				 */
				double calculate44( const double a, const double b, const double c, const double d, const double e, const double f, const double g, const double h, const double i ) const;

				/**
				 * Calculate a Quadric from a minimal 8-Parameter parametrization
				 */
				void calculateQuadric( const double a, const double b, const double c, const double d, const double e, const double f, const double g, const double h, const double x, AMatrix &quadric ) const;

				double calculateWij( const int i, const int j, const double a, const double b, const double c, const double d, const double e, const double f, const double g, const double h, const double x, const AMatrix &cam ) const;

		};

	}

}

#endif
