

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_AUTOCALIBRATIONCALIBRATIONPARAMVERSATILE_H
#define ADORE_AUTOCALIBRATIONCALIBRATIONPARAMVERSATILE_H

#include <adrcore/parameterization/calibrationparammixin.h>
#include <adrcore/parameterization/camparamchoice.h>
#include <adrcore/parameterization/softconstraints.h>

namespace adore
{

	namespace core
	{

		/**
		 
		*/
		class CalibrationParamVersatile : private CalibrationParamMixIn
		{
			public:

				CalibrationParamVersatile();

				CalibrationParamVersatile( const CalibrationParamVersatile &cv );

				~CalibrationParamVersatile();

				void calculateQuadricAndConics( const AVector &params, int paramOff, AMatrix &quadric, vector<AMatrix> &conics );

				void getInitializedParams( AVector &params, int paramOff = 0, AMatrix *initQ = NULL, vector<AMatrix> *w = NULL );

				void setBiasIdCam( const bool& theValue )
				{
					biasID = theValue;
					if ( biasID ) numQParams = 3;
					else numQParams = 8;
				}

				bool getBiasIdCam() const
				{
					return biasID;
				}

//				void setChoice( CamParamChoice* theValue )
//				{
//					choice = theValue;
//				}

				CamParamChoice &getChoice()
				{
					return choice;
				}

				const CamParamChoice &getChoice() const
				{
					return choice;
				}

				int getNumParams() const;

//				void setSoftConstraints( SoftConstraintSet* theValue )
//				{
//					softConstraints = theValue;
//				}

				SoftConstraintSet &getSoftConstraints()
				{
					return softConstraints;
				}

				const SoftConstraintSet &getSoftConstraints() const
				{
					return softConstraints;
				}

				/*void setCONSTSoftConstraints(SoftConstraintSet* theValue)
				{
				    constSoftConstraints = theValue;
				}*/

				SoftConstraintSet &getCONSTSoftConstraints()
				{
					return constSoftConstraints;
				}

				const SoftConstraintSet &getCONSTSoftConstraints() const
				{
					return constSoftConstraints;
				}

				void evaluateSoftConstraints( AVector &err, int errOff );

				int getNumSoftConstraints();

				/**
				 * The value it should have
				 */
				void getSoftConstraintsGoal( AVector &goal, int offset );

			private:

				CamParamChoice choice;
				SoftConstraintSet softConstraints;
				SoftConstraintSet constSoftConstraints;

				bool biasID;
				int numQParams;

				vector<AMatrix> K;

				void calculateQuadric( const AVector &params, int paramOff, AMatrix &q ) const
				{
					double a = params( paramOff++ );
					double b = params( paramOff++ );
					double c = params( paramOff++ );
					double d = params( paramOff++ );
					double e = params( paramOff++ );
					double f = params( paramOff++ );
					double g = params( paramOff++ );
					double h = params( paramOff );
					CalibrationParamMixIn::calculateQuadric( a, b, c, d, e, f, g, h, calculate44( a, b, c, d, e, f, g, h ), q );
				}

				void calculateQuadric( const AVector &params, int paramOff, const AMatrix &conic0, AMatrix &q )
				{
					double a = conic0( 0, 0 );
					double b = conic0( 0, 1 );
					double c = conic0( 0, 2 );
                    double d = params( paramOff++ );
					double e = conic0( 1, 1 );
					double f = conic0( 1, 2 );
                    double g = params( paramOff++ );
                    double h = params( paramOff );
					//double x = d*d + g*g + h*h;  //this results in a _much_ slower convergence
					double x = calculate44( a, b, c, d, e, f, g, h ); //than this
					CalibrationParamMixIn::calculateQuadric( a, b, c, d, e, f, g, h, x, q );
				}

				void calculateConics( const AVector &params, int paramOff, vector<AMatrix> &conics );
				void fillIt( AVector &params, int &row, int mode, vector<double> &vals );
				void initialize( AVector &param, int &row, int mode, vector<double> &vals );
				void ensureKSize( uint size );

				void initVector( vector<double> &vals, double initVal )
				{
					for ( uint i = 0; i < vals.size(); ++i ) vals[i] = initVal;
				}
		};

	}

}

#endif
