

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PERSISTENCE_BOOST_AARRAY_H
#define PERSISTENCE_BOOST_AARRAY_H

#include <adrcore/types/aarray.h>

namespace boost
{
	namespace serialization
	{
		using namespace adore::core;

		template<typename Archive, typename Type, int rank>
		void save( Archive & ar, const AArray<Type, rank>& t, unsigned int version )
		{
			if ( 1 == rank )
			{
				ar << ( const int& )t.size();
			}
			typename AArray<Type, rank>::const_iterator begin = t.begin();
			typename AArray<Type, rank>::const_iterator end = t.end();
			for ( ; begin != end; ++begin )
				ar << *begin;
		}

		template <typename Archive, typename Type, int rank> struct resize_on_load
		{
			void operator()(Archive& /*ar*/, AArray<Type, rank>& /*t*/) const {}
		};

		template <typename Archive, typename Type> struct resize_on_load<Archive, Type, 1>
		{
			void operator()(Archive& ar, AArray<Type, 1>& t) const
			{
				int size;
				ar >> size;
				t.resize( size );
			}
		};

		template<typename Archive, typename Type, int rank>
		void load( Archive & ar, AArray<Type, rank>& t, unsigned int version )
		{
			resize_on_load<Archive, Type, rank> resizer;
			resizer(ar, t);

			typename AArray<Type, rank>::iterator begin = t.begin();
			typename AArray<Type, rank>::iterator end = t.end();
			for ( ; begin != end; ++begin )
				ar >> *begin;
		}

		template<typename Archive, typename Type, int rank>
		inline void serialize( Archive & ar, AArray<Type, rank>& t, const unsigned int file_version )
		{
			split_free( ar, t, file_version );
		}
	}
}

#endif // PERSISTENCE_BOOST_AARRAY_H
