

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PERSISTENCE_BOOST_NVIEW_H
#define PERSISTENCE_BOOST_NVIEW_H

#include <map>

#include <boost/serialization/split_free.hpp>
#include <boost/serialization/vector.hpp>

#include <adrcore/types/nview.h>

#include <adrcore/persistence/boost/aarray.h>

namespace boost
{
	namespace serialization
	{
		using namespace adore::core;
		using namespace persist;
		using namespace std;

		template<class Archive>
		void save( Archive & ar, const NView& nview, unsigned int version )
		{
			ar << ( const bool& )nview.isNormalized();

			NView::ViewIterator vit = nview.getViewsBegin();
			NView::ViewIterator vend = nview.getViewsEnd();
			for ( ; vit != vend; ++vit )
			{
				Camera* cam = nview.getCamera ( *vit );
				if ( cam )
				{
					ar << ( const bool& ) true; /// hasCam
					ar << ( const int & )( *vit )->getId();
					ar << ( const AMatrix& ) cam->getProjMat();
				}
				else ar << ( const bool& ) false; /// hasCam
			}

			ar << ( const uint& ) nview.getNumPointCorrs();
			NView::CorrIterator cit = nview.getPointCorrsBegin();
			NView::CorrIterator cend = nview.getPointCorrsEnd();
			for ( ; cit != cend; ++cit )
			{
				ar << ( const float& ) ( *cit )->getBadness();
				ar << ( const uint & )( *cit )->getSize(); /// featsNum
				PointCorr::Iterator fit = ( *cit )->getBegin();
				PointCorr::Iterator fend = ( *cit )->getEnd();
				for ( ; fit != fend; ++fit )
					ar << ( const uint& )( *fit )->getId();
			}

			ar << ( const uint& ) nview.getNumPoints3D();
			NView::ConstPoint3DIterator pit = nview.getPoints3DBegin();
			NView::ConstPoint3DIterator pend = nview.getPoints3DEnd();
			for ( ; pit != pend; ++pit )
			{
				ar << ( const AVector& )**pit;
			}

			ar << ( const uint& ) nview.getNumLineCorrs();
			NView::LineCorrIterator lcit = nview.getLineCorrsBegin();
			NView::LineCorrIterator lcend = nview.getLineCorrsEnd();
			for ( ; lcit != lcend; ++lcit)
			{
				ar << (const float& ) ( *lcit )->getBadness();
				ar << ( const uint& ) ( *lcit )->getSize();
				LineCorr::Iterator lit = ( *lcit )->getBegin();
				LineCorr::Iterator lend = ( *lcit )->getEnd();
				for ( ; lit != lend; ++lit)
					ar << ( const uint& )( *lit )->getId();
			}

			/// info
			if ( nview.isNViewSet() )
			{
				adore::core::NViewInfo* nvi = nview.getInfo();
				ar << ( const bool& )true; /// hasInfo
				ar << ( const int& )static_cast<int>( nvi->getType() );
				ar << ( const float& )nvi->getLambda();
				AMatrix fmat = nvi->getFMatrix();
				if ( fmat.rows() == 3 && fmat.cols() == 3 )
				{
					ar << ( const bool& )true; /** SERIALIZING ***has_matrix*** */
					ar << ( const AMatrix& )fmat;  /** SERIALIZING ***FMatrix*** */
				}
				else ar << ( const bool& )false; /** SERIALIZING ***has_matrix*** */

				ATensor3 tten = nvi->getTTensor();
				//if ( tten.rows() == 3 && tten.cols() == 3 )
				if ( tten.size1() == 3 && tten.size2() == 3 && tten.size3() == 3 )
				{
					ar << ( const bool& )true; /** SERIALIZING ***has_tensor*** */
					ar << ( const ATensor3 & )tten;
				}
				else ar << ( const bool& )false; /** SERIALIZING ***has_tensor*** */
			}
			else ar << ( const bool& )false; /// hasInfo
		}

		template<class Archive>
		void load( Archive & ar, NView& nview, unsigned int version )
		{
			map<uint, ViewBase*> viewmap;
			map<uint, FeaturePoint*> featmap;
			map<uint, FeatureLine*> featLineMap;

			NView::ViewIterator vit = nview.getViewsBegin();
			NView::ViewIterator vend = nview.getViewsEnd();
			for ( ; vit != vend; ++vit)
			{
				ViewBase* view = *vit;
				viewmap[view->getId()] = view;
				for ( uint i = 0; i < maxFeatureTypeNo; ++i )
				{
					if ( view->hasFeatures( i ) )
					{
						ViewBase::PointsIterator fit = view->getPointsBegin( i );
						ViewBase::PointsIterator fend = view->getPointsEnd( i );
						for ( ; fit != fend; ++fit )
							featmap[( *fit )->getId()] = *fit;
					}
				}
				for ( uint i = 0; i < view->getNumSegments(); ++i) {
					FeatureLine* fl = view->getSegment(i).get();
					featLineMap[fl->getId()] = fl;
				}
			}

			bool normalized; ar >> normalized;
			nview.setNormalized( normalized );

			uint viewsNum = nview.getNumViews();
			for(uint i = 0; i < viewsNum; ++i)
			{
				bool hasCam; ar >> hasCam;
				if ( !hasCam ) continue;

				int viewId; ar >> viewId;
				Camera* cam;
				cam = nview.getCamera ( viewmap[viewId], true );
				AMatrix projMat( 3, 4 ); ar >> projMat;
				cam->getProjMat() = projMat;
			}

			uint corrsNum; ar >> corrsNum;
			vector<FeaturePoint*> data( viewsNum );
			vector<PointCorr *> corrCollection;
			for ( uint i = 0; i < corrsNum; ++i )
			{
				//determine featcount of correspondence
				float badness; ar >> badness;
				uint featsNum; ar >> featsNum;
				vector<int> idxs;
				for ( uint j = 0; j < featsNum; ++j )
				{
					//load the feature id and put it into idxs
					int fid; ar >> fid;
					idxs.push_back( fid );
				}
				for ( uint v = 0; v < viewsNum; v++ )
				{
					if ( v < idxs.size() )
					{
						data[v] = featmap[idxs[v]];
					}
					else
					{
						data[v] = NULL;
					}
				}
				PointCorr * corr = nview.addPointCorrNoCheck ( data.begin(), data.end(), badness );
				corrCollection.push_back( corr );
			}
			/*
				loads 3D points and assumes same order as corrs
				remember: reading everything is mandatory
			*/
			uint p3dsNum; ar >> p3dsNum;
			vector<AVector*> pts3d;
			for ( uint j = 0; j < p3dsNum; ++j )
			{
				AVector* pt3d;
				pt3d = new AVector( 4 );
				ar >> *pt3d;
				pts3d.push_back ( pt3d );
			}
			if ( p3dsNum == corrsNum )
			{
				for ( uint c = 0; c < corrsNum; c++ )
					nview.setPoint3D ( pts3d[c], corrCollection[c] );
			}

			uint lineCorrsNum; ar >> lineCorrsNum;
			vector<FeatureLine*> lineData( viewsNum );
			for ( uint i = 0; i < lineCorrsNum; ++i )
			{
				//determine featcount of correspondence
				float badness; ar >> badness;
				uint linesNum; ar >> linesNum;
				vector<int> idxs;
				for ( uint j = 0; j < linesNum; ++j )
				{
					//load the feature id and put it into idxs
					int id; ar >> id;
					idxs.push_back( id );
				}
				for ( uint v = 0; v < viewsNum; v++ )
				{
					if ( v < idxs.size() )
					{
						lineData[v] = featLineMap[idxs[v]];
					}
					else
					{
						featLineMap[v] = NULL;
					}
				}
				nview.addLineCorrNoCheck ( lineData.begin(), lineData.end(), badness );
			}

			bool has_info; ar >> has_info;
			if ( has_info )
			{
				int infoType; ar >> infoType;
				NViewInfo::TYPE type = static_cast<NViewInfo::TYPE>( infoType );
				NViewInfo* info = createNViewInfo( type );
				float lambda; ar >> lambda;
				info->setLambda( lambda );
				bool has_matrix; ar >> has_matrix;
				if ( has_matrix )
				{
					AMatrix fmat( 3, 3 ); ar >> fmat;
					info->setFMatrix( fmat );
					info->setOkay( true );
				}
				bool has_tensor; ar >> has_tensor;
				if ( has_tensor )
				{
					ATensor3 tten( 3, 3, 3 ); ar >> tten;
					info->setTTensor( tten );
					info->setOkay( true );
				}
				nview.setInfo( info );
			}
		}

		template<class Archive>
		inline void serialize( Archive & ar, NView& nview, const unsigned int file_version )
		{
			split_free( ar, nview, file_version );
		}
	}
}

#endif // PERSISTENCE_BOOST_NVIEW_H
