

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "imgloader.h"

namespace adore
{
	namespace core
	{

		bool ImgLoaderBase::isImage ( string filename )
		{
			int width, height, channels;
			getInfo ( filename, width, height, channels );
			if ( width == -1 || height == -1 || channels == -1 ) return false;
			return true;
		}

		void ImgLoaderBase::getInfo ( string filename, int &width, int &height, int &channels )
		{
			switch ( extract_suffix ( filename ) )
			{
				case Unknown:
					width=-1;
					height=-1;
					break;
				case Png:
				{
					PNGPersistence pngloader;
					pngloader.getInfo ( filename.c_str(), width, height, channels );
				}
				break;
				case Jpeg:
				{
					JPEGPersistence jpegloader;
					bool ok = jpegloader.getInfo ( filename.c_str(), width, height, channels );
					if(!ok)
					{
						width=-1;
						height=-1;
					}
				}
				break;
				default:
					cout << "ImgLoader::getInfo: unexpected case" << endl;
					exit ( 1 );
			}
			return;
		}

		void ImgLoaderBase::getInfo( string filename, const vector<char>& data,
		                             int & width, int & height, int & channels )
		{
			switch ( extract_suffix ( filename ) )
			{
				case Unknown:
					width=-1;
					height=-1;
					break;
				case Png:
				{
//					PngLoaderBase pngloader;
					width=-1;
					height=-1;
//					pngloader.getInfo ( filename, width, height, channels );
				}
				break;
				case Jpeg:
				{
					JPEGPersistence jpegloader;
					bool ok = jpegloader.getInfo ( &data[0], data.size(), width, height, channels );
					if(!ok)
					{
						width=-1;
						height=-1;
					}
				}
				break;
				default:
					cout << "ImgLoader::getInfo: unexpected case" << endl;
					exit ( 1 );
			}
			return;
		}

		ImgFileType ImgLoaderBase::extract_suffix ( string filename )
		{
			int pos;
			pos=filename.rfind ( ".",string::npos );
			if ( pos==0 || pos== ( int ) string::npos )
			{
				cout<<"ImgLoaderBase: Filename not valid: " << filename <<endl;
				return Unknown;
			};
			string suffix=filename.substr ( filename.rfind ( ".",string::npos ),string::npos );

			if ( suffix==".jpg" || suffix==".JPG" )
				return Jpeg;
			else
				if ( suffix==".png" || suffix==".PNG" )
					return Png;
				else
					return Unknown;
		}


	}
} //namespcae

