

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PERSIST_H
#define PERSIST_H

#include <vector>
#include <string>

#include <adrcore/types/nviewmap.h>

namespace persist
{

	using namespace std;
	using namespace adore::core;

	class Options
	{
			bool embed_images;
			bool store_descr;
		public:
			explicit Options( bool embed = false, bool descr = true )
					: embed_images( embed ), store_descr( descr ) {}
			bool embedImages() const
			{
				return embed_images;
			}
			bool storeDescriptors() const
			{
				return store_descr;
			}
			template<class Archive>
			void serialize( Archive & ar, const unsigned int version )
			{
				ar & embed_images;
				ar & store_descr;
			}
	};

	static const string default_obj_name = "_dummy";

	template < typename D >
	class Persist
	{
		public:
			Persist ( const string name )
			{
				impl.open ( name );
			}

			~Persist()
			{
				impl.close();
			}

			vector< string > objectNames() const
			{
				return impl.objectNames();
			}

			/**
			 * if the nview has been generated from another file, this method returns its name
			 */
			string getSrcFile ( string objname ) const
			{
				return impl.getSrcFile ( objname );
			}

			void setSrcFile ( string objname, string filename ) const
			{
				impl.setSrcFile ( objname, filename );
			}

			void deleteObject ( string objname )
			{
				impl.deleteObject ( objname );
			}

			void saveNViewMap ( const NViewMap& nvm, const string name = default_obj_name,
			                    bool simple = false, const Options& opts = Options(),
			                    const string srcFile = "" ) const
			{
				impl.saveNViewMap ( nvm, name, simple, opts, srcFile );
			}

			bool loadNViewMap ( NViewMap& nvm, const string name = default_obj_name, const bool loadImages = true ) const
			{
				return impl.loadNViewMap ( nvm, name );
			}

			/**
				convenience function that takes an NView instead of an NViewMap
			*/
			void saveNView ( NView &nview, const string name = default_obj_name,
			                 bool simple = false, const Options& opts = Options(),
			                 const string srcFile = "" )
			{
				NViewMap map;
				map.setNViewResetMode ( KEEP_NVIEW );
				map.setViewResetMode ( KEEP_VIEW );
				map.setNView ( nview.getViewsBegin(), nview.getViewsEnd(), &nview );
				DBGOUT ( DBGNORMAL, "Saving NView Wrapped in NViewMap (srcfile = %s)\n", srcFile.c_str() );
				saveNViewMap ( map, name, simple, opts, srcFile );
			}

			/**
				loads a single NView. This method wants the db to have
				one NView only. If this assumption is violated, additional
				NViews and Views are currently kept in memory and are not
				properly deleted.
			*/
			void loadNView ( NView &nview, const string name = default_obj_name, const bool loadImages = true )
			{
				NViewMap map;
				map.setNViewResetMode ( DELETE_NVIEW );
				map.setViewResetMode ( KEEP_VIEW );
				DBGOUT ( DBGNORMAL, "Loading NView Wrapped in NViewMap (obj name = %s)\n", name.c_str() );
				loadNViewMap ( map, name, loadImages );
				NView *mapnv = *map.getEntriesBegin();
				mapnv->setCamResetMode( KEEP_CAM );
				mapnv->setCorrResetMode( KEEP_CORR );
				mapnv->setPoint3DResetMode( KEEP_POINT3D );
				nview = *mapnv;
				DBGOUT ( DBGNORMAL, "Persist: NView loaded\n" );
				DBGOUT ( DBGNORMAL, "%d views, %d corrs, %d 3dpoints, %d cams\n", nview.getNumViews(), nview.getNumPointCorrs(), nview.getNumPoints3D(), nview.getNumCameras() );
			}

		private:
			D impl;
	};

}

#endif
