

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PERSIST_BOOST_H
#define PERSIST_BOOST_H

#include <vector>
#include <string>
#include <utility>

#include <adrcore/types/nviewmap.h>
#include "boost/tuple/tuple.hpp"

#include "ofzstream.h"
#include "ifzstream.h"

namespace persist
{

	using namespace std;
	using namespace adore::core;
	using boost::tuple;
	using boost::get;

	class PersistBoost
	{
		private:
			typedef std::ofstream ObjDirOStream;
			typedef zstream::ifzstream ObjDirIStream;
			typedef zstream::ofzstream ObjOStream;
			typedef zstream::ifzstream ObjIStream;

		public:
			typedef tuple < string, string, string > ParseTuple;
			typedef vector< ParseTuple > ObjectList;

			enum
			{
				OBJNAME = 0,
				PATH = 1,
				SRCFILE = 2
			};

		private:
			enum FileType
			{
				New,
				Header,
				Object,
				Invalid
			};

			string filename;
			ObjectList objList;
			FileType lastState;

		public:
			void open ( const string& name );
			void close();
			vector< string > objectNames() const;

			string getSrcFile ( string objname ) const;

			void setSrcFile ( string /*objname*/, string /*filename*/ ) const
			{
				assert ( false );
			}

			void deleteObject ( string objname );

			void saveNViewMap ( const NViewMap& nvm, const string& name, bool simple = false,
			                    const Options& opts = Options(), const string srcFile = "" ) const;

			bool loadNViewMap ( NViewMap& nvm, const string& name ) const;

		private:
			FileType checkFile() const;

			bool fillObjectList();

		public:
			static string generateObjFileName ( const string& filename, const string& objname );
	};

}

#endif
