

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "zinbuf.h"

#include <iostream>
#include <cstring>

#include <zlib.h>

namespace zstream
{

	using namespace std;

	zinbuf::zinbuf()
			: lastError( 0 ), file( 0 )
	{
	}

	zinbuf::~ zinbuf()
	{
		if ( file ) close();
	}

	void zinbuf::open( const char * name )
	{
		if ( file ) close();
		file = gzopen( name, "rb" );
		setg( buffer + 4, buffer + 4, buffer + 4 );
	}

	bool zinbuf::is_open() const
	{
		return 0 != file;
	}

	zinbuf * zinbuf::close()
	{
		if ( !file ) return this;

		lastError = gzclose( file );
		file = 0;
		if ( Z_OK != lastError )
		{
			cout << "zinbuf::close(): error - " << lastError << endl;
			return 0;
		}
		return this;
	}


	zinbuf::int_type zinbuf::underflow()
	{
		if ( gptr() < egptr() )
		{
			return traits_type::to_int_type( *gptr() );
		}

		int numBack = gptr() - eback();
		if ( 4 < numBack ) numBack = 4;

		std::memmove( buffer + ( 4 - numBack ),
		              gptr() - numBack, numBack );

		int num = gzread( file, buffer + 4, buf_size - 4 );
		if ( 0 >= num )
		{
			return EOF;
		}

		setg( buffer + ( 4 - numBack ), buffer + 4, buffer + 4 + num );

		return traits_type::to_int_type( *gptr() );
	}

	zinbuf::pos_type zinbuf::seekoff( off_type offset, std::ios_base::seekdir dir,
	                                  std::ios_base::openmode mode )
	{
		if ( offset == 0 && dir == std::ios_base::cur && ios_base::in == mode )
		{
			z_off_t gz_off = gztell( file );
			z_off_t buff_off_back = egptr() - gptr();
			return static_cast<pos_type>( gz_off - buff_off_back );
		}
		return static_cast<pos_type>( -1 );
	}

	zinbuf::pos_type zinbuf::seekpos( pos_type sp, std::ios_base::openmode mode )
	{
		if ( ios_base::in == mode )
		{
			z_off_t result = gzseek( file, static_cast<z_off_t>( sp ), SEEK_SET );
			setg( buffer + 4, buffer + 4, buffer + 4 );
			return static_cast<pos_type>( result );
		}
		return static_cast<pos_type>( -1 );
	}

} // zstream
