

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "integerrandomset.h"
#include <cstdio>
#include <adrcore/random/random.h>
namespace adore
{
	namespace core
	{

		IntegerRandomSet::IntegerRandomSet():
				method( METHOD1 ),
				q(),
				remaining( 0 ),
				idx( 0 ),
				numNumbers( 0 )
		{
		}


		IntegerRandomSet::~IntegerRandomSet()
		{
		}

		void IntegerRandomSet::createChoice( vector<int> &a, int lower, int upper, vector<int>* mask, METHOD_TYPE m )
		{
			int size = ( int )a.size();
			assert( lower <= upper );               //Interval correct
			assert( size > 0 );    //enough space

			int method = this->method;
			if ( m != DEFAULT ) method = m;

			if ( method == METHOD1 )
			{
				int i;
				int *numbers = new int[upper - lower + 1];
				int **numptrs = new int*[upper -lower + 1];
				int ***maskptrs = new int**[upper -lower + 1];
				for ( i = lower; i <= upper; i++ )
				{
					numbers[i - lower] = i;
					numptrs[i - lower] = &numbers[i - lower];
					maskptrs[i - lower] = &numptrs[i - lower];
				}

				//debug
//				printf( "mask before:\n" );
//				for ( int i = 0; i < maskSize; ++i )
//				{
//					printf( "%d  ", mask[i] );
//				}
//				printf( "\n" );
//
//				printf( "array before:\n" );
//				for ( int i = 0; i <= upper-lower; ++i )
//				{
//					printf( "%d  ", *numptrs[i] );
//				}
//				printf( "\n" );

				int masked = 0;
				if ( mask && mask->size() > 0 )
				{
					for ( uint i = 0; i < mask->size(); ++i )
					{
						int maskVal = ( *mask )[i];
						if ( maskVal >= lower && maskVal <= upper )
						{
							masked++;
							maskVal -= lower; //maskVal has now the index
							int *tmpptr = numptrs[upper - lower - masked + 1];
							numptrs[upper -lower - masked + 1] = *maskptrs[maskVal];
							*maskptrs[maskVal] = tmpptr;
							maskptrs[maskVal] = &numptrs[upper -lower - masked + 1];
							maskptrs[upper -lower - masked + 1] = &numptrs[maskVal];
						}
					}
				}

				//debug
//				printf( "array after:\n" );
//				for ( int i = 0; i <= upper-lower; ++i )
//				{
//					printf( "%d  ", *numptrs[i] );
//				}
//				printf( "\n" );

				assert( ( int )size <= upper - lower + 1 - masked );  //enough space

				int pos = 0;
				int *tmpptr = NULL;
				while ( pos < size )
				{
					int idx = pos + ( int ) ( ( ( double )( upper - lower - pos - masked ) ) * intRand() / ( RAND_MAX + 1.0 ) );
					a[pos] = *numptrs[idx];

					//swap used number to position that will not be used again
					tmpptr = numptrs[pos];
					numptrs[pos] = numptrs[idx];
					numptrs[idx] = tmpptr;

					++pos;
				}

				delete [] numbers;
				delete [] numptrs;
				delete [] maskptrs;

			}
			else if ( method == METHOD2 )
			{
				uint i;

				remaining = size;//total number of elemets to compute
				idx = 0;        //next array element to compute
				numNumbers = upper - lower + 1;

				q.push_back( lower );//lower bound of random number (inclusive)
				q.push_back( upper );//upper bound of random number (inclusive)

				if ( mask && mask->size() > 0 )
				{
					printf( "Method 2 doesn't support a mask yet\n" );
				}

				do
				{

					//choose Fragment randomly
					int fragmember = ( int ) ( ( ( double )numNumbers ) * intRand() / ( RAND_MAX + 1.0 ) );
					int done = 0;
					int lower = 0;
					int upper = 0;

					//A fragment is chosen according to its length
					list<int>::iterator ii = q.begin();

					//printf("q-size: %d   fragmember: %d\n", q.size(), fragmember);

					for ( i = 0; i < q.size(); i+=2 )
					{
						lower = *ii++;
						upper = *ii++;

						done += upper - lower + 1;
						if ( done > fragmember ) break; //the intervall with fragmemberth number has been found
					}

					--ii;
					ii = q.erase( ii );
					--ii;
					ii = q.erase( ii );

					int number = lower + ( int ) ( ( ( double )( upper - lower ) ) * intRand() / ( RAND_MAX + 1.0 ) );
					//printf("lower: %d, upper: %d, number: %d\n", lower, upper, number);

					a[idx++] = number;
					--remaining;

					if ( number > lower )
					{
						q.push_back( lower );
						q.push_back( number - 1 );
					}
					if ( number < upper )
					{
						q.push_back( number + 1 );
						q.push_back( upper );
					}
				}
				while ( remaining );

			}
			else
			{
				printf( "Wrong method" );
				exit( -1 );
			}
		}

		void IntegerRandomSet::print()
		{
			printf( "remaining: %d  idx: %d  numNumbers: %d\n",remaining, idx, numNumbers );
		}

		void IntegerRandomSet::printList()
		{
			uint i;
			list<int>::iterator ii = q.begin();
			printf( "List: " );
			for ( i = 0; i < q.size(); i += 2 )
			{
				int lower = *ii++;
				int upper = *ii++;
				printf( "(%d, %d) ", lower, upper );
			}
			printf( "\n" );
		}


	}
}
