

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREUNIONFIND_H
#define ADORE_COREUNIONFIND_H

#include <set>
#include <vector>
#include <algorithm>
#include <cassert>

using namespace std;

namespace adore
{

	namespace core
	{

		/**
			A stupid union find using stl classes
			nothing fancy like path compression
		*/

		const int NullSetId = -1;

		template<class ElemT>
		class UnionFind
		{
			private:
				int numSets;

				struct Node
				{
					int rank;
					int index;
					Node* parent;
					ElemT element;

					Node( ElemT &e, int idx ): rank( 0 ), index( idx ), parent( NULL ), element( e ) { }
				};

				vector<Node> nodesList;

			public:
				typedef int SetId;
			public:

				/**
				 * @param num reserve some space
				 */
				UnionFind( int num = 100 ):numSets( 0 )
				{
					nodesList.reserve( num );
				}

				int findSet( uint elementId )
				{
					if ( ( getNumElements() == 0 ) || ( elementId >= getNumElements() ) )
						return NullSetId;

					Node &curNode = nodesList[elementId];
					while ( curNode.parent != NULL )
						curNode = *curNode.parent;
					Node &root = curNode;

					//path compression
					curNode = nodesList[elementId];
					while ( curNode.index != root.index )
					{
						Node *next = curNode.parent;
						curNode.parent = &root;
						curNode = *next;
					}

					return root.index;
				}

				void unionSets( int setID1, int setID2 )
				{
					if ( setID1 == setID2 )
						return;

					Node &set1 = nodesList[setID1];
					Node &set2 = nodesList[setID2];

					if ( set1.Node::rank > set2.Node::rank )
						set2.parent = &set1;
					else if ( set1.Node::rank < set2.Node::rank )
						set1.parent = &set2;
					else // if set1->rank == set2->rank
					{
						set2.parent = &set1;
						++set1.rank;
					}
					--numSets;
				}

				void addSet( ElemT &e )
				{
					nodesList.push_back( Node( e, getNumElements() ) );

					++numSets;
				}

				uint getNumElements()
				{
					return nodesList.size();
				}

				int getNumSets()
				{
					return numSets;
				}
		};

	}

}

#endif
