

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "integralimage.h"

namespace adore
{
	namespace core
	{

		template <typename dst>
		const typename IntegralImage<dst>::imgT IntegralImage<dst>::imgTmax = numeric_limits<imgT>::max();

		template <typename dst>
		IntegralImage<dst>::IntegralImage( ImageT * image ) :
				data( NULL ),
				height( 0 ),
				width( 0 )
		{
			init(image);
		}

		template <typename dst>
		void IntegralImage<dst>::init( ImageT * image )
		{
			if ( NULL == image ) return;
			if (image->getNumChannels() != 1)
			{
				DBGOUT(DBGCRITICAL, "IntegralImage: wrong number of channels\n");
				return;
			}

			height = image->getROI().getHeight(), width = image->getROI().getWidth();
			double maxval = height * width * imgTmax;
			if ( static_cast<double>( std::numeric_limits<dst>::max() ) < maxval )
			{
				DBGOUT(DBGCRITICAL, "IntegralImage: Image may not be correct due to insufficient range of numerical type\n");
				return;
			}

			data.reset( new dst[ width * height ] );

			compute( image );
		}
		
		template <typename dst>
		void IntegralImage<dst>::compute( ImageT * image )
		{
			uint step = image->getWStep();
			Image1DIterator<imgT> linesrc = image->getXIterator( image->getROI().getY(), image->getROI().getX() );
			Image1DIterator<imgT> it = linesrc;
			dst* ptr = data.get();
			dst x_sum = 0;

			for ( uint x = 0; x < width; ++x, ++it, ++ptr )
			{
				x_sum += ( dst )*it;
				*ptr = x_sum;
			}
			linesrc.nextLine( step );

			dst* ptrlastRow = data.get();
			for ( uint y = 1; y < height; ++y, linesrc.nextLine( step ) )
			{
				x_sum = 0;
				it = linesrc;
				for ( uint x = 0; x < width; ++x, ++it, ++ptr, ++ptrlastRow )
				{
					x_sum += ( dst )*it;
					*ptr = *ptrlastRow + x_sum;
				}
			}
		}

		template class IntegralImage<int>;
		template class IntegralImage<float>;
		template class IntegralImage<double>;

	} // core
} // adore
