

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef SURFDESCRIPTOR_H
#define SURFDESCRIPTOR_H

#include "surfcommon.h"
#include "orientationhistogram.h"

#include <vector>
#include <boost/shared_ptr.hpp>

#include <adrcore/viewmatch/descriptorcreator.h>

namespace adore
{
	namespace core
	{

		class SurfDescriptor : public DescriptorCreator
		{

			private:

				typedef ImagePlane<imgT> ImageT;
				typedef FeatureDescriptor::DescVecT DescVecT;
				typedef float real_type;
				
				typedef OrientationHistogram<real_type>::vec2 vec2;
				typedef AArray<real_type, 2> RotMatT;

				enum State { Invalid, Valid };
				State state;

				boost::shared_ptr<ImageT> local_image_gray;
				ImageT * image_gray;
				SurfIntegralImageT local_iimage;
				SurfIntegralImageT * iimage;

				bool simple;
				int subregions;
				bool splitSums;
				int windowSize;

				//temporary variables for histogram computation
				mutable OrientationHistogram<real_type> histogram;
				mutable vec2 tmpVec2;
				typedef std::vector<real_type> vectorT;
				mutable vectorT orients;

				enum { SubRegsMin = 2, OrientSampleSize = 4, SubRegSize = 5, OrientRadius = 6, SubRegsMax = 7 };
				
			public:

				SurfDescriptor( uint featID = 0, uint descrID = 0, bool upright = false, int subregs = 4, bool split = false );
				SurfDescriptor( const SurfDescriptor &sd );
				DescriptorCreator * clone();

				~SurfDescriptor();
				void reset();

				/// see surffinder.h. you can use these 2 functions, and avoid copy/construct overhead.
				void prepare( ImagePlane<imgT>* image, SurfIntegralImageT * intImage = NULL );
				void makeDescriptor( FeaturePoint& fp ) const;

			protected:

				void performActionDerived( ViewT * view );

			private:

				void prepare( ViewT *view );

				void makeDescriptors( ViewT * view );
				void makeDescriptorPriv( FeaturePoint& fp ) const;

				bool checkBorders( FeaturePoint& fp, int factor ) const;
				bool isWithin( real_type x, real_type y ) const;
				bool checkBorders( FeaturePoint& fp, const RotMatT & Ainv, real_type factor ) const;

				void getMainOrientations( FeaturePoint& fp ) const;

				void responseXandY( int x, int y, int sz, vec2 &result ) const;
				void responseTopXandTopY( int x, int y, int sz, SurfIntegralImageT& intImage, vec2 &result ) const;

				void rotationMatrix2d( real_type angle, RotMatT &result ) const;

				void calculateDescriptors( FeaturePoint& fp, const vectorT& orients ) const;
				void calculateDescriptorSimple( FeaturePoint& fp ) const;

				bool hasTable( int max, int s ) const;
				void prepareGaussTable( uint max, int s ) const;
				real_type gauss( int x, int y, int s ) const;

				mutable std::vector< boost::shared_ptr< AMatrix > > gaussTables;
		};

	} // core
} // adore

#endif // SURFDESCRIPTOR_H
