

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_TESTASSESSOR_H
#define ADORE_TESTASSESSOR_H

#include<adrcore/types/nview.h>
#include<adrcore/types/bijectivity.h>
#include<adrcore/types/pointcorr.h>
#include<adrcore/nview/nviewoperator.h>

namespace adore
{
	namespace core
	{

		/**
		 
		*/
		class ViewErrors
		{
			public:

				ViewErrors();

				virtual ~ViewErrors();

				virtual void reset();

				virtual void computeErrors( ViewBase *v, NView &nview );

				double getReproErrorSum()
				{
					return reproErrorSum;
				}

				double getReproErrorMin()
				{
					return reproErrorMin;
				}

				double getReproErrorMax()
				{
					return reproErrorMax;
				}

				double getReproErrorAvg()
				{
					return reproErrorAvg;
				}

				double getReproErrorVar()
				{
					return reproErrorVar;
				}



				double getXErrorSum()
				{
					return xErrorSum;
				}

				double getXErrorMin()
				{
					return xErrorMin;
				}

				double getXErrorMax()
				{
					return xErrorMax;
				}

				double getXErrorAvg()
				{
					return xErrorAvg;
				}

				double getXErrorVar()
				{
					return xErrorVar;
				}



				double getYErrorSum()
				{
					return yErrorSum;
				}

				double getYErrorMin()
				{
					return yErrorMin;
				}

				double getYErrorMax()
				{
					return yErrorMax;
				}

				double getYErrorAvg()
				{
					return yErrorAvg;
				}

				double getYErrorVar()
				{
					return yErrorVar;
				}


				virtual int getSize()
				{
					return nvToDiff.getSize();
				}

				virtual void getXDifferences( vector<double> &diffs );

				virtual void getYDifferences( vector<double> &diffs );

				virtual void getReproErrors( vector<double> &diffs );

				virtual void getReproErrorsAndFeatures( vector< pair<FeaturePoint*, double> > & errs );

			protected:

				virtual void computeDiffs( ViewBase *v, NView &nview );
				virtual void computeResults();

				Bijectivity<FeaturePoint *, AVector2D *> nvToDiff;
				double reproErrorSum;
				double reproErrorMin;
				double reproErrorMax;
				double reproErrorAvg;
				double reproErrorVar;
				double xErrorSum;
				double xErrorMin;
				double xErrorMax;
				double xErrorAvg;
				double xErrorVar;
				double yErrorSum;
				double yErrorMin;
				double yErrorMax;
				double yErrorAvg;
				double yErrorVar;
		};




		/**
		 
		 */
		class Assessor:public ViewErrors, NViewOperator
		{
			public:
				Assessor();

				~Assessor();

				void reset();

				void modify( NView &nview ) {assess( nview );}

				void assess( NView &nview );

				int getSize();

				void getXDifferences( vector<double> &diffs );

				void getYDifferences( vector<double> &diffs );

				void getReproErrors( vector<double> &diffs );

				ViewErrors *getErrorsOf( ViewBase *v )
				{
					return viewToErrors.getB( v );
				}
				int getNotInFront( Camera* cam )
				{
					if ( notInFront.find( cam ) == notInFront.end() )
						return -1;
					return notInFront[cam];
				}
			protected:

				void computeErrors( ViewBase *v, NView &nview );
				void computeDiffs( ViewBase *v, NView &nview );
				void computeResults();
				map<Camera*, int> computeCamErrors( NView& nview );

				Bijectivity<ViewBase *, ViewErrors *> viewToErrors;
				map<Camera*, int> notInFront;

		};

	}

}

#endif
