

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef ADORE_CORR1TON_H
#define ADORE_CORR1TON_H

#include <adrcore/types/feature.h>
#include <adrcore/types/inorderset.h>

#include <cassert>
#include <set>
#include <map>

namespace adore
{
	namespace core
	{

		using namespace std;

		class Corr1ToN
		{

				friend class NView;

				ViewBase* view1, * view2;
				Feature* f1;
				InOrderSet<Feature*> fN;
				map<Feature*, float> f2badness;
				multimap<float, Feature*> badness2f;

				typedef map<Feature*, float>::iterator f2b_iterator;
				typedef multimap<float, Feature*>::iterator b2f_iterator;

			public:
				typedef InOrderSet<Feature*>::ConstIterator ConstIterator;


				ConstIterator getBegin() const
				{
					return fN.getBegin();
				}

				ConstIterator begin() const
				{
					return fN.getBegin();
				}

				ConstIterator getEnd() const
				{
					return fN.getEnd();
				}

				ConstIterator end() const
				{
					return fN.getEnd();
				}

				Feature* getFeature( ViewBase* v, uint idx = 0 )
				{
					if ( 0 == idx && v == view1 )
						return f1;
					assert( v == view2 );
					if ( v != view2 )
						return NULL;
					Feature* f = NULL;
					ConstIterator it = fN.getBegin();
					for ( uint i = 0; i < idx && it != fN.getEnd(); ++i )
						++it;
					if ( it != fN.getEnd() )
						f = *it;
					return f;
				}

			private:

				Corr1ToN() : view1( NULL ), view2( NULL ), f1( NULL ) {}

				void addFeature( Feature* f, float badness = 0.0 )
				{
					assert( NULL != f->getView() );
					if ( NULL == view1 )
					{
						view1 = f->getView();
						f1 = f;
						return;
					}
					if ( NULL == f->getView() )
						return;
					if ( NULL == view2 )
					{
						view2 = f->getView();
					}
					f2badness[f] = badness;
					badness2f.insert( make_pair( badness, f ) );
					updateSet();
				}

				void removeFeature( Feature * f )
				{
					ConstIterator it = fN.find( f );
					if ( it != fN.getEnd() )
					{
						f2b_iterator it2 = f2badness.find( f );
						pair<b2f_iterator,b2f_iterator> range = badness2f.equal_range( it2->second );
						f2badness.erase( it2 );
						badness2f.erase( range.first, range.second );
						if ( badness2f.empty() )
							view2 = NULL;
						updateSet();
					}
				}

				void updateSet()
				{
					fN.clear();
					b2f_iterator it = badness2f.begin();
					for ( ; it != badness2f.end(); ++it )
						fN.insert( it->second );
				}

		};

	}
}

#endif // ADORE_CORR1TON_H
