

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_TYPESINORDERSET_H
#define ADORE_TYPESINORDERSET_H

#include <list>
#include <set>
#include <algorithm>
#include <cassert>

namespace adore
{
	namespace core
	{

		using namespace std;

		/**
		 * This is a set. The special property of this set is, that an iterator to the elements will give the elements in the order of insertion.
		 *  
		*/
		template<class A>
		class InOrderSet
		{
			public:
				typedef typename std::list<A>::iterator Iterator;
				typedef typename std::list<A>::const_iterator ConstIterator;

				InOrderSet()
				{
				}

				~InOrderSet()
				{
				}

				void add( A &a )
				{
					insert( a );
				}

				void insert( A &a )
				{
					typename set<A>::iterator it = s.find( a );
					if ( it != s.end() ) return;

					s.insert( a );
					l.push_back( a );
				}

				void remove( A &a )
				{
					typename set<A>::iterator it = s.find( a );
					if ( it == s.end() ) return;
					s.erase( it );
					typename list<A>::iterator lit = std::find( l.begin(), l.end(), a );
					l.erase( lit );
				}

				/**
				 * Get an iterator to the begin of all entities.
				 * @return the iterator.
				 */
				ConstIterator getBegin() const
				{
					return l.begin();
				}

				/**
				 * Get an iterator to the end of all entities.
				 * @return the iterator.
				 */
				ConstIterator getEnd() const
				{
					return l.end();
				}

				/**
				 * Get an iterator to the begin of all entities.
				 * @return the iterator.
				 */
				Iterator getBegin()
				{
					return l.begin();
				}

				/**
				 * Get an iterator to the end of all entities.
				 * @return the iterator.
				 */
				Iterator getEnd()
				{
					return l.end();
				}

				void clear()
				{
					s.clear();
					l.clear();
				}

				/**
				 * returns the number of elements.
				 * @return the number of elements.
				 */
				int getSize() const
				{
					assert( s.size() == l.size() );
					return s.size();
				}

				Iterator find( A &a )
				{
					typename set<A>::iterator it = s.find( a );
					if ( it == s.end() )
						return l.end();
					return std::find( l.begin(), l.end(), a );
				}

				ConstIterator find( const A & a ) const
				{
					typename set<A>::const_iterator it = s.find( a );
					if ( it == s.end() )
						return l.end();
					return std::find( l.begin(), l.end(), a );
				}

			private:
				set<A> s;
				list<A> l;
		};


#ifdef WUNSCHPUNSCH

		template<class A, class Comparator = less<A> >
		class InOrderSet
		{
			public:
				typedef typename set<A, Comparator >::iterator Iterator;
				typedef typename set<A, Comparator >::const_iterator ConstIterator;

				InOrderSet()
				{
				}

				~InOrderSet()
				{
				}

				void add( A &a )
				{
					insert( a );
				}

				void insert( A &a )
				{
					Iterator it = s.find( a );
					if ( it != s.end() ) return;

					s.insert( a );
				}

				void remove( A &a )
				{
					Iterator it = s.find( a );
					if ( it == s.end() ) return;
					s.erase( it );
				}

				/**
				 * Get an iterator to the begin of all entities.
				 * @return the iterator.
				 */
				ConstIterator getBegin() const
				{
					return s.begin();
				}

				/**
				 * Get an iterator to the end of all entities.
				 * @return the iterator.
				 */
				ConstIterator getEnd() const
				{
					return s.end();
				}

				/**
				 * Get an iterator to the begin of all entities.
				 * @return the iterator.
				 */
				Iterator getBegin()
				{
					return s.begin();
				}

				/**
				 * Get an iterator to the end of all entities.
				 * @return the iterator.
				 */
				Iterator getEnd()
				{
					return s.end();
				}

				void clear()
				{
					s.clear();
				}

				/**
				 * returns the number of elements.
				 * @return the number of elements.
				 */
				int getSize() const
				{
					return s.size();
				}

				Iterator find( A &a )
				{
					return s.find( a );
				}

			private:
				set<A, Comparator > s;
		};

#endif

	}

}

#endif
