

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORENVIEWINFO_H
#define ADORE_CORENVIEWINFO_H

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		class NViewInfo
		{
			public:
				enum TYPE
				{
					NONE = 0,
					BASE,
					FMATRIX,
					TTENSOR,
					RDFMATRIX,
					PROJECTIVITY
				};
				bool okay;
			public:
				NViewInfo():okay( false ) {;}

				virtual ~NViewInfo() {;}

				virtual TYPE getType() {return BASE;}

				virtual const ATensor3 & getTTensor() const {extern ATensor3 NullTensor3; return NullTensor3;}
				virtual ATensor3 & getTTensor() {extern ATensor3 NullTensor3; return NullTensor3;}
				virtual void setTTensor( const ATensor3 & ) {;}

				virtual const AMatrix & getFMatrix() const {extern AMatrix NullMatrix; return NullMatrix;}
				virtual AMatrix & getFMatrix() {extern AMatrix NullMatrix; return NullMatrix;}
				virtual void setFMatrix( const AMatrix & ) {;}

				virtual const AMatrix & getH() const {extern AMatrix NullMatrix; return NullMatrix;}
				virtual AMatrix & getH() {extern AMatrix NullMatrix; return NullMatrix;}
				virtual void setH( const AMatrix & ) {;}

				virtual float getLambda() {return 0;}
				virtual void setLambda( const float ) {;}

				bool isOkay() const
				{
					return okay;
				}
				void setOkay ( bool theValue )
				{
					okay = theValue;
				}
		};

		class NViewInfoFM : public NViewInfo
		{
			private:
				AMatrix fmat;

			public:
				NViewInfoFM( const AMatrix & fm ):NViewInfo(), fmat( fm ) {}//OK6
				NViewInfoFM():NViewInfo(), fmat( 3, 3 ) {}

				TYPE getType() {return FMATRIX;}

				const AMatrix & getFMatrix() const {return fmat;}
				AMatrix & getFMatrix() {return fmat;}
				void setFMatrix( const AMatrix & f ) {fmat = f;}
		};

		class NViewInfoRDFM : public NViewInfo
		{
			private:
				AMatrix fmat;
				float lambda;

			public:
				NViewInfoRDFM( const AMatrix & fm ):NViewInfo(), fmat( fm ) {}
				NViewInfoRDFM():NViewInfo(), fmat( 3, 3 ) {}

				TYPE getType() {return RDFMATRIX;}

				const AMatrix & getFMatrix() const {return fmat;}
				AMatrix & getFMatrix() {return fmat;}
				void setFMatrix( const AMatrix & f ) {fmat = f;}

				void setLambda ( float theValue )
				{
					lambda = theValue;
				}

				float getLambda() const
				{
					return lambda;
				}
		};

		class NViewInfoTT : public NViewInfo
		{
			private:
				ATensor3 tt;

			public:
				NViewInfoTT( const ATensor3 & t ):NViewInfo(), tt( t ) {}//OK6
				NViewInfoTT():NViewInfo(), tt( 3, 3, 3 ) {}

				TYPE getType() {return TTENSOR;}

				const ATensor3 & getTTensor() const {return tt;}
				ATensor3 & getTTensor() {return tt;}
				void setTTensor( const ATensor3 & t ) {tt = t;}
		};

		class NViewInfoP : public NViewInfo
		{
			private:
				AMatrix H;

			public:
				NViewInfoP( const AMatrix & h ):NViewInfo(), H( h ) {}
				NViewInfoP():NViewInfo(), H( 3, 3 ) {}

				TYPE getType() {return PROJECTIVITY;}

				const AMatrix & getH() const {return H;}
				AMatrix & getH() {return H;}
				void setH( const AMatrix & h ) {H = h;}
		};



		inline NViewInfo *createNViewInfo( NViewInfo::TYPE type )
		{
			switch ( type )
			{
				case NViewInfo::PROJECTIVITY:
					return new NViewInfoP;
				case NViewInfo::FMATRIX:
					return new NViewInfoFM;
				case NViewInfo::TTENSOR:
					return new NViewInfoTT;
				case NViewInfo::RDFMATRIX:
					return new NViewInfoRDFM;
				default:
					return NULL;
			}
			return NULL;
		}

	}
}

#endif
