

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "nviewmap.h"

namespace adore
{

	namespace core
	{

		NViewMap::NViewMap() :
				viewsToSet ( "", NULL ),
				nviewResetMode ( DELETE_NVIEW ),
				master ( FORGET_VIEW & RESET_VIEW, DELETE_CORR )
		{
			DBGOUT ( DBGALL, "creating NViewMap\n" );
		}

		NViewMap::~NViewMap()
		{
			DBGOUT ( DBGALL, "deleting NViewMap\n" );
			reset();
		}

		void NViewMap::reset()
		{
			vector<NView *> desc;

			switch ( nviewResetMode )
			{
				case DELETE_NVIEW:
					getEntries ( desc );
					for ( uint i = 0; i < desc.size(); ++i ) delete desc[i];
					break;
				default:
					break;
			}

			master.reset();
			viewsToSet.clear();
		}


//
//		creation of sets
//

		NView *NViewMap::getNView ( ViewBase *v1, ViewBase *v2 )
		{
			ViewsKeyString key = getKey ( v1, v2 );
			return viewsToSet[key];
		}

		NView *NViewMap::getNView ( ViewBase *v1, ViewBase *v2, ViewBase *v3 )
		{
			ViewsKeyString key = getKey ( v1, v2, v3 );
			return viewsToSet[key];
		}

		NView *NViewMap::getNView ( ViewBase *v1, ViewBase *v2, ViewBase *v3, ViewBase *v4 )
		{
			ViewsKeyString key = getKey ( v1, v2, v3, v4 );
			return viewsToSet[key];
		}

		void NViewMap::setNView ( ViewBase *v1, ViewBase *v2, NView *nview )
		{
			ViewsKeyString key = getKey ( v1, v2 );

			if ( nview )
			{
				addViewToNView ( v1, nview );
				addViewToNView ( v2, nview );
				viewsToSet.add ( key, nview );
			}
			else
			{
				viewsToSet.remove( key );
			}
		}

		void NViewMap::setNView ( ViewBase *v1, ViewBase *v2, ViewBase *v3, NView *nview )
		{
			ViewsKeyString key = getKey ( v1, v2, v3 );

			if ( nview )
			{
				addViewToNView ( v1, nview );
				addViewToNView ( v2, nview );
				addViewToNView ( v3, nview );
				viewsToSet.add ( key, nview );
			}
			else
			{
				viewsToSet.remove( key );
			}
		}

		void NViewMap::setNView ( ViewBase *v1, ViewBase *v2, ViewBase *v3, ViewBase *v4, NView *nview )
		{
			ViewsKeyString key = getKey ( v1, v2, v3, v4 );

			if ( nview )
			{
				addViewToNView ( v1, nview );
				addViewToNView ( v2, nview );
				addViewToNView ( v3, nview );
				addViewToNView ( v4, nview );
				viewsToSet.add ( key, nview );
			}
			else
			{
				viewsToSet.remove( key );
			}
		}

//
//		getter
//


		void NViewMap::getNViewSets ( vector<NView *> &desc )
		{
			NViewIterator it = getEntriesBegin();
			for ( ; it != getEntriesEnd(); ++it )
			{
				if ( ( *it )->isNViewSet() )
				{
					desc.push_back ( *it );
				}
			}
		}

		void NViewMap::getEntries ( vector<NView *> &desc )
		{
			EntryIterator it = getEntriesBegin();
			for ( ; it != getEntriesEnd(); ++it ) desc.push_back ( *it );
		}

//		void NViewMap::getRoots ( vector<NView *> &desc )
//		{
//			set<NView *> roots;
//			EntryIterator it = getEntriesBegin();
//			for ( ; it != getEntriesEnd(); ++it )
//			{
//				NView *nv = *it;//(*it)->getParent();
//				while ( nv && nv->getParent() )
//				{
//					nv = nv->getParent();
//				}
//				if ( nv && !nv->isNViewSet() )
//				{
//					roots.insert ( nv );
//				}
//			}
//			set<NView *>::iterator sit = roots.begin();
//			for ( ; sit != roots.end(); ++sit )
//			{
//				desc.push_back ( *sit );
//			}
//		}

		NView *NViewMap::createNView ( ViewBase *v1, ViewBase *v2 )
		{
			ViewsKeyString key = getKey ( v1, v2 );

			NView *nv = viewsToSet[key];
			if ( nv == NULL )
			{
				nv = new NView();
				addViewToNView ( v1, nv );
				addViewToNView ( v2, nv );
				viewsToSet.add ( key, nv );
			}
//			return static_cast<NViewT *>( nv );
			return nv;
		}

		NView *NViewMap::createNView ( ViewBase *v1, ViewBase *v2, ViewBase *v3 )
		{
			ViewsKeyString key = getKey ( v1, v2, v3 );

			NView *nv = viewsToSet[key];
			if ( nv == NULL )
			{
				nv = new NView();
				addViewToNView ( v1, nv );
				addViewToNView ( v2, nv );
				addViewToNView ( v3, nv );
				viewsToSet.add ( key, nv );
			}
//			return static_cast<NViewT *>( nv );
			return nv;
		}

		NView *NViewMap::createNView ( ViewBase *v1, ViewBase *v2, ViewBase *v3, ViewBase *v4 )
		{
			ViewsKeyString key = getKey ( v1, v2, v3, v4 );

			NView *nv = viewsToSet[key];
			if ( nv == NULL )
			{
				nv = new NView();
				addViewToNView ( v1, nv );
				addViewToNView ( v2, nv );
				addViewToNView ( v3, nv );
				addViewToNView ( v4, nv );
				viewsToSet.add ( key, nv );
			}
//			return static_cast<NViewT *>( nv );
			return nv;
		}

	}
}
