

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWPOINTCORR_H
#define ADORE_NVIEWPOINTCORR_H

#include <adrcore/types/featurepoint.h>
#include <adrcore/util/debug.h>

#include <set>
#include <iomanip>

namespace adore
{
	namespace core
	{

		using namespace std;

		/**
		 * Essentially, a correspondence is a set of points.
		 * Additionally, this class allowes a query of the form "Point of a given View".
		 * This class is intended to be used only by NViewSet, so it's constructor is private.
		 *  
		*/
		class PointCorr
		{

				friend class NView;

			private:
				FeaturePoint dummy;//for searching purposes

				set<FeaturePoint *, FeatureViewAmbiComparator<> > corr;

				float badness;

			public:
				/**
				* Use this type to tterate over all points in the Correspondence.
				*/
				typedef set<FeaturePoint *, FeatureViewAmbiComparator<> >::iterator Iterator;
				typedef Iterator iterator;

				/**
				* Get an Iterator pointing to the begin of the corresponding points.
				* @return the iterator.
				*/
				Iterator getBegin()
				{
					return corr.begin();
				}

				Iterator begin()
				{
					return corr.begin();
				}

				/**
				* Get an Iterator pointing to the end of the corresponding points.
				* @return the iterator.
				*/
				Iterator getEnd()
				{
					return corr.end();
				}

				Iterator end()
				{
					return corr.end();
				}

				/**
				* Return the point that is associated with a given View.
				* @param v the View.
				* @return the point.
				*/
				FeaturePoint *getFeature( ViewBase *v )
				{
					if ( !v ) return NULL;
					dummy.setView( v );
					Iterator it = corr.find( &dummy );
					if ( it == corr.end() ) return NULL;
					return *it;
				}

				/**
				* returns the number of points in this Correspondence.
				*/
				uint getSize()
				{
					return corr.size();
				}

				uint size()
				{
					return corr.size();
				}

				void setBadness ( float theValue )
				{
					badness = theValue;
				}

				float getBadness() const
				{
					return badness;
				}

				std::string toDbgString()
				{
					std::ostringstream oss;
					oss << "Corr-badness: " << setw( 2 ) << badness << ", points: ";
					for ( Iterator it = begin(); it != end(); ++it )
					{
						oss << adore::core::toDbgString( *it ) << " ";
					}
					return oss.str();
				}

			private:

				/**
				* Constructor.
				* @param set the constructing NViewSet
				*/
				PointCorr():dummy(), badness( 0 ) {;}

				/**
				* Destructor.
				*/
				~PointCorr() {/*DBGOUT(DBGNORMAL, "destroying Pointcorr %p\n", this);*/}

				/**
				* Add a point to the correspondence.
				* this is only invokable by NView, because the NView has to
				* kepp track of them.
				* @param pt the point.
				* @param v the View the point was found in.
				*/
				void addFeature( FeaturePoint *f )
				{
					corr.insert( f );
				}

				void removeFeature( FeaturePoint *f )
				{
					Iterator it = corr.find( f );
					if ( it != corr.end() ) corr.erase( it );
				}

		};

	}

}

#endif
