

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_TYPESVIEWGRAPH_H
#define ADORE_TYPESVIEWGRAPH_H

#include <adrcore/types/view.h>
#include <adrcore/types/camera.h>
#include <map>
#include <boost/config.hpp>
#include <boost/graph/adjacency_list.hpp>

namespace adore
{

	namespace core
	{

		using namespace boost;

		struct ViewVertexProps
		{
			ViewBase *view;
			Camera *cam;
		};

		typedef adjacency_list<setS, listS, undirectedS, ViewVertexProps, float> Graph; //float is similarity
		typedef graph_traits<Graph>::vertex_descriptor ViewVertex;
		typedef graph_traits<Graph>::edge_descriptor ViewEdge;

		/**
		     
		*/
		class ViewGraph:public Graph
		{
			public:

				ViewVertex addVertex( ViewBase *v, Camera *cam = NULL )
				{
					assert( viewToVertex[v] == NULL );

					ViewVertexProps p;
					p.view = v;
					p.cam = cam;
					ViewVertex vv = add_vertex( p, *this );
					viewToVertex[v] = vv;
					return vv;
				}

				ViewVertex getVertex( ViewBase *v )
				{
					return viewToVertex[v];
				}

				ViewEdge addEdge( ViewVertex v1, ViewVertex v2, float similarity )
				{
					std::pair<edge_descriptor, bool> ret = add_edge( v1, v2, similarity, *this );
					return ret.first;
				}

				string toString()
				{
					stringstream ss;

					graph_traits<ViewGraph>::vertex_iterator vi, vi_end;
					for ( tie( vi, vi_end ) = vertices( *this ); vi != vi_end; ++vi )
					{
						ss << "(" << ( *this )[*vi].view << " | " << ( *this )[*vi].cam << ") - ";

						graph_traits<ViewGraph>::adjacency_iterator avi, avi_end;
						for ( tie( avi, avi_end ) = adjacent_vertices( *vi, *this ); avi != avi_end; ++avi )
						{
							ss << "(" << ( *this )[*avi].view << " | " << ( *this )[*avi].cam << ")";
						}
						ss << endl;
					}

					return ss.str();
				}

			private:

				map<ViewBase *, ViewVertex> viewToVertex;

		};

	}

}

#endif
