

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_DEBUG_H
#define ADORE_DEBUG_H

#include <cstdio>
#include <cstdlib>
#include <cfloat>
#include <string>
#include <sys/time.h>
#include <ctime>
namespace adore
{
	namespace core
	{

#define DBGALL 6
#define DBGVERBOSE 5
#define DBGNORMAL 4
#define DBGSOME 3
#define DBGSERIOUS 2
#define DBGCRITICAL 1
#define DBGNONE 0

#ifndef NDEBUG
#ifndef DBGLEVEL
#define DBGLEVEL DBGNORMAL
#endif

#define DBGLINE(level, format, ...) {if (level <= DBGLEVEL) {fprintf(stderr, "File: %s, Line: %d\n", __FILE__, __LINE__); \
    fprintf (stderr, format, ## __VA_ARGS__); fprintf(stderr, "\n");}}
#define DBGPLAIN(level, format, ...) {if (level <= DBGLEVEL) fprintf (stderr, format, ## __VA_ARGS__);}
#define DBGOUT(level, format, ...) {DBGPLAIN(level, format, ## __VA_ARGS__);}

#define COUT(level, x) {if (level <= DBGLEVEL) {cout << x;}}
#else
#define DBGOUT(level, format, ...) {((void) 0);}

#define COUT(level, x) {((void) 0);}
#endif

#define PRINT_TIME(function) {timeval t1, t2; gettimeofday(&t1, NULL); function; gettimeofday(&t2, NULL); \
fprintf(stderr, "dt: %ld\n", 1000000 * t2.tv_sec + t2.tv_usec - 1000000 * t1.tv_sec - t1.tv_usec);}

#define ADD_TIME(function, timeaccu) {timeval t1, t2; gettimeofday(&t1, NULL); function; gettimeofday(&t2, NULL); \
(timeaccu) += 1000000 * t2.tv_sec + t2.tv_usec - 1000000 * t1.tv_sec - t1.tv_usec;}

#ifndef NDEBUG
#define assert_msg(expr, format, ...) \
    {if (!(expr)) { fprintf(stderr, "Assertion (%s) failed:\n File %s, Line %d\n", #expr, __FILE__, __LINE__); \
    fprintf (stderr, format, ## __VA_ARGS__); abort(); }}
#else
#define assert_msg(expr, format, ...) {((void) 0);}
#endif

#define error(format, ...) \
    { fprintf(stderr, "ERROR in %s: ", __FILE__); fprintf (stderr, format, ## __VA_ARGS__); abort(); }

#define ADORE_UNUSED(x) (void)(x)

		/**
		 * Expects the user to press a key.
		 */
		inline void pressKey() {printf( "Press any key to continue\n" ); getchar();}


		inline void pressKey( int debugLevel )
		{
#ifndef NDEBUG
			if ( debugLevel > DBGLEVEL ) return; printf( "Press any key to continue\n" ); getchar();
#else
			ADORE_UNUSED( debugLevel );
#endif
		}

		inline void getTmpStr( char *c, int len )
		{
			for ( int n = 0; n < len; ++n ) c[n] = rand() % 26 + 'a';
			c[len]='\0';
		}

		template<class T>
		std::string toDbgString( T* obj )
		{
#ifndef NDEBUG
			if ( !obj ) return "<NULL>";
			else return obj->toDbgString();
#else
			ADORE_UNUSED( obj ); return "";
#endif
		}

	}
}

#endif
