

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "divisiondistortion.h"
#include <adrcore/util/mathutil.h>
#include <float.h>

namespace adore
{

	namespace core
	{

		template<class floaT>
		bool undistortRadial( floaT &x, floaT &y, const floaT lambda )
		{
			const floaT divisor = 1 + lambda * ( x * x + y * y );
			if ( tabs<floaT>( divisor ) <= FLT_EPSILON ) return false;

			x = x/divisor;
			y = y/divisor;

			return true;
		}

		template<class floaT>
		bool distortRadial( floaT &x, floaT &y, const floaT lambda )
		{
			if ( tabs<floaT>( lambda ) <= FLT_EPSILON ) return true;

			//pinhole radius
			const floaT pr2 = x * x + y * y;
			const floaT pr = sqrtf( pr2 );

			//angle
			const floaT alpha = atan2f( y, x );

			//distorted radius
			if ( ( lambda * pr2 ) > 0.25F ) {return false;}//no real solution
			const floaT dr =
			        ( 1.0 - tsqrt<floaT>( 1.0-4.0*lambda*pr2 ) )/( 2.0*lambda*pr );

			x = tcos<floaT>( alpha ) * dr;
			y = tsin<floaT>( alpha ) * dr;
			return true;
		}

		template bool undistortRadial<float>( float &x, float &y, const float lambda );
		template bool undistortRadial<double>( double &x, double &y, const double lambda );

		template bool distortRadial<float>( float &x, float &y, const float lambda );
		template bool distortRadial<double>( double &x, double &y, const double lambda );


	}
}
