

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "ldlt.h"
#include <cfloat>
#include <adrcore/util/debug.h>
#include <adrcore/util/printmat.h>

namespace adore
{
	namespace core
	{

		bool factorizeLDLT( const AMatrix &mat, AMatrix &L, AVector &D )
		{
			L = mat;
			if ( !factorizeLDLT( L ) ) return false;
                        for ( unsigned i = 0; i < L.rows(); ++i )
			{
				D( i ) = L( i, i );
				L( i, i ) = 1.0;
                                for ( unsigned j = i + 1; j < L.rows(); ++j )
				{
					L( i, j ) =  0;
				}
			}
			return true;
		}

		bool factorizeLDLT( AMatrix &mat )
		{
			assert( mat.rows() == mat.cols() );
			AVector v( mat.rows() - 1 );

			return factorizeLDLT( mat, v );
		}

		bool factorizeLDLT( AMatrix &mat, AVector &v )
		{
			bool success = true;
			double vj;
			int n = mat.rows();

			for ( int j = 0; j < n-1; ++j )
			{
				double subIt = 0;
				for ( int i = 0; i < j; ++i )
				{
					v( i ) = mat( j, i ) * mat( i, i );
					subIt += mat( j, i ) * v( i );
				}
				vj = mat( j, j ) - subIt;
				mat( j, j ) =  vj;
				//if(vj < FLT_EPSILON) return false;
				if ( fabs( vj ) >= FLT_EPSILON )
				{
					for ( int k = j+1; k < n; ++k )
					{
						subIt = 0;
						for ( int i = 0; i < j; ++i )
						{
							subIt += mat( k, i ) * v( i );
						}
						//printf("subit: %f\n", subIt);
						assert( fabs( vj ) > FLT_EPSILON );
						mat( k, j ) = ( mat( k, j ) - subIt ) / vj;
					}
				}
				else
				{
					success = false;
				}
			}
//			printf( "mat\n" );
//			printMat( mat );
			//j == n-1 separate
			double subIt1 = 0;
			--n;
			for ( int i = 0; i < n; ++i )
			{
				subIt1 += mat( n, i ) * mat( n, i ) * mat( i, i );
			}
			vj = mat( n, n ) - subIt1;

			mat( n, n ) =  vj;
			return success;
		}

		void forwardSubstitute( AMatrix &L, AVector &b, AVector &result, std::vector<int> &min )
		{
                        assert_msg( L.cols() == result.rows(), "(%zd != %zd)\n", L.cols(), result.rows() );
                        assert_msg( b.rows() == result.rows(), "(%zd != %zd)\n", b.rows(), result.rows() );
                        assert( min.size() == result.rows() );

			int m = 0;
                        for ( unsigned i = 0; i < result.rows(); ++i )
			{
				double sub = 0;
				m = min[i];
//				printf( "min: %d-%d(%d)\n", m, i, result.rows() ); pressKey();
                                for ( unsigned j = m; j < i; ++j )
				{
					sub += L( i, j ) * result( j );
				}
				result( i ) = b( i ) - sub;
			}
		}

		void backSubstitute( AMatrix &L, AVector &b, AVector &result, std::vector<int> &max )
		{
                        assert( max.size() == result.rows() );

			int m = result.rows();
			for ( int i = result.rows() - 1; i >= 0; --i )
			{
				double sub = 0;
				m = max[i];
//				printf( "max: %d-%d(%d)\n", i + 1, m, result.rows() ); pressKey();
				for ( int j = i + 1; j < m; ++j )
				{
					sub += L( j, i ) * result( j );
				}
				result( i ) = b( i ) - sub;
			}
		}

		void solveLDLT( AMatrix &L, AVector &D, AVector &b, AVector &result,
                                AVector &x, std::vector<int> &min, std::vector<int> &max )
		{
			assert( result.rows() == x.rows() );

			forwardSubstitute( L, b, x, min );
                        for ( unsigned i = 0; i < D.rows(); ++i )
			{
				assert( fabs( D( i ) ) > DBL_EPSILON );
				x( i ) = x( i ) / D( i );
			}
			backSubstitute( L, x, result, max );
		}

		void solveLDLTc( AMatrix &LD, AVector &b, AVector &result, AVector &x,
                                 std::vector<int> &min, std::vector<int> &max )
		{
			x( 0 ) = 0;
			result( result.rows() - 1 ) = 0;
			forwardSubstitute( LD, b, x, min );
                        for ( unsigned i = 0; i < LD.rows(); ++i )
			{
				if ( fabs( x( i ) ) > FLT_EPSILON )
				{
					assert_msg( fabs( LD( i, i ) ) > DBL_EPSILON, "%f, %f\n", x( i ), LD( i, i ) );
					x( i ) = x( i ) / LD( i, i );
				}
			}
//			printf( "solve\n" );
//			printMat( LD );
//			printMat( x );
//			pressKey();
			backSubstitute( LD, x, result, max );
		}



	}
}

