

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_UTILLDLT_H
#define ADORE_UTILLDLT_H

#include <vector>
#include <adrcore/types/aarray.h>

namespace adore
{
	namespace core
	{

		/**
			calls the function below and unpacks the result
			mat is not overwritten
		*/
		bool factorizeLDLT ( const AMatrix &mat, AMatrix &L, AVector &D );

		/**
			calls the function below, providing its own auxiliary space
		*/
		bool factorizeLDLT ( AMatrix &mat );

		/**
			replaces mat with the factorization
			v is needed as auxiliary space
		*/
		bool factorizeLDLT ( AMatrix &mat, AVector &v );

		void forwardSubstitute ( AMatrix &L, AVector &b, AMatrix &result, std::vector<int> &min );

		void backSubstitute ( AMatrix &L, AVector &b, AMatrix &result, std::vector<int> &max );

		void solveLDLT ( AMatrix &L, AVector &D, AVector &b, AVector &result,
		                 AVector &x, std::vector<int> &min, std::vector<int> &max );

		inline void solveLDLT ( AMatrix &L, AVector &D, AVector &b, AVector &result,
		                        std::vector<int> &min, std::vector<int> &max )
		{
			AVector x ( result.rows() );
			solveLDLT ( L, D, b, result, x, min, max );
		}


		void solveLDLTc ( AMatrix &LD, AVector &b, AVector &result, AVector &x,
		                  std::vector<int> &min, std::vector<int> &max );

		inline void solveLDLTc ( AMatrix &LD, AVector &b, AVector &result,
		                         std::vector<int> &min, std::vector<int> &max )
		{
			AVector x ( result.rows() );
			assert ( x.rows() > 0 );
			solveLDLTc ( LD, b, result, x, min, max );
		}





	}
}

#endif


