

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef CORE_UTIL_LINORM
#define CORE_UTIL_LINORM
namespace adore
{
	namespace core
	{


		/**
		 * calculates the coefficients necessary to map the interval of some given values
		 * onto the interval [0; range]
		 * the formula for mapping is factor * float(x - min)
		 * factor and min are calculated
		 * @return false, if degenerate
		 */
		template <class InIt>
		bool normalizeLinearCoeffs( InIt begin, InIt end,
		                            float range, float &factor, typename InIt::value_type &min )
		{
			typedef typename InIt::value_type Type;

			min = *begin;
			Type max = *begin;

			for ( InIt it = begin; it != end; ++it )
			{
				Type val = *it;

				if ( val < min )
				{
					min = val;
				}

				if ( val > max )
				{
					max = val;
				}
			}

			if ( min == max )
			{
				factor = 1.0F;
				return false;
			}
			else
			{
				factor = 1.0F;
				if ( range > 0 )
				{
					factor = range / float( max - min );
				}
			}
			return true;
		}


		/**
		 * remaps the values pointed to by the input iterators to the intervall [0;range]
		 * If range <= 0 the values are shifted only.
		 */
		template <class InIt, class OutIt>
		void normalizeLinear( InIt begin, InIt end, OutIt out, typename OutIt::value_type range = 0 )
		{
			typedef typename InIt::value_type Type;

			Type min;
			float factor;
			if ( normalizeLinearCoeffs<InIt>( begin, end, range, factor, min ) )
			{
				for ( InIt it = begin; it != end; ++it, ++out )
				{
					*out = typename OutIt::value_type( factor * float( *it - min ) );
				}
			}
			else
			{
				for ( InIt it = begin; it != end; ++it, ++out )
				{
					*out = typename OutIt::value_type( 0 );
				}
			}
		}

	}
}

#endif

