

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_UTILMATMUL2_H
#define ADORE_UTILMATMUL2_H

#include <adrcore/types/aarray.h>

namespace adore
{
	namespace core
	{

		template<class T>
		inline void matMul2 ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result, T factor )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int j = b.cols() - 1; j >= 0; --j )
				{
					result ( i, j ) = 0;
					for ( int k = a.cols() -1; k >= 0; --k )
					{
						result ( i, j ) += factor * a ( i, k ) * b ( k, j );
					}
				}
		}

		template<class T>
		inline void matMul2 ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = 0; i < (int)a.rows(); ++i )
				for ( int j = 0; j < (int)b.cols(); ++j )
				{
					result ( i, j ) = 0;
					for ( int k = 0; k < (int)a.cols(); ++k )
					{
						result ( i, j ) += a ( i, k ) * b ( k, j );
					}
				}
//			for ( int i = a.rows() - 1; i >= 0; --i )
//				for ( int j = b.cols() - 1; j >= 0; --j )
//				{
//					result ( i, j ) = 0;
//					for ( int k = a.cols() -1; k >= 0; --k )
//					{
//						result ( i, j ) += a ( i, k ) * b ( k, j );
//					}
//				}
		}

		template<class T>
		inline AArray<T, 2> matMul ( const AArray<T, 2> &a, const AArray<T, 2> &b )
		{
			AArray<T, 2> result(a.rows(), b.cols());
			matMul2(a, b, result);
			return result;
		}

		template<class T>
		inline void matMulAdd2 ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int j = b.cols() - 1; j >= 0; --j )
					for ( int k = a.cols() -1; k >= 0; --k )
					{
						result ( i, j ) += a ( i, k ) * b ( k, j );
					}
		}

		template<class T>
		inline void matMulSub2 ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int j = b.cols() - 1; j >= 0; --j )
					for ( int k = a.cols() -1; k >= 0; --k )
					{
						result ( i, j ) -= a ( i, k ) * b ( k, j );
					}
		}

		/////////////////////////////////////////////////////////////

		template<class T>
		inline void matMul2_AT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result, T factor )
		{
			for ( int i = a.cols() - 1; i >= 0; --i )
				for ( int j = b.cols() - 1; j >= 0; --j )
				{
					result ( i, j ) = 0;
					for ( int k = a.rows() -1; k >= 0; --k )
					{
						result ( i, j ) += factor * a ( k, i ) * b ( k, j );
					}
				}
		}

		template<class T>
		inline void matMul2_AT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = 0; i < (int)a.cols(); ++i )
				for ( int j = 0; j < (int)b.cols(); ++j )
				{
					result ( i, j ) = 0;
					for ( int k = 0; k < (int)a.rows(); ++k )
					{
						result ( i, j ) += a ( k, i ) * b ( k, j );
					}
				}
//			for ( int i = a.cols() - 1; i >= 0; --i )
//				for ( int j = b.cols() - 1; j >= 0; --j )
//				{
//					result ( i, j ) = 0;
//					for ( int k = a.rows() -1; k >= 0; --k )
//					{
//						result ( i, j ) += a ( k, i ) * b ( k, j );
//					}
//				}
		}

		template<class T>
		inline AArray<T, 2> matMul_AT ( const AArray<T, 2> &a, const AArray<T, 2> &b )
		{
			AArray<T, 2> result(a.cols(), b.cols());
			matMul2_AT(a, b, result);
			return result;
		}

		template<class T>
		inline void matMulAdd2_AT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.cols() - 1; i >= 0; --i )
				for ( int j = b.cols() - 1; j >= 0; --j )
					for ( int k = a.rows() -1; k >= 0; --k )
					{
						result ( i, j ) += a ( k, i ) * b ( k, j );
					}
		}

		template<class T>
		inline void matMulSub2_AT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.cols() - 1; i >= 0; --i )
				for ( int j = b.cols() - 1; j >= 0; --j )
					for ( int k = a.rows() -1; k >= 0; --k )
					{
						result ( i, j ) -= a ( k, i ) * b ( k, j );
					}
		}

		//////////////////////////////////////////

		template<class T>
		inline void matMul2_BT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result, T factor )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int j = b.rows() - 1; j >= 0; --j )
				{
					result ( i, j ) = 0;
					for ( int k = a.cols() -1; k >= 0; --k )
					{
						result ( i, j ) += factor * a ( i, k ) * b ( j, k );
					}
				}
		}

		template<class T>
		inline void matMul2_BT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int j = b.rows() - 1; j >= 0; --j )
				{
					result ( i, j ) = 0;
					for ( int k = a.cols() -1; k >= 0; --k )
					{
						result ( i, j ) += a ( i, k ) * b ( j, k );
					}
				}
		}

		template<class T>
		inline void matMulAdd2_BT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int j = b.rows() - 1; j >= 0; --j )
					for ( int k = a.cols() -1; k >= 0; --k )
					{
						result ( i, j ) += a ( i, k ) * b ( j, k );
					}
		}

		template<class T>
		inline void matMulSub2_BT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int j = b.rows() - 1; j >= 0; --j )
					for ( int k = a.cols() -1; k >= 0; --k )
					{
						result ( i, j ) -= a ( i, k ) * b ( j, k );
					}
		}

		///////////////////////////////////////////

		template<class T>
		inline void matMul2_AT_BT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result, T factor )
		{
			for ( int i = a.cols() - 1; i >= 0; --i )
				for ( int j = b.rows() - 1; j >= 0; --j )
				{
					result ( i, j ) = 0;
					for ( int k = a.rows() -1; k >= 0; --k )
					{
						result ( i, j ) += factor * a ( k, i ) * b ( j, k );
					}
				}
		}

		template<class T>
		inline void matMul2_AT_BT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.cols() - 1; i >= 0; --i )
				for ( int j = b.rows() - 1; j >= 0; --j )
				{
					result ( i, j ) = 0;
					for ( int k = a.rows() -1; k >= 0; --k )
					{
						result ( i, j ) += a ( k, i ) * b ( j, k );
					}
				}
		}

		template<class T>
		inline void matMulAdd2_AT_BT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.cols() - 1; i >= 0; --i )
				for ( int j = b.rows() - 1; j >= 0; --j )
					for ( int k = a.rows() -1; k >= 0; --k )
					{
						result ( i, j ) += a ( k, i ) * b ( j, k );
					}
		}

		template<class T>
		inline void matMulSub2_AT_BT ( const AArray<T, 2> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			for ( int i = a.cols() - 1; i >= 0; --i )
				for ( int j = b.rows() - 1; j >= 0; --j )
					for ( int k = a.rows() -1; k >= 0; --k )
					{
						result ( i, j ) -= a ( k, i ) * b ( j, k );
					}
		}

		template<class T>
		inline void matMul2_AT_A ( const AArray<T, 2> &a, AArray<T, 2> &result )
		{
			/**
			/ x x x
			/ o x x
			/ o o x
			*/
			for ( unsigned i = 0; i < a.cols(); ++i )
				for ( unsigned j = i; j < a.cols(); ++j )
				{
					result ( i, j ) = 0;
					for ( unsigned k = 0; k < a.rows(); ++k )
					{
						result ( i, j ) += a ( k, i ) * a ( k, j );
					}
				}
			/**
			/ x x x
			/ = x x
			/ = = x
			*/
			for ( unsigned i = 1; i < a.cols(); ++i )
				for ( unsigned j = 0; j < i; ++j )
					result ( i, j ) = result ( j, i );
		}

		/////////////////////////////////

		template<class T>
		inline void matMul2 ( const AArray<T, 2> &a, const AArray<T, 1> &b, AArray<T, 1> &result, T factor )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
			{
				result( i ) = 0;

				for ( int k = a.cols() -1; k >= 0; --k )
				{
					result ( i ) += factor * a ( i, k ) * b ( k );
				}
			}
		}

		template<class T>
		inline void matMul2 ( const AArray<T, 2> &a, const AArray<T, 1> &b, AArray<T, 1> &result )
		{
			for ( int i = 0; i < (int)a.rows(); ++i )
			{
				result( i ) = 0;

				for ( int k = 0; k < (int)a.cols(); ++k )
				{
					result ( i ) += a ( i, k ) * b ( k );
				}
			}
//			for ( int i = a.rows() - 1; i >= 0; --i )
//			{
//				result( i ) = 0;

//				for ( int k = a.cols() -1; k >= 0; --k )
//				{
//					result ( i ) += a ( i, k ) * b ( k );
//				}
//			}
		}

		template<class T>
		inline AArray<T, 1> matMul ( const AArray<T, 2> &a, const AArray<T, 1> &b )
		{
			AArray<T, 1> result(b.rows());
			matMul2(a, b, result);
			return result;
		}

		template<class T>
		inline void matMulAdd2 ( const AArray<T, 2> &a, const AArray<T, 1> &b, AArray<T, 1> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int k = a.cols() -1; k >= 0; --k )
				{
					result ( i ) += a ( i, k ) * b ( k );
				}
		}

		template<class T>
		inline void matMulSub2 ( const AArray<T, 2> &a, const AArray<T, 1> &b, AArray<T, 1> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int k = a.cols() -1; k >= 0; --k )
				{
					result ( i ) -= a ( i, k ) * b ( k );
				}
		}

		//////////////////////////////////////

		template<class T>
		inline void matMul2_AT ( const AArray<T, 2> &a, const AArray<T, 1> &b, AArray<T, 1> &result, T factor )
		{
			for ( int i = 0; i < (int)a.cols(); ++i )
			{
				result( i ) = 0;

				for ( int k = 0; k < (int)a.rows(); ++k )
				{
					result ( i ) += factor * a ( k, i ) * b ( k );
				}
			}
//			for ( int i = a.cols() - 1; i >= 0; --i )
//			{
//				result( i ) = 0;

//				for ( int k = a.rows() -1; k >= 0; --k )
//				{
//					result ( i ) += factor * a ( k, i ) * b ( k );
//				}
//			}
		}

		template<class T>
		inline void matMul2_AT ( const AArray<T, 2> &a, const AArray<T, 1> &b, AArray<T, 1> &result )
		{
			for ( int i = 0; i < (int)a.cols(); ++i )
			{
				result( i ) = 0;

				for ( int k = 0; k < (int)a.rows(); ++k )
				{
					result ( i ) += a ( k, i ) * b ( k );
				}
			}
//			for ( int i = a.cols() - 1; i >= 0; --i )
//			{
//				result( i ) = 0;

//				for ( int k = a.rows() -1; k >= 0; --k )
//				{
//					result ( i ) += a ( k, i ) * b ( k );
//				}
//			}
		}

		template<class T>
		inline void matMul_AT ( const AArray<T, 2> &a, const AArray<T, 1> &b )
		{
			AArray<T, 1> result(b.rows());
			matMul2_AT(a, b, result);
			return result;
		}

		template<class T>
		inline void matMulAdd2_AT ( const AArray<T, 2> &a, const AArray<T, 1> &b, AArray<T, 1> &result )
		{
			for ( int i = a.cols() - 1; i >= 0; --i )
				for ( int k = a.rows() -1; k >= 0; --k )
				{
					result ( i ) += a ( k, i ) * b ( k );
				}
		}

		template<class T>
		inline void matMulSub2_AT ( const AArray<T, 2> &a, const AArray<T, 1> &b, AArray<T, 1> &result )
		{
			for ( int i = a.cols() - 1; i >= 0; --i )
				for ( int k = a.rows() -1; k >= 0; --k )
				{
					result ( i ) -= a ( k, i ) * b ( k );
				}
		}

		/////////////////////////////////////

		template<class T>
		inline const double dot2 ( const AArray<T, 1> &a, const AArray<T, 1> &b )
		{
			double result = 0;
			for ( int k = a.rows() -1; k >= 0; --k )
			{
				result += a ( k ) * b ( k );
			}
			return result;
		}

		template<class T>
		inline const double matMul2 ( const AArray<T, 1> &a, const AArray<T, 1> &b , T factor )
		{
			return factor * dot2 ( a, b );
		}

		template<class T>
		inline const double matMul2 ( const AArray<T, 1> &a, const AArray<T, 1> &b )
		{
			return dot2 ( a, b );
		}

		///////////////////////////////////////////

		template<class T>
		inline void matMul2_BT ( const AArray<T, 1> &a, const AArray<T, 1> &b, AArray<T, 2> &result, T factor )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int k = b.rows() -1; k >= 0; --k )
				{
					result ( i, k ) = factor * a ( i ) * b ( k );
				}
		}

		template<class T>
		inline void matMul2_BT ( const AArray<T, 1> &a, const AArray<T, 1> &b, AArray<T, 2> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int k = b.rows() -1; k >= 0; --k )
				{
					result ( i, k ) = a ( i ) * b ( k );
				}
		}

		template<class T>
		inline void matMulAdd2_BT ( const AArray<T, 1> &a, const AArray<T, 1> &b, AArray<T, 2> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int k = b.rows() -1; k >= 0; --k )
				{
					result ( i, k ) += a ( i ) * b ( k );
				}
		}

		template<class T>
		inline void matMulSub2_BT ( const AArray<T, 1> &a, const AArray<T, 1> &b, AArray<T, 2> &result )
		{
			for ( int i = a.rows() - 1; i >= 0; --i )
				for ( int k = b.rows() -1; k >= 0; --k )
				{
					result ( i, k ) -= a ( i ) * b ( k );
				}
		}

		/////////////////////////////////////////////

		template<class T>
		inline void matMul2Diag_BT ( const AArray<T, 1> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			assert ( b.rows() == a.rows() );
			assert ( b.cols() == a.rows() );
			assert ( result.rows() == b.rows() );
			assert ( result.cols() == b.cols() );

			for ( int j = 0; j < b.cols(); ++j )
			{
				double d = a ( j );
				for ( int i = 0; i < b.rows(); ++i )
				{
					result ( i, j ) = d * b ( i, j );
				}
			}
		}

		template<class T>
		inline void matMul2Diag ( const AArray<T, 1> &a, const AArray<T, 2> &b, AArray<T, 2> &result )
		{
			assert ( b.rows() == a.rows() );
			assert ( b.cols() == a.rows() );
			assert ( result.rows() == b.rows() );
			assert ( result.cols() == b.cols() );

                        for ( unsigned i = 0; i < b.rows(); ++i )
			{
				double d = a ( i );
                                for ( unsigned j = 0; j < b.cols(); ++j )
				{
					result ( i, j ) = d * b ( i, j );
				}
			}
		}

		/**
		 * a is a the diagonal of a matrix.
		 * this essentially multiplies 2 vectors element by element
		 * */
		template<class T>
		inline void matMul2Diag ( const AArray<T, 1> &a, const AArray<T, 1> &b, AArray<T, 1> &result )
		{
			assert ( b.rows() == a.rows() );
			for ( int j = 0; j < b.rows(); ++j )
			{
				result ( j ) = a ( j ) * b ( j );
			}
		}

		template<class T>
		inline void matAddDiag ( AArray<T, 2> &a, const AArray<T, 1> &b )
		{
			assert ( b.rows() == a.rows() );
			for ( int j = 0; j < b.rows(); ++j )
			{
				a ( j, j ) += b ( j );
			}
		}

		template<class T>
		inline void matAddDiag ( AArray<T, 2> &a, const double b )
		{
			for ( unsigned j = 0; j < a.rows(); ++j )
			{
				a ( j, j ) += b;
			}
		}

		template<class T, int rank>
		void sub2 ( const AArray<T, rank> &arr1, const AArray<T, rank> &arr2, AArray<T, rank> &result )
		{
			typename AArray<T, rank>::const_iterator arr1It = arr1.begin();
			typename AArray<T, rank>::const_iterator arr2It = arr2.begin();
			typename AArray<T, rank>::iterator outIt = result.begin();
			for ( ; outIt != result.end(); ++outIt, ++arr1It, ++arr2It )
			{
				*outIt = *arr1It - *arr2It;
			}
		}

		//arr1 += arr2
		template<class T, int rank>
		void add2 ( AArray<T, rank> &arr1, const AArray<T, rank> &arr2 )
		{
			typename AArray<T, rank>::iterator arr1It = arr1.begin();
			typename AArray<T, rank>::const_iterator arr2It = arr2.begin();
			for ( ; arr1It != arr1.end(); ++arr1It, ++arr2It )
			{
				*arr1It += *arr2It;
			}
		}

		template<class T, int rank>
		void add2 ( const AArray<T, rank> &arr1, const AArray<T, rank> &arr2, AArray<T, rank> &result )
		{
			typename AArray<T, rank>::const_iterator arr1It = arr1.begin();
			typename AArray<T, rank>::const_iterator arr2It = arr2.begin();
			typename AArray<T, rank>::iterator outIt = result.begin();
			for ( ; outIt != result.end(); ++outIt, ++arr1It, ++arr2It )
			{
				*outIt = *arr1It + *arr2It;
			}
		}

		template<class T, int rank>
		void mul2 ( const AArray<T, rank> &arr1, const AArray<T, rank> &arr2, AArray<T, rank> &result )
		{
			typename AArray<T, rank>::const_iterator arr1It = arr1.begin();
			typename AArray<T, rank>::const_iterator arr2It = arr2.begin();
			typename AArray<T, rank>::iterator outIt = result.begin();
			for ( ; outIt != result.end(); ++outIt, ++arr1It, ++arr2It )
			{
				*outIt = *arr1It * *arr2It;
			}
		}

		template<class T, int rank>
		void div2 ( const AArray<T, rank> &arr1, const AArray<T, rank> &arr2, AArray<T, rank> &result )
		{
			typename AArray<T, rank>::const_iterator arr1It = arr1.begin();
			typename AArray<T, rank>::const_iterator arr2It = arr2.begin();
			typename AArray<T, rank>::iterator outIt = result.begin();
			for ( ; outIt != result.end(); ++outIt, ++arr1It, ++arr2It )
			{
				*outIt = *arr1It / *arr2It;
			}
		}

		template<class T, int rank>
		void saxpy2 ( const double factor, const AArray<T, rank> &arr1, const AArray<T, rank> &arr2, AArray<T, rank> &result )
		{
			typename AArray<T, rank>::const_iterator arr1It = arr1.begin();
			typename AArray<T, rank>::const_iterator arr2It = arr2.begin();
			typename AArray<T, rank>::iterator outIt = result.begin();
			for ( ; outIt != result.end(); ++outIt, ++arr1It, ++arr2It )
			{
				*outIt = factor * *arr1It + *arr2It;
			}
		}

		/**
		 * interprets vector vec as a skew symmetric matrix
		 */
		template<class T>
		void matMul2Skew( const AArray<T, 1> &vec, const AArray<T, 2> &mat, AArray<T, 2> &result )
		{
			for ( int col = mat.cols() - 1; col >= 0; --col )
			{
				result( 0, col ) = vec( 1 ) * mat( 2, col ) - vec( 2 ) * mat( 1, col );
				result( 1, col ) = vec( 2 ) * mat( 0, col ) - vec( 0 ) * mat( 2, col );
				result( 2, col ) = vec( 0 ) * mat( 1, col ) - vec( 1 ) * mat( 0, col );
			}
		}

	}
}

#endif
