

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORENUMERICALANALYSIS_H
#define ADORE_CORENUMERICALANALYSIS_H

#include "solvesvd.h"
#include <adrcore/util/debug.h>

//#define DBGCOND DBGNORMAL
#define DBGCOND DBGALL

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		class ConditionInfo
		{
			public:
				float number; // the condition number
				int rank; //the assumed rank

			public:
				ConditionInfo( float cond, int r ):number( cond ), rank( r ) {;}
		};

		/**
		 * computes the condition number of a matrix using the 2-norm
		 * That means, the ratio of the largest singular value to the smallest one ist returned.
		 */
		template<class T>
		ConditionInfo getCondition ( const AArray<T, 2> &m, double thresh = FLT_EPSILON )
		{
			SVDStorage<T> store ( m );
			svd ( m, store );
			int minSValIdx = store.D.rows() - 1;

			//if matrix does not have full rank, search for a sane minimum singular value
			for ( ; minSValIdx >= 0; --minSValIdx )
			{
				if ( store.D ( minSValIdx ) > thresh ) { break; }
			}

			return ConditionInfo( store.D ( 0 ) / store.D ( minSValIdx ), minSValIdx + 1 );
		}

		template<class T>
		void printCondition( int debugLevel, const AArray<T, 2> &m, double thresh = FLT_EPSILON )
		{
#ifndef NDEBUG
			if ( debugLevel > DBGLEVEL ) return;
			ConditionInfo info = getCondition ( m, thresh );
                        printf( "Matrix(%zd, %zd) hat rank %d and condition number %.4f\n", m.rows(), m.cols(), info.rank, info.number );
#else
			ADORE_UNUSED( debugLevel );
			ADORE_UNUSED( m );
			ADORE_UNUSED( thresh );
#endif
		}

	}

}

#endif
