

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORE_RECOMMENDATIONS_H
#define ADORE_CORE_RECOMMENDATIONS_H

#include <adrcore/util/debug.h>
#include <stdint.h>

namespace adore
{
	namespace core
	{

		////
		//// forward declaratinos (don't include all the headers)
		////

		template <class which> class DiffColorDescrAt;
		template <class which> class SnapDescrAt;
		class HarrisFinder;
		class ScaleHarrisFinder;
		class SurfFinder;
		class DiffColorDescriptor;
		class SiftDescriptor;
		class SnapDescriptor;

		////
		//// Descriptor part
		////
		/**
		 * returns a good squared threshold value for matching descriptors
		 * generated by "what"
		 */
		template <class what> float getGoodDiffThreshSqr() {error( "no recommendation for this\n" );}
		template <> inline float getGoodDiffThreshSqr< DiffColorDescrAt<uint8_t> >() {return 60000.0F;}
		template <> inline float getGoodDiffThreshSqr< DiffColorDescriptor >() {return 60000.0F;}
		template <> inline float getGoodDiffThreshSqr< SnapDescrAt<uint8_t> >() {return 0.1F;}
		template <> inline float getGoodDiffThreshSqr< SnapDescriptor >() {return 0.1F;}
		template <> inline float getGoodDiffThreshSqr< SiftDescriptor >() {return 0.3F;}

		////
		//// Finder part
		////
		/**
		 * returns a good minimum strength threshold value for feature creation
		 * done by "what"
		 */
		template <class what> float getGoodMinStrength() {error( "no recommendation for this\n" );}
		template <> inline float getGoodMinStrength<ScaleHarrisFinder>() {return 500.0F;}
		template <> inline float getGoodMinStrength<HarrisFinder>() {return 300.0F;}
		template <> inline float getGoodMinStrength<SurfFinder>() {return 2.0F;}

		template <class what> float getGoodMinFraction() {error( "no recommendation for this\n" );}
		template <> inline float getGoodMinFraction<HarrisFinder>() {return 0.00001F;}
		
		////
		//// Corner features, dependent on image type, therefore separate
		////
		template<class T> struct CFDefaults
		{
			static const T minThresh = 1;
		};

		template<> struct CFDefaults<float>
		{
			static const float minThresh = 0.001;
		};

		template<> struct CFDefaults<int>
		{
			static const float minThresh = 5;
		};
		
		////
		//// Extrema features, dependent on image type, therefore separate
		////
		template<class T> struct EFDefaults
		{
			static const T minThresh = 1;
		};

		template<> struct EFDefaults<float>
		{
			static const float minThresh = 0.001;
		};

		template<> struct EFDefaults<int>
		{
			static const float minThresh = 5;
		};


	}
}
#endif

