

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_UTILTENSORTRAFO_H
#define ADORE_UTILTENSORTRAFO_H

#include <adrcore/util/mathutil.h>
#include <adrcore/util/floatshift.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/

		/**
		 * Last Step of Algo 16.1 (p. 394)
		 * Denormalizes a Tensor that was computed with data normalized by transformatin matrices A, B, C.
		 * The matrices are expected to be upper triangle.
		 * @param tensor the Tensor.
		 * @param A transformation matrix of the points in the first View
		 * @param Binv inverse transformatin matrix of the points in the second View
		 * @param Cinv inverse transformatin matrix of the points in the third View
		 * @param destTensor the result is stored in this tensor.
		     */
		ATensor3 transformT( const ATensor3 & tensor, const AMatrix & A, const AMatrix & Binv, const AMatrix & Cinv );

		/**
		 * Last Step of Algo 16.1 (p. 394)
		 * Denormalizes a Tensor that was computed with data normalized by transformatin matrices A, B, C.
		 * The matrices are expected to be upper triangle.
		 * @param tensor the Tensor. The result is copied back, so the tensor gets altered.
		 * @param A transformation matrix of the points in the first View
		 * @param Binv inverse transformatin matrix of the points in the second View
		 * @param Cinv inverse transformatin matrix of the points in the third View
		 */
		void transform( ATensor3 & tensor, const AMatrix & A, const AMatrix & Binv, const AMatrix & Cinv );
		/**
		 * Last Step of Algo 16.1 (p. 394)
		 * Denormalizes a Tensor that was computed with data normalized by transformatin matrices A, B, C.
		 * The matrices are expected to be upper triangle.
		 * @param lineTensor the Tensor (27-vector).
		 * @param A transformation matrix of the points in the first View
		 * @param Binv inverse transformatin matrix of the points in the second View
		 * @param Cinv inverse transformatin matrix of the points in the third View
		 * @param destTensor the result is stored in this tensor.
		 */
		ATensor3 transformT( const AVector & lineTensor, const AMatrix & A, const AMatrix & Binv, const AMatrix & Cinv );
		/**
		 * Last Step of Algo 16.1 (p. 394)
		 * Denormalizes a Tensor that was computed with data normalized by transformatin matrices A, B, C.
		 * The matrices are expected to be upper triangle.
		 * @param lineTensor the Tensor (27-vector).
		 * @param A transformation matrix of the points in the first View
		 * @param Binv inverse transformatin matrix of the points in the second View
		 * @param Cinv inverse transformatin matrix of the points in the third View
		 * @return destLineTensor the result is stored in this tensor (27-vector).
		 */
		AVector transformV( const AVector & lineTensor, const AMatrix & A, const AMatrix & Binv, const AMatrix & Cinv );

		inline void transformFM( AMatrix & fmat,
								 const AMatrix & trafo1,
								 const AMatrix & trafo2 )
		{
                    AMatrix mat(3, 3);
                    matMul2( fmat, trafo1, mat );
                    matMul2_AT( trafo2, mat, fmat );
                    normalizeExponents( fmat.begin(), fmat.end() );
		}

		inline void transformH( AMatrix & H,
								const AMatrix & trafo1,
								const AMatrix & trafo2 )
		{
                    AMatrix mat(3, 3);
                    matMul2( H, trafo1, mat );
                    matMul2( trafo2, mat, H );
                    normalizeExponents( H.begin(), H.end() );
		}

	}

}

#endif
