

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "cannyfinder.h"
#include <adrcore/imgprocessing/dizenzogradients.h>
#include <adrcore/imgprocessing/canny.h>
#include <adrcore/imgprocessing/orientationandmagnitude.h>
#include <adrcore/imgprocessing/kernelextremum_3_3.h>
#include <adrcore/imgprocessing/convolver2d.h>
#include <adrcore/imgprocessing/rgbtogray.h>
#include <adrcore/features/cornerfeatures.h>
#include <memory>

namespace adore
{

	namespace core
	{

		CannyFinder::CannyFinder( uint ID, uint cHood, float mindist, ouT minstrength,float minfraction, int maxpoints ):
				fid( ID ),
				minDist( mindist ),
				minFraction( minfraction ),
				minStrength( minstrength ),
				maxPoints( maxpoints ),
				cannyHood( cHood )
		{;}

		CannyFinder::CannyFinder( const CannyFinder &cf ):
				fid( cf.fid ),
				minDist( cf.minDist ),
				minFraction( cf.minFraction ),
				minStrength( cf.minStrength ),
				maxPoints( cf.maxPoints ),
				cannyHood( cf.cannyHood )
		{
		}

		unsigned int CannyFinder::findFeaturesDerived( ViewT *v )
		{
			vector<FeaturePoint *> pts;
			ImagePlane<imgT> *img = v->getImage();
			assert( img );

			ImagePlane<imgT> *gray=NULL;
			if ( img->getNumChannels()==3 )
			{
				gray = new ImagePlane<imgT>( ( int ) round( v->getWidth() ), ( int ) round( v->getHeight() ), 1 );
				RGBtoGray<imgT> converter;
				Channel<imgT>::Set in( true );
				img->createChannels( in );
				converter.process( in,gray );
				img=gray;
			}
			auto_ptr< ImagePlane<imgT> > grayDeletor( gray ); //does nothing if gray is NULL



			calcDizenzo( *img );

			//canny part

			OrientationAndMagnitude<ouT, uint8_t, ouT> OM;
			OM.setThresh( 2 );
			mag.init( gxx.getWidth(), gxx.getHeight(), 1 );
			sec.init( gxx.getWidth(), gxx.getHeight(), 1 );
			OM.sm( &gxx, &gyy, &gxy, &sec, &mag );

			cImg.init( gxx.getWidth(), gxx.getHeight(), 1 );
			Canny<ouT, uint8_t> canny;
			canny.setMin( 5 ); //integer
			canny.setMax( 20 ); //integer
			canny.process( &mag, &sec, &cImg );
			cImg.save( "trackerharris_canny1.png" );

			//dilate canny image
			typedef KernelExtremum_3_3<uint8_t, std::greater<uint8_t> > Kernel;
			Convolver2D<Kernel> convolver( BORDER_ZERO );
			convolver.process( &cImg, &cImg );
			cImg.save( "trackerharris_canny2.png" );

			CornerFeatures<uint8_t, ouT> corners( fid, minDist, 10, minFraction, maxPoints,
			                                      /*hStrength*/ minStrength, /*needed2nd*/ 12,
			                                      /*hood*/ cannyHood, /*numAngleBins*/ 12 );
			corners.setGradImages( &gxx, &gyy, &gxy );
			corners.getFeatures( cImg, pts );
			for ( uint i = 0; i < pts.size(); ++i )
			{
				pts[i]->setView( v );
				if ( v->pick( *pts[i] ) )
				{
					cout<<"closest point x and y dist:";
					cout<<( *pts[i] )( 0 )-( *v->pick( *pts[i] ) )( 0 )<<" ";
					cout<<( *pts[i] )( 1 )-( *v->pick( *pts[i] ) )( 1 )<<endl;
					pts[i]=v->pick( *pts[i] );
				}
				else
					v->add( pts[i] );
			}
			return pts.size();
		}

		void CannyFinder::calcDizenzo( ImagePlane<imgT> &img )
		{
			//re-initialize g??
			gxx.init( img.getWidth(), img.getHeight(), 1 );
			gyy.init( img.getWidth(), img.getHeight(), 1 );
			gxy.init( img.getWidth(), img.getHeight(), 1 );

			UsingChannelMode cm( &img );//don't forget to restore the original value

			DiZenzoGradients<imgT, ouT> grad;

			ChannelSet<imgT> gradInput( img, true );

			ChannelSet<ouT> out;
			out.add( &gxx );
			out.add( &gyy );
			out.add( &gxy );

			grad.process( gradInput, out );
		}

	}
}//ns
