

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREEPILINEMATCHER2_H
#define ADORE_COREEPILINEMATCHER2_H

#include <adrcore/search/simplekdtree.h>
#include <adrcore/search/quadtree.h>

#include <adrcore/viewmatch/matcher.h>
#include <adrcore/types/featuredescriptor.h>
#include <adrcore/nview/fmatrixcalculator.h>
#include <adrcore/util/matmul2.h>

namespace adore
{

	namespace core
	{
		/**
		 * uses only the points in a distance around an epipolar line for matching
			 
		*/
		template <typename Tree, typename Assessor>
		class EpiLineMatcher2 : public Matcher
		{
			private:

				typedef FeatureDescriptor::DescVecT DescVecT;
				typedef std::pair<FeaturePoint *, float> PairT;

			private:

				SVDStoraged store3;
				AVector epi2;
				AMatrix fmat;
				float lineDistThresh;
				float featDistThresh;
				float ratioThresh;
				float countThresh; //theshold on number of corrs
				Tree tree;

			public:
				EpiLineMatcher2 ( float lineDistThreshold, float featDistThreshold ) :
						store3 ( 3, 3, SVD_VT ),
						epi2 ( 3 ),
						lineDistThresh ( lineDistThreshold ),
						featDistThresh ( featDistThreshold ),
						ratioThresh ( 0.7 ),
						countThresh ( INT_MAX )
				{setNullArray ( fmat );}

                EpiLineMatcher2 ( const AMatrix & fmatrix, float lineDistThreshold, float featDistThreshold ) :
						store3 ( 3, 3, SVD_VT ),
						epi2 ( 3 ),
						fmat ( fmatrix ),
						lineDistThresh ( lineDistThreshold ),
						featDistThresh ( featDistThreshold ),
						ratioThresh ( 0.7 ),
						countThresh ( INT_MAX )
				{;}

				EpiLineMatcher2 ( const EpiLineMatcher2 &elm ) :
						store3 ( 3, 3, SVD_VT ),
						epi2 ( 3 ),
                        fmat ( elm.fmat ),
						lineDistThresh ( elm.lineDistThresh ),
						featDistThresh ( elm.featDistThresh ),
						ratioThresh ( elm.ratioThresh ),
						countThresh ( elm.countThresh )
				{
				}

				virtual NViewOperator* clone() {return new EpiLineMatcher2 ( *this );}
				/**
					may multiply fmat by -1 to re-orient it
				*/
				void match ( ViewBase *v1, ViewBase *v2, NView *set, AMatrix &fmat );

				void match ( ViewBase *v1, ViewBase *v2, NView *set );

				void setCountThresh ( float theValue )
				{
					countThresh = theValue;
				}

				float getCountThresh() const
				{
					return countThresh;
				}

				void setLineDistThresh ( float theValue )
				{
					lineDistThresh = theValue;
				}


				float getLineDistThresh() const
				{
					return lineDistThresh;
				}

				void setFeatDistThresh ( float theValue )
				{
					featDistThresh = theValue;
				}


				float getFeatDistThresh() const
				{
					return featDistThresh;
				}



			private:

				PairT findSuitable ( FeaturePoint &fp, ViewBase *v2, const AMatrix &fmat, const AVector &epi2, Assessor& assessor );

				float getDistance ( DescVecT &v1, DescVecT &v2 )
				{
					DescVecT sub = v1 - v2;
					return dot ( sub, sub );
				}

			protected:
				void matchDerived ( ViewT *v1, ViewT *v2, NView *set )
				{match ( static_cast<ViewBase *> ( v1 ), static_cast<ViewBase *> ( v2 ), set );}

		};

		typedef SimpleKDTree<AVector, FeaturePoint, 2> KDTree2D;
		typedef EpiLineMatcher2<KDTree2D, KDTreeAssessorLine2d<KDTree2D> > EpiLineMatcherKDTree;

		typedef EpiLineMatcher2<QuadTree, QuadTreeLineAssessor> EpiLineMatcherQuadTree;
	}

}

#endif
