

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef ADORE_CORE_LINESEGMENTFINDER_H
#define ADORE_CORE_LINESEGMENTFINDER_H

#include <adrcore/viewmatch/featurefinder.h>
#include <adrcore/surf/integralimage.h>

#include <memory>
#include <vector>

#include <boost/shared_ptr.hpp>

namespace adore
{
	namespace core
	{
		class LineSegmentFinder : public FeatureFinder
		{
				typedef unsigned int uint;
				typedef IntegralImage<int> IImageT;
				typedef ImagePlane<double> ImageHaarT;
				typedef std::pair<boost::shared_ptr<ImageHaarT>, boost::shared_ptr<ImageHaarT> > haarImagesT;
				typedef std::pair<AVector, AVector> segmentT;

				class EdgeCell
				{
						int dx, dy;
						void setup( int dir )
						{
							switch ( dir )
							{
								case 0: dx++; break;
								case 1: dx++; dy++; break;
								case 2: dy++; break;
								case 3: dx--; dy++; break;
								case 4: dx--; break;
								case 5: dx--; dy--; break;
								case 6: dy--; break;
								case 7: dx++; dy--; break;
								default: break;
							}
						}
						explicit EdgeCell( int x, int y ) : dx( x ), dy( y ) { }
					public:
						// angle: -PI..PI
						explicit EdgeCell( double angle ) :
								dx( 0 ), dy( 0 )
						{
							double a = fmod( angle, M_PI ); //-PI..PI
							a /= M_PI * 0.25; //-4..4
							int dir = round( a );
							if ( 0 > dir ) dir += 8;
							setup( dir );
						}
						void next( int& x, int& y ) { x += dx; y += dy; }
						void nextOrtho( int& x, int& y )
						{
							x -= dy; y += dx;
						}
						void left( int& x, int& y )
						{
							int rx = dx - dy;
							int ry = dx + dy;
							if ( abs( rx ) > 1 ) rx /= 2;
							if ( abs( ry ) > 1 ) ry /= 2;
							x += rx;
							y += ry;
						}
						void right( int& x, int& y )
						{
							int rx = dx + dy;
							int ry = dy - dx;
							if ( abs( rx ) > 1 ) rx /= 2;
							if ( abs( ry ) > 1 ) ry /= 2;
							x += rx;
							y += ry;
						}
						bool operator==( const EdgeCell& rhs ) const
						{
							return dx == rhs.dx && dy == rhs.dy;
						}
						EdgeCell operator-() const
						{
							return EdgeCell( -dx, -dy );
						}
				};

				unsigned int findFeaturesDerived( ViewT * v );

				bool initImages( ViewT* v );
				haarImagesT getHaarImages();
				template <typename T> AArray< T, 1 > responseXandY( int x, int y );
				void suppress( haarImagesT& imgs );
				std::vector<segmentT > find( haarImagesT& imgs );
				bool nextCell( int& x, int& y, EdgeCell& ec, double orientation, double min, haarImagesT & imgs );
				double filteredOrientation( int x, int y, double orientation, double minDiff, double min, const haarImagesT & imgs ) const;
				std::vector< std::pair<size_t, size_t> > filterChain( const std::vector<std::pair<int, int> >& chain, size_t first, size_t last ) const;
				bool approximateLine( const std::vector<std::pair<int, int> >& chain, size_t first, size_t last, AVector& p1, AVector& p2 ) const;

				uint featureID;
				ImagePlane<imgT>* workImage;
				std::auto_ptr< IImageT > workIntImage;
				std::auto_ptr< ImagePlane<imgT> > grayDeletor;
				int width, height;
				double minSegmentLength;
				double pointLineMaxDist;
				int filterSize;
				double filterMax, filterMin;

				LineSegmentFinder( const LineSegmentFinder& rhs );
			public:
				LineSegmentFinder( double minLength = 15.0, double maxDist = 1.45, int filterSz = 3, int typeID = 0 );
				FeatureFinder* clone();
		};
	}
}

#endif // ADORE_CORE_LINESEGMENTFINDER_H
