

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_MATCHERTNG_H
#define ADORE_MATCHERTNG_H

#include "viewknitter.h"
#include <adrcore/types/featurepoint.h>
#include <adrcore/types/nviewmap.h>
#include <adrcore/nview/nviewoperator.h>
#include <adrcore/viewmatch/matchsorter.h>

namespace adore
{
	namespace core
	{

		class Matcher : public NViewOperator
		{
			protected:
				typedef uint8_t imgT;
				typedef View<imgT> ViewT;
				uint matchID;
				uint descrID;

			public:
				virtual ~Matcher() {;}
				Matcher() {matchID=0;descrID=0;}

				Matcher( const Matcher &m ):matchID( m.matchID ), descrID( m.descrID ) {}

				virtual NViewOperator* clone() {error( "missing implementation\n" );}

				void match ( ViewT *v1, ViewT *v2, NView *set )
				{
					matchDerived ( v1, v2, set );
				}

				/**
				 * takes all 2-tuples from knitter and matches those
				 */
				void match ( NView &nview, const ViewKnitter &knit )
				{
					for ( ViewKnitter::const_iterator it = knit.begin ( 2 ); it != knit.end ( 2 ); ++it )
					{
						ViewT *v1 = dynamic_cast<ViewT *> ( ( * ( *it ).get<2>() ) [0] );
						ViewT *v2 = dynamic_cast<ViewT *> ( ( * ( *it ).get<2>() ) [1] );
						match ( v1, v2, &nview );
					}
				}

				void match ( NViewMap &map, const ViewKnitter &knit )
				{
					match ( map.getMasterNView(), knit );
				}

				/**
				 * Takes all NViews with 2 views and matches those
				 */
				void match ( NViewMap &map )
				{
					for ( NViewMap::EntryIterator eit = map.getEntriesBegin(); eit != map.getEntriesEnd(); ++eit )
					{
						modify( **eit );
					}
				}

				void match ( NViewMap &map, FurtherMatches &wastebin )
				{
					for ( NViewMap::EntryIterator eit = map.getEntriesBegin(); eit != map.getEntriesEnd(); ++eit )
					{
						MatchSorter *ms = wastebin.createBin( *eit );
						assert( ms );
						setWasteBin( ms );
						modify( **eit );
					}
				}

				/**
				 * sets the feature type which has to be matched
				 */
                void setFeatureIDToMatch ( uint ID = 0 ) {matchID=ID;}


				/**
				 * sets the descriptor type which has to be matched
				 */
				void setDescriptorIDToMatch ( uint ID = 0 ) {descrID=ID;}

				/**
				 * adds a pair of features as a correspondence to an nview.
				 * unlike the addPointCorr method of nview, in which the last
				 * added correspondence wins whenever to points compete for the same View,
				 * this method ensures, the first one wins.
				 * However, only the second (doubtful) point is tested.
				 * In short: if you have
				 * certain <-> doubtful1
				 * certain <-> doubtful2
				 * certain <-> doubtful3
				 * use this to ensure that doubtful1 wins.
				 *
				 * returns true if it actually added a new correspondence
				 */
				bool addPairToNview ( FeaturePoint *certain, FeaturePoint *doubtful, NView *set, float badness = 0.0F )
				{
					FeaturePoint *feats[2] = {certain, doubtful};

//					PointCorr *doubtCorr = set->getPointCorr( doubtful );

					PointCorr *corr = set->getPointCorr ( certain );
					if ( corr == NULL )
					{
						set->addPointCorr ( &feats[0], &feats[2], badness );
						return true;
					}
					else
					{
//						assert( !doubtCorr || doubtCorr == corr );

						FeaturePoint *isKnown = corr->getFeature ( doubtful->getView() );
						if ( isKnown == NULL )
						{
							PointCorr *destcorr = set->addPointCorr ( &feats[0], &feats[2], badness );
							assert ( !destcorr || destcorr == corr );
							return ( destcorr != NULL );
						}
					}
					return false;
				}

				void modify( NView &nview )
				{
					ViewT *v1 = dynamic_cast<ViewT *> ( *nview.getViewsBegin() );
					ViewT *v2 = dynamic_cast<ViewT *> ( *++nview.getViewsBegin() );
					match ( v1, v2, &nview );
				}

			protected:

				virtual void matchDerived ( ViewT *v1, ViewT *v2, NView *set ) = 0;
				virtual void setWasteBin( MatchSorter * ) {;}
		};

	}
}//ns

#endif
