

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREMATCHSORTER_H
#define ADORE_COREMATCHSORTER_H

#include <adrcore/types/featurepoint.h>


namespace adore
{

	namespace core
	{

		/**
		 * An auxiliary class that helps sorting matches
			 
		*/
		class MatchSorter
		{
			public:

				class Match
				{
					public:
						FeaturePoint *fp1;
						FeaturePoint *fp2;
						float badness;
				};

			private:
				class MatchComparator
				{
					public:
						bool operator()( const Match &a, const Match &b ) const
						{
							return ( a.badness < b.badness );
						}
				};

				typedef vector<Match> VectorT;

			public:

				typedef vector<Match>::iterator iterator;

			public:
				MatchSorter( int reserve = 500 )
				{
					matches.reserve( reserve );
				}

				~MatchSorter() {}

				void reset()
				{
					matches.clear();
				}

				void add( FeaturePoint *fp1, FeaturePoint *fp2, float badness )
				{
					Match m;
					m.fp1 = fp1;
					m.fp2 = fp2;
					m.badness = badness;
					matches.push_back( m );
				}

				void sort()
				{
					std::sort( matches.begin(), matches.end(), comp );
				}

				iterator begin()
				{
					return matches.begin();
				}

				iterator end()
				{
					return matches.end();
				}

				int size()
				{
					return matches.size();
				}

			private:

				vector<Match> matches;
				MatchComparator comp;
		};

		/**
		 * a map from NView to Matchsorter
		 * this is also used as the actual storage
		 */
		class FurtherMatches
		{
			public:
				typedef std::map<NView *, MatchSorter> Bins;
				typedef Bins::iterator iterator;

			private:
				Bins bins;

			public:

				MatchSorter *createBin( NView* nv )
				{
					return &bins[nv];
				}

				MatchSorter *getBin( NView* nv )
				{
					iterator it = bins.find( nv );
					if ( it == bins.end() ) return NULL;
					return &( *it ).second;
				}
		};

	}

}

#endif
