

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_PARALLELFEATUREFINDER_H
#define ADORE_PARALLELFEATUREFINDER_H


#include <adrcore/viewmatch/featurefinder.h>
#include <adrcore/util/thread.h>

namespace adore
{
	namespace core
	{
		class ParallelFeatureFinder
		{
			protected:
				typedef uint8_t imgT;
				typedef View<imgT> ViewT;

			private:
				class FFJob : public Job
				{
					private:
						FeatureFinder *ff;
						ViewT *view;

					public:
						FFJob():Job(), ff( NULL ),view( NULL ) {}

						void setFeatureFinder( FeatureFinder *f )
						{
							ff = f;
						}

						FeatureFinder *getFeatureFinder()
						{
							return ff;
						}

						void setView( ViewT *v )
						{
							view = v;
						}

					protected:
						//overloaded
						void execute()
						{
							printf( "job finds features\n" );
							assert( ff );
							assert( view );
							ff->findFeatures( view );
						}
				};

				ThreadPool pool;
				vector<FFJob> jobs;

			public:

				ParallelFeatureFinder( FeatureFinder *finder, uint nThreads ):pool( nThreads ), jobs( nThreads )
				{
					assert( pool.getNumThreads() == nThreads );

					DBGOUT( DBGNORMAL, "ParallelFeatureFinder: creating %d threads\n", nThreads );

					jobs[0].setFeatureFinder( finder );
					pool.getThread( 0 )->setJob( &jobs[0] );
					for ( uint i = 1; i < nThreads; ++i )
					{
						jobs[i].setFeatureFinder( finder->clone() );
						pool.getThread( i )->setJob( &jobs[i] );
					}
				}

				~ParallelFeatureFinder()
				{
					for ( uint i = 1; i < jobs.size(); ++i )
					{
						delete jobs[i].getFeatureFinder();
						jobs[i].setFeatureFinder( NULL );
					}
				}


				template<class ViewIterator>
				void findFeatures ( ViewIterator vit, int numViews )
				{
//					printf( "parallel findFeatures - begin\n" );
					for ( int i = 0; i < numViews; ++i, ++vit )
					{
//						printf( "parallel findFeatures - view %d\n", i );
						//blocks, if none is idle
						JobThread *t = pool.getIdle();
						ViewT *v = dynamic_cast<ViewT *> ( *vit );
						FFJob *j = static_cast<FFJob *> ( t->getJob() );
						j->setView( v );
						t->doJob( j );
					}
					//wait for running jobs to finish
					pool.joinJobs();

//					printf( "parallel findFeatures - end\n" );
				}

				void findFeatures ( NViewMap &map )
				{
					findFeatures ( map.getAllViewsBegin(), map.getNumAllViews() );
				}

		};

	}
}//ns

#endif
