

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORESCALEDOGFINDER_H
#define ADORE_CORESCALEDOGFINDER_H

#include <adrcore/imgprocessing/stretchnskewremover.h>
#include <adrcore/viewmatch/featurefinder.h>
//#include <adrcore/imgprocessing/scalespacedog.h>
//#include <adrcore/imgprocessing/imagering.h>
#include <adrcore/imgprocessing/derivatives.h>
#include <adrcore/viewmatch/descriptorcreator.h>
#include <adrcore/features/extremafeatures.h>
#include <adrcore/types/view.h>
#include <adrcore/features/channelextrema.h>
//#include <adrcore/imgprocessing/harrisfast.h>
#include <adrcore/imgprocessing/harrisfasthood.h>
#include <adrcore/imgprocessing/channelfunctions.h>
#include <adrcore/imgprocessing/scalehelpers.h>

#ifndef NDEBUG
//uncomment this
#define NSCALEDEBUG
#else
#define NSCALEDEBUG
#endif

#define DBGSDF DBGALL

namespace adore
{

	namespace core
	{

		/**
		 *
			 
		*/
		class ScaleDOGFinder : public FeatureFinder
		{
			public:
				typedef signedT< imgCalcT<imgT>::type >::type diffT; //uint8_t -> int16_t
				typedef signedT< calcT<imgT>::type >::type calcT; //uint8_t -> int32_t

			private:
				uint fid;//feature id

				ImagePlane< diffT > dogImg;
				ImagePlane< imgT > patchImg;//for StretchNSkew-Removal
				ChannelExtrema< diffT , ScaleDOGFinder, greater_equal<diffT> > extremaG;
				ChannelExtrema< diffT , ScaleDOGFinder, less_equal<diffT> > extremaL;
				FeatureGrid candidates;

				int numSmooths;
				int numImages;
				float scale;
				int spacing;
				DescriptorCreator *descriptorcreator;

				int maxPts;
				float minDist;
				float minFraction;

				ViewT currentView;//a View that wraps the images to please the descriptorcreators
				int patchFactor;//this times scaleVal gets the patchsize for SnS-Revoe and Decriptor-Affine-Det

				bool snsRemove;
				AMatrixf snsMat;
				StretchNSkewRemover<imgT> snsrm;

			public:
				/**
				 * @param numSmooths number of smooths between subsamplings
				 * @param numImages number of smoothed images computed
				 */
				ScaleDOGFinder( uint ID, int numImages, calcT minHarrisThresh, int maxPoints = INT_MAX, float minDistance = 3.0F, float minFract = 0.000001 );

				ScaleDOGFinder( const ScaleDOGFinder &sdf );

				FeatureFinder *clone() {return new ScaleDOGFinder( *this );}

				inline void operator()( const Channel<diffT> &img, diffT *localMaximum )
				{
//					DBGOUT( DBGALL, "ScaleHarris: extrema callback\n" );
					const float dxMax = 0.5F, dyMax = 0.5F;

					uint x = 0, y = 0;
					float dx = 0, dy = 0;
					img.getCoors( localMaximum, x, y );

					bool success = subCoorQuadratic( img.getIterator( y - 1, x - 1, 3 ), dx, dy );
//					printf( "success: %s, Dx: %f, Dy: %f\n", success ? "true" : "false", dx, dy );
//					pressKey();
					if ( success && fabs( dx ) < dxMax && fabs( dy ) < dyMax )
					{
						dx += x;
						dy += y;


						float scaleVal = max( 1.0F, scale / spacing );
						int hood = patchFactor * int( scaleVal + 0.5F );
						int width = 2 * hood + 1;

						bool success = true;
						if ( snsRemove )
						{
							snsrm.setPatchSize( width );
							UsingROI ur1( currentView.getImage(), ROI( x - hood, y - hood, width, width ) );
							UsingROI ur2( &patchImg, ROI( 0, 0, width, width ) );
							success = snsrm.process( currentView.getImage(), dx, dy, &patchImg );
							snsMat = snsrm.getMatrix();
							if ( !success )
							{
								DBGOUT( DBGNORMAL, "sns-remove failed for (%f, %f)\n", dx, dy );
							}
						}

						if ( success )
						{
							//dont's scale the coordinate here
							//there are used inside the descriptorcreators which
							//access the also scaled image
							FeaturePoint *pt = new FeaturePoint( dx, dy, fid );
							pt->setStrength( fabs( double ( *localMaximum ) ) );//TODO - this is approximately
							pt->setScale( scale );
							pt->getInvAffineMat() = scaleVal * snsMat;

							if ( descriptorcreator )
							{
//								DBGOUT( DBGSDF, "ScaleHarris: currentView add\n" );
								if ( GRID_SUCCESS != currentView.add( pt ) ) error( "Grid in trouble\n" );
							}
							else
							{
								//scale now
								( *pt )( 0 ) = getPixelPosition( dx, spacing );
								( *pt )( 1 ) = getPixelPosition( dy, spacing );
//								DBGOUT( DBGSDF, "ScaleHarris: candidates add %p(%f, %f)\n", pt, ( *pt )( 0 ), ( *pt )( 1 ) );
								candidates.add( pt );
							}
						}
					}
				}

				void setDescriptorCreator ( DescriptorCreator* theValue )
				{
					descriptorcreator = theValue;
				}

				DescriptorCreator* getDescriptorCreator() const
				{
					return descriptorcreator;
				}

				void setSnsRemove ( bool theValue )
				{
					snsRemove = theValue;
				}

				bool getSnsRemove() const
				{
					return snsRemove;
				}

			protected:
				unsigned int findFeaturesDerived( ViewT *v );


		};

	}

}

#endif
