

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORESIFTDESCRIPTOR_H
#define ADORE_CORESIFTDESCRIPTOR_H

#include "descriptorcreator.h"
#include <adrcore/imgprocessing/siftat.h>
#include <adrcore/imgprocessing/rgbtogray.h>
#include <memory>

namespace adore
{

	namespace core
	{

		/**
			 * Calculates a sift descriptor
			 
		*/
		class SiftDescriptor : public DescriptorCreator
		{
			private:

				typedef FeatureDescriptor::DescVecT DescVecT;
				ImagePlane<imgT> gray;

			public:
				SiftDescriptor( uint featID = 0, uint descrID = 0 ):DescriptorCreator( featID, descrID ) {;}

				SiftDescriptor( const SiftDescriptor &sd ):DescriptorCreator( sd.getFeatureID(), sd.getDescriptorID() ) {;}

				~SiftDescriptor() {;}

				DescriptorCreator *clone() {return new SiftDescriptor( *this );}

			protected:
				void performActionDerived( ViewT *view )
				{
					ImagePlane<imgT> *src = view->getImage();
					assert( src );

					if ( src->getNumChannels()==3 )
					{
						COUT( DBGNORMAL, "Converting RGB to grayscale" << endl );
						gray.init(( int ) src->getWidth(),( int ) src->getHeight(),1 );

						RGBtoGray<imgT> converter;
						Channel<imgT>::Set in( true );
						src->createChannels( in );

						converter.process( in, &gray );
						src=&gray;
					};

					SiftAt<imgT> descr( *src );
					uint dSize = descr.getDescriptorSize();

					for ( ViewT::PointsIterator it = view->getPointsBegin( fid ); it != view->getPointsEnd( fid ); ++it )
					{
						assert( *it );
						FeaturePoint *fpPtr = *it;
						assert( fpPtr );
						FeaturePoint &fp = *fpPtr;
						assert( fp( 2 ) == 1.0F );

//						DBGOUT( DBGNORMAL, "Sift: Processing point (%f, %f) with strength %.2f\n", fp( 0 ), fp( 1 ), fp.getStrength() );

						//prepare calculates all main orientations
						//for each orientation a descriptor is created in
						//the subsequent loop
						if ( fp.hasInvAffineMat() )
						{
//							DBGOUT( DBGNORMAL, "Sift: using invAffine\n" );
//							COUT( DBGNORMAL, fp.getInvAffineMat() << endl );
							descr.prepare( fp( 0 ), fp( 1 ), fp.getInvAffineMat() );
						}
						else
						{
							DBGOUT( DBGALL, "Sift: no invAffine\n" );
							descr.prepare( fp( 0 ), fp( 1 ), Identity2x2f );
						}

						if ( descr.getNumOrientations() )
						{
							FeatureDescriptor *fd = new FeatureDescriptor( 0/*size*/, did );
//							DBGOUT( DBGNORMAL, "Sift: Num orientations: %d\n", descr.getNumOrientations() );
							//cycle through orientations
							for ( SiftAt<imgT>::iterator sit = descr.begin(); sit != descr.end(); ++sit )
							{
								// 					printf("%f\n", *sit);
								bool ok = true;
								DescVecT vec( dSize );
								vec = descr( *sit, &ok );
								if ( ok ) fd->addVector( vec );
							}

							fp.setDescriptor( fd );
						}
						else
						{
							DBGOUT( DBGNORMAL, "Sift: Can't attach an orientation to (%f, %f)\n", fp( 0 ), fp( 1 ) );
						}
					}
				}

		};

	}

}

#endif
