

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_OPENGLDISPLAY_H
#define ADORE_OPENGLDISPLAY_H

#include <GL/glut.h>
#include <GL/gl.h>  
#include <GL/glu.h> 
#include <unistd.h> 
#include "displayclient.h"
#include <vector>
#include <functional>
#include <algorithm>

#define ESCAPE 27

namespace adore {
namespace visual {
    
    /**
         
    */

void display_drawScene();
void display_idleFunc();
void display_keyPressed(unsigned char key, int x, int y);
void display_reshapeFunc(int w, int h);
    
class Display{
    private:
        
        static Display *globalDisplay;
    
    public:
        
        typedef std::vector<DisplayClient *>::iterator Iterator;
        
    public:
        Display(int w = 640, int h = 480):
            width(w),
            height(h)
        {
            globalDisplay = this;
        }
    
        void init()
        {
            int argc = 1;
            //char *argv = {'b', 'l', 'a', '1', '\0'};
            char *argv = const_cast<char *>("bla1");
            glutInit(&argc, &argv);  
            glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_ALPHA | GLUT_DEPTH);  

            glutInitWindowSize(width, height);  
            glutInitWindowPosition(0, 0);  
            window = glutCreateWindow("Display");  
            glutDisplayFunc(display_drawScene);  
//             glutFullScreen();
            glutIdleFunc(display_idleFunc);
            glutReshapeFunc(display_reshapeFunc);
            glutKeyboardFunc(display_keyPressed);
        }
        
        void addClient(DisplayClient *client)
        {
            clients.push_back(client);
        }
        
        void clearClients()
        {
            clients.clear();
        }
        
        void show()
        {
            initGL();
            glutMainLoop();  
        }
         
        int getWindow() const
        {
            return window;
        }
        
        Iterator getBegin()
        {
            return clients.begin();
        }
        
        Iterator getEnd()
        {
            return clients.end();
        }
        
        uint getNumClients()
        {
            return clients.size();
        }
        
        static Display *getGlobalDisplay(){return globalDisplay;}
        
    private:
        int window;
        int width, height;
        std::vector<DisplayClient *> clients;
        
        void initGL()          // We call this right after our OpenGL window is created.
        {
            glClearColor(0.0f, 0.0f, 0.0f, 0.0f);     // This Will Clear The Background Color To Black
            glClearDepth(1.0);                // Enables Clearing Of The Depth Buffer
            glDepthFunc(GL_LESS);             // The Type Of Depth Test To Do
            glEnable(GL_DEPTH_TEST);          // Enables Depth Testing
            glShadeModel(GL_SMOOTH);          // Enables Smooth Color Shading
            
            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();             // Reset The Projection Matrix
            
            gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f); // Calculate The Aspect Ratio Of The Window
            
            glMatrixMode(GL_MODELVIEW);
        }
};


}}//namespace

#endif
