

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#include "glavatar.h"

namespace adore
{
	namespace visual
	{

		void GLAvatar::box(GLMesh& mesh, float dx, float dy, float dz)
		{
			dx *= 0.5f;
			dy *= 0.5f;
			dz *= 0.5f;

			const int numvert = 8;
			const int numface = 12;
			const int numnorm = 6;

			// create resources
			mesh.createVertices(numvert);
			mesh.createFaces(numface);
			mesh.createNormals(numnorm);

			// lock resources
			Vertex* vertexdata = mesh.lockVertices(0,numvert);
			Face* facedata = mesh.lockFaces(0,numface);
			Vector3D* normaldata = mesh.lockNormals(0,numnorm);

			vertexdata[0].pos.set(dx,-dy,-dz);	vertexdata[0].tex_u = 0.0f;
			vertexdata[0].tex_v = 0.0f;			vertexdata[0].extra = 0;
			vertexdata[1].pos.set(dx,-dy,dz);	vertexdata[1].tex_u = 1.0f;
			vertexdata[1].tex_v = 0.0f;			vertexdata[1].extra = 1;
			vertexdata[2].pos.set(dx,dy,dz);	vertexdata[2].tex_u = 1.0f;
			vertexdata[2].tex_v = 1.0f;			vertexdata[2].extra = 2;
			vertexdata[3].pos.set(dx,dy,-dz);	vertexdata[3].tex_u = 0.0f;
			vertexdata[3].tex_v = 1.0f;			vertexdata[3].extra = 3;

			vertexdata[4].pos.set(-dx,-dy,-dz);	vertexdata[4].tex_u = 1.0f;
			vertexdata[4].tex_v = 0.0f;			vertexdata[4].extra = 4;
			vertexdata[5].pos.set(-dx,-dy,dz);	vertexdata[5].tex_u = 1.0f;
			vertexdata[5].tex_v = 1.0f;			vertexdata[5].extra = 5;
			vertexdata[6].pos.set(-dx,dy,dz);	vertexdata[6].tex_u = 0.0f;
			vertexdata[6].tex_v = 1.0f;			vertexdata[6].extra = 6;
			vertexdata[7].pos.set(-dx,dy,-dz);	vertexdata[7].tex_u = 0.0f;
			vertexdata[7].tex_v = 0.0f;			vertexdata[7].extra = 7;

			normaldata[0].set(1.0f,0.0f,0.0f);
			normaldata[1].set(-1.0f,0.0f,0.0f);
			normaldata[2].set(0.0f,1.0f,0.0f);
			normaldata[3].set(0.0f,-1.0f,0.0f);
			normaldata[4].set(0.0f,0.0f,1.0f);
			normaldata[5].set(0.0f,0.0f,-1.0f);

			facedata[0].v1 = 2;		facedata[0].v2 = 1;		facedata[0].v3 = 0;		facedata[0].normal = 0;
			facedata[1].v1 = 0;		facedata[1].v2 = 3;		facedata[1].v3 = 2;		facedata[1].normal = 0;

			facedata[2].v1 = 5;		facedata[2].v2 = 6;		facedata[2].v3 = 7;		facedata[2].normal = 1;
			facedata[3].v1 = 7;		facedata[3].v2 = 4;		facedata[3].v3 = 5;		facedata[3].normal = 1;
			facedata[4].v1 = 6;		facedata[4].v2 = 2;		facedata[4].v3 = 3;		facedata[4].normal = 2;
			facedata[5].v1 = 3;		facedata[5].v2 = 7;		facedata[5].v3 = 6;		facedata[5].normal = 2;
			facedata[6].v1 = 1;		facedata[6].v2 = 5;		facedata[6].v3 = 4;		facedata[6].normal = 3;
			facedata[7].v1 = 4;		facedata[7].v2 = 0;		facedata[7].v3 = 1;		facedata[7].normal = 3;
			facedata[8].v1 = 6;		facedata[8].v2 = 5;		facedata[8].v3 = 1;		facedata[8].normal = 4;
			facedata[9].v1 = 1;		facedata[9].v2 = 2;		facedata[9].v3 = 6;		facedata[9].normal = 4;
			facedata[10].v1 = 4;	facedata[10].v2 = 7;	facedata[10].v3 = 3;	facedata[10].normal = 5;
			facedata[11].v1 = 3;	facedata[11].v2 = 0;	facedata[11].v3 = 4;	facedata[11].normal = 5;

			// unlock resources
			mesh.unlockVertices();
			mesh.unlockFaces();
			mesh.unlockNormals();
		}

		void GLAvatar::polygon(GLMesh& mesh, unsigned sides)
		{
			assert(sides >= 3);

			int numvert = sides;
			int numface = sides - 2;
			const int numnorm = 1;		// planar figure

			// create resources
			mesh.createVertices(numvert);
			mesh.createFaces(numface);
			mesh.createNormals(numnorm);

			// lock resources
			Vertex* vertexdata = mesh.lockVertices(0,numvert);
			Face* facedata = mesh.lockFaces(0,numface);
			Vector3D* normaldata = mesh.lockNormals(0,numnorm);

			float delta = 360.0f * DEG2RAD / sides;
			float start = 225.0f * DEG2RAD - delta * 0.5f;
			float angle;

			normaldata[0].set(0.0f,0.0f,-1.0f);

			for(int i = 0; i < numvert; ++i)
			{
				angle = start + delta * i;
				vertexdata[i].pos.x = std::sin(angle);
				vertexdata[i].pos.y = std::cos(angle);
				vertexdata[i].pos.z = 0.0f;
				vertexdata[i].tex_u = vertexdata[i].pos.x;
				vertexdata[i].tex_v = vertexdata[i].pos.y;
				vertexdata[i].extra = i;
			}

			for(int i = 0; i < numface; ++i)
			{
				facedata[i].v1 = 0;
				facedata[i].v2 = i+1;
				facedata[i].v3 = i+2;
				facedata[i].normal = 0;
			}

			// unlock resources
			mesh.unlockVertices();
			mesh.unlockFaces();
			mesh.unlockNormals();
		}

		void GLAvatar::tetrahedron(GLMesh& mesh)
		{
			const float sqrt_2		=	1.4142135623730950488016887242097f;
			const float sqrt_3		=	1.7320508075688772935274463415059f;
			const float sqrt_2_3	=	0.81649658092772603273242802490196f;
			const float sqrt_1_3	=	0.57735026918962576450914878050196f;

			const int numvert = 4;
			const int numface = 4;
			const int numnorm = 4;

			// create resources
			mesh.createVertices(numvert);
			mesh.createFaces(numface);
			mesh.createNormals(numnorm);

			// lock resources
			Vertex* vertexdata = mesh.lockVertices(0,numvert);
			Face* facedata = mesh.lockFaces(0,numface);
			Vector3D* normaldata = mesh.lockNormals(0,numnorm);

			vertexdata[0].pos.set(0.0f,0.0f,sqrt_3);
			vertexdata[1].pos.set(0.0f,2.0f*sqrt_2_3,-sqrt_1_3);
			vertexdata[2].pos.set(-sqrt_2,-sqrt_2_3,-sqrt_1_3);
			vertexdata[3].pos.set(sqrt_2,-sqrt_2_3,-sqrt_1_3);

			vertexdata[0].tex_u =	0.0f;	vertexdata[0].tex_v = 0.0f;
			vertexdata[1].tex_u =	1.0f;	vertexdata[1].tex_v = 0.0f;
			vertexdata[2].tex_u =	0.0f;	vertexdata[2].tex_v = 1.0f;
			vertexdata[3].tex_u =	1.0f;	vertexdata[3].tex_v = 1.0f;

			facedata[0].v1 = 0;	facedata[0].v2 = 1;	facedata[0].v3 = 2;	facedata[0].normal = 0;
			facedata[1].v1 = 0;	facedata[1].v2 = 2;	facedata[1].v3 = 3;	facedata[1].normal = 1;
			facedata[2].v1 = 0;	facedata[2].v2 = 3;	facedata[2].v3 = 1;	facedata[2].normal = 2;
			facedata[3].v1 = 1;	facedata[3].v2 = 3;	facedata[3].v3 = 2;	facedata[3].normal = 3;

			for(int i = 0; i<4;++i)
			{
				normaldata[i] = vertexdata[facedata[i].v1].pos +
					vertexdata[facedata[i].v2].pos + vertexdata[facedata[i].v3].pos;
				normaldata[i] /= 3.0f;
			}

			// unlock resources
			mesh.unlockVertices();
			mesh.unlockFaces();
			mesh.unlockNormals();
		}

		void GLAvatar::camera(GLMesh& mesh)
		{
			// load hard-coded avatar mesh
			const int numvert = 20;
			const int numface = 30;
			const int numnorm = 10;

			// create resources
			mesh.createVertices(numvert);
			mesh.createFaces(numface);
			mesh.createNormals(numnorm);

			// lock resources
			Vertex* vertexdata = mesh.lockVertices(0,numvert);
			Face* facedata = mesh.lockFaces(0,numface);
			Vector3D* normaldata = mesh.lockNormals(0,numnorm);

			// build mesh
			std::memset(vertexdata, 0, sizeof(Vertex)*numvert);

			// vertices
			vertexdata[0].pos.set(-1.0f,-1.0f,0.0f);
			vertexdata[1].pos.set(1.0f,-1.0f,0.0f);
			vertexdata[2].pos.set(1.0f,1.0f,0.0f);
			vertexdata[3].pos.set(-1.0f,1.0f,0.0f);

			vertexdata[4].pos.set(-0.5f,-0.5f,1.0f);
			vertexdata[5].pos.set(0.5f,-0.5f,1.0f);
			vertexdata[6].pos.set(0.5f,0.5f,1.0f);
			vertexdata[7].pos.set(-0.5f,0.5f,1.0f);

			vertexdata[8].pos.set(-0.5f,-0.5f,2.0f);
			vertexdata[9].pos.set(0.5f,-0.5f,2.0f);
			vertexdata[10].pos.set(0.5f,0.5f,2.0f);
			vertexdata[11].pos.set(-0.5f,0.5f,2.0f);

			vertexdata[12].pos.set(-1.0f,-2.0f,2.0f);
			vertexdata[13].pos.set(1.0f,-2.0f,2.0f);
			vertexdata[14].pos.set(1.0f,1.0f,2.0f);
			vertexdata[15].pos.set(-1.0f,1.0f,2.0f);

			vertexdata[16].pos.set(-1.0f,-2.0f,6.0f);
			vertexdata[17].pos.set(1.0f,-2.0f,6.0f);
			vertexdata[18].pos.set(1.0f,1.0f,6.0f);
			vertexdata[19].pos.set(-1.0f,1.0f,6.0f);

			// faces
			facedata[0].v1 = 1;	facedata[0].v2 = 0;	facedata[0].v3 = 3;	facedata[0].normal = 0;
			facedata[1].v1 = 1;	facedata[1].v2 = 3;	facedata[1].v3 = 2;	facedata[1].normal = 0;

			facedata[2].v1 = 0;	facedata[2].v2 = 1;	facedata[2].v3 = 5;	facedata[2].normal = 1;
			facedata[3].v1 = 0;	facedata[3].v2 = 5;	facedata[3].v3 = 4;	facedata[3].normal = 1;
			facedata[4].v1 = 1;	facedata[4].v2 = 2;	facedata[4].v3 = 6;	facedata[4].normal = 2;
			facedata[5].v1 = 1;	facedata[5].v2 = 6;	facedata[5].v3 = 5;	facedata[5].normal = 2;
			facedata[6].v1 = 2;	facedata[6].v2 = 3;	facedata[6].v3 = 7;	facedata[6].normal = 3;
			facedata[7].v1 = 2;	facedata[7].v2 = 7;	facedata[7].v3 = 6;	facedata[7].normal = 3;
			facedata[8].v1 = 3;	facedata[8].v2 = 0;	facedata[8].v3 = 4;	facedata[8].normal = 4;
			facedata[9].v1 = 3;	facedata[9].v2 = 4;	facedata[9].v3 = 7;	facedata[9].normal = 4;

			facedata[10].v1 = 4;	facedata[10].v2 = 5;	facedata[10].v3 = 9;	facedata[10].normal = 5;
			facedata[11].v1 = 4;	facedata[11].v2 = 9;	facedata[11].v3 = 8;	facedata[11].normal = 5;
			facedata[12].v1 = 5;	facedata[12].v2 = 6;	facedata[12].v3 = 10;	facedata[12].normal = 6;
			facedata[13].v1 = 5;	facedata[13].v2 = 10;	facedata[13].v3 = 9;	facedata[13].normal = 6;
			facedata[14].v1 = 6;	facedata[14].v2 = 7;	facedata[14].v3 = 11;	facedata[14].normal = 7;
			facedata[15].v1 = 6;	facedata[15].v2 = 11;	facedata[15].v3 = 10;	facedata[15].normal = 7;
			facedata[16].v1 = 7;	facedata[16].v2 = 4;	facedata[16].v3 = 8;	facedata[16].normal = 8;
			facedata[17].v1 = 7;	facedata[17].v2 = 8;	facedata[17].v3 = 11;	facedata[17].normal = 8;

			facedata[18].v1 = 13;	facedata[18].v2 = 12;	facedata[18].v3 = 15;	facedata[18].normal = 0;
			facedata[19].v1 = 13;	facedata[19].v2 = 15;	facedata[19].v3 = 14;	facedata[19].normal = 0;

			facedata[20].v1 = 12;	facedata[20].v2 = 13;	facedata[20].v3 = 17;	facedata[20].normal = 5;
			facedata[21].v1 = 12;	facedata[21].v2 = 17;	facedata[21].v3 = 16;	facedata[21].normal = 5;
			facedata[22].v1 = 13;	facedata[22].v2 = 14;	facedata[22].v3 = 18;	facedata[22].normal = 6;
			facedata[23].v1 = 13;	facedata[23].v2 = 18;	facedata[23].v3 = 17;	facedata[23].normal = 6;
			facedata[24].v1 = 14;	facedata[24].v2 = 15;	facedata[24].v3 = 19;	facedata[24].normal = 7;
			facedata[25].v1 = 14;	facedata[25].v2 = 19;	facedata[25].v3 = 18;	facedata[25].normal = 7;
			facedata[26].v1 = 15;	facedata[26].v2 = 12;	facedata[26].v3 = 16;	facedata[26].normal = 8;
			facedata[27].v1 = 15;	facedata[27].v2 = 16;	facedata[27].v3 = 19;	facedata[27].normal = 8;

			facedata[28].v1 = 16;	facedata[28].v2 = 17;	facedata[28].v3 = 18;	facedata[28].normal = 9;
			facedata[29].v1 = 16;	facedata[29].v2 = 18;	facedata[29].v3 = 19;	facedata[29].normal = 9;

			// normals
			for(int i = 0; i<numface;++i)
			{
				Vector3D e1 = vertexdata[facedata[i].v2].pos - vertexdata[facedata[i].v1].pos;
				Vector3D e2 = vertexdata[facedata[i].v1].pos - vertexdata[facedata[i].v3].pos;
				normaldata[facedata[i].normal].cross(e1,e2);
				normaldata[facedata[i].normal].normalize();
			}

			// unlock resources
			mesh.unlockVertices();
			mesh.unlockFaces();
			mesh.unlockNormals();
		}
	}
}
