

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#ifndef ADORE_GLENTITY_H
#define ADORE_GLENTITY_H

#include "gltransform.h"
#include "glmesh.h"
#include "gltexture.h"

namespace adore
{
	namespace visual
	{
		/**	Entity is a unit of rendering.
		 *
		 *	It has full information about its own form, transformation,
		 *	texture and material.
		 *
		 *	 
		 */
		class GLEntity
		{
		protected:
			static bool statready;		///< static members are ready
			static GLMesh bbrd;			///< mesh for visualization of a single point

		public:
			GLTransform world;			///< object transformation
			GLMesh mesh;				///< mesh
			GLTexture texture;			///< texture
			int material;				///< material id
			float pointsize;			///< point size in pixels
			float thickness;			///< line thickness in pixels
			RGBA pointcolor;			///< point color
			RGBA linecolor;				///< line color
			int highlighted;			///< highlighted vertex

			bool drawmesh;				///< visualize mesh
			bool drawvertices;			///< visualize vertices
			bool drawnormals;			///< visualize normals as lines
			bool drawframe;				///< visualize object space frame

			//a vector for turntable motion
			Vector3D objUp;
			
			/**	Constructor
			 */
			GLEntity();

			/**	Get the material index
			 */
			int getMaterial();

			/**	Render the entity
			 */
			void render();

			/**	Draw model mesh
			 */
			void drawMesh();

			/**	Draw vertices as 3d points
			 */
			void drawVertices();

			/** Draw normals to faces
			 */
			void drawNormals();

			/**	Draw object local coordinate frame
			 */
			void drawFrame();

			/**	Select some vertex treating it as a sphere in the world space.
			 *	Radius is always defined in the screen space and is view-dependent.
			 *	@param org Origin of the picking ray
			 *	@param dir Direction vector of the picking ray
			 *	@param fov field of view
			 *	@return the first vertex which is hit by the ray org|-dir
			 */
			int select(Vector3D org, Vector3D dir);

			/**	Test if the mesh of the entity is hit by the ray
			 *	@return Intersection parameter
			 */
			float hit(Vector3D org, Vector3D dir);

			void setObjUp ( const Vector3D& theValue )
			{
				objUp = theValue;
			}

			Vector3D getObjUp() const
			{
				return objUp;
			}
			
		};
	}
}

#endif	//ADORE_GLENTITY_H
