

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#ifndef ADORE_GLNVIEWRENDERER_H
#define ADORE_GLNVIEWRENDERER_H

#include <vector>
#include <map>

#include "gldelaunayrenderer.h"
#include "glplaneloader.h"
#include <adrcore/imgprocessing/imageplane.h>

namespace adore
{
	namespace visual
	{
		/**	Templated version of GLDelaunayRenderer
		 *
		 *	 
		 */
		template<class T> class GLNViewRenderer : public GLDelaunayRenderer
		{
			core::NView* oldNv;
		public:
			GLNViewRenderer():GLDelaunayRenderer(), oldNv(NULL){}
			
			virtual void attach(core::NView* nv)
			{
				oldNv = nv_ref;
				GLDelaunayRenderer::attach(nv);
			}
			
			virtual void init()
			{
				if(!nv_ref)
				{
					clear();
					return;
				}
				
				//the following if/else exists solely to allow the deletion of points without reseting the viewpoint
				if(oldNv != nv_ref)//otherwise assumes, that the things accessed in this block where not mofified
				{
					DBGOUT(DBGNORMAL, "GLNViewRenderer: A fresh new NView.\n");
					clear();
				
					data.resize(nv_ref->getNumViews());
					unsigned index = 0;
					core::NView::ViewIterator b = nv_ref->getViewsBegin();
					core::NView::ViewIterator e = nv_ref->getViewsEnd();
					for(core::NView::ViewIterator iter = b; iter!=e; ++iter)
					{
						core::View<T>* view = static_cast<View<T> *>(*iter);
						core::ImagePlane<T>* img = view->getImage();

						data[index].view = view;
						data[index].width = img->getWidth();
						data[index].height = img->getHeight();
						data[index].pixels = new RGBA[data[index].width * data[index].height];
						data[index].entity = new GLEntity;
						DBGOUT(DBGNORMAL, "GLNViewRenderer: At index %d entity %p (%d, %d) has been created.\n", index, data[index].entity, data[index].entity->texture.getWidth(), data[index].entity->texture.getHeight());

						typename core::Channel<T>::Set cset(*img, true);
						GLPlaneLoader<T> loader(cset);
						loader.load(reinterpret_cast<T*>(data[index].pixels));

						++index;
					}
				}
				else
				{
					//we need to copy the image pixels again, because GLDelaunayRenderer::init() deletes them.
					DBGOUT(DBGNORMAL, "GLNViewRenderer: Same NView again.\n");
					assert(nv_ref->getNumViews() == oldNv->getNumViews());
					unsigned index = 0;
					core::NView::ViewIterator b = nv_ref->getViewsBegin();
					core::NView::ViewIterator e = nv_ref->getViewsEnd();
					for(core::NView::ViewIterator iter = b; iter!=e; ++iter)
					{
						core::View<T>* view = static_cast<View<T> *>(*iter);
						core::ImagePlane<T>* img = view->getImage();
						data[index].width = img->getWidth();
						data[index].height = img->getHeight();
						data[index].pixels = new RGBA[data[index].width * data[index].height];
						typename core::Channel<T>::Set cset(*img, true);
						GLPlaneLoader<T> loader(cset);
						loader.load(reinterpret_cast<T*>(data[index].pixels));
						++index;
					}
				}
				GLDelaunayRenderer::init();
			}
		};
	}
}

#endif	// ADORE_GLNVIEWRENDERER_H
