

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#ifndef ADORE_GLRENDERER_H
#define ADORE_GLRENDERER_H

#include "glvisualizer.h"
#include "gllight.h"
#include "glmaterial.h"
#include "glcamera.h"
#include "glentity.h"

namespace adore
{
	namespace visual
	{
		/**	Visualizer implementation
		 *
		 *	This class draws the NView data as a textured and lit
		 *	polygonal model using OpenGL API. Materials and lights
		 *	can be defined in the settings file using simple synthax.
		 *
		 *	 
		 */
		class GLRenderer : public GLVisualizer
		{
			public:
				static const int maxlights = 8;			///< maximum lights per scene
				static const int maxmaterials = 8;		///< maximum materials per scene

				GLLight lights[maxlights];				///< light settings
				GLMaterial materials[maxmaterials];		///< material settings
				GLCamera camera;						///< active camera

			protected:
				int displayflags;						///< presentation parametres
				RGBA background;						///< background color
				RGBA pencolor;							///< point and line color
				float pensize;							///< point size and line thickness
				GLEntity* entity;						///< active scene
				float fieldofview;						///< field of view
				float aspect;							///< aspect ratio

				virtual void select ( Vector3D& org, Vector3D& dir );

			public:
				/**	Constructor
				 */
				GLRenderer();

				/**	Destructor
				 */
				virtual ~GLRenderer();

				/**	Load and parse configuration from stream
				 */
				bool loadConfig ( std::istream& in );

				/**	Load and parse configuration from file
				 */
				bool loadConfig ( const char* filename );

				virtual int settings();
				virtual void settings ( int flags );
				virtual void init();
				virtual void deviation ( float value );
				void deviationTicks (int ticks, int maxTicks = 1000);
				virtual void focus();
				virtual void unfocus();
				virtual void paint();
				virtual void resize ( int width, int height );
				virtual void pick ( int x, int y );
				virtual void perspective ( float fov );
				virtual void lookAt ( const float eye[3], const float aim[3], const float up[3] );
				virtual void roll ( float angle );
				virtual void yaw ( float angle );
				virtual void pitch ( float angle );
				virtual void turn ( float angle );
				virtual void tilt ( float angle, float gap );
				virtual void slide ( const float delta[3] );
				virtual void translate ( const float delta[3], bool absolute );
				virtual void rotate ( const float axis[3], float angle, bool absolute );
				virtual void turnTable ( float angle, bool absolute );
				virtual void scale ( const float factor[3], bool absolute );
				virtual void identity();
				virtual void bgcolor ( const unsigned char color[4] );
				virtual void pen ( const unsigned char color[4], float size );
				void penTicks ( const unsigned char color[4], int ticks, int maxTicks = 1000 );

				void setCamera ( const GLCamera &cam ) {camera = cam;}
				GLCamera getCamera() const {return camera;}

				/**
				 * returns the entity that is drawn
				 */
				GLEntity* getEntity() const
				{
					return entity;
				}
				
				void setTurnTableSpine ( const float axis[3] );
				
			private:
				int last_mat;

				void applyDisplayFlags();
				void applyPenStyle();
				void applyColorStyle();
				void applyLightSettings();
		};
	}
}

#endif	// ADORE_GLRENDERER_H
