

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Andrei Tchouev


#include "gltransform.h"

#include <GL/gl.h>

namespace adore
{
	namespace visual
	{
		GLTransform::GLTransform()
		{
			identity();
		}

		const Vector3D& GLTransform::getTranslation() const
		{
			return tr;
		}

		const Vector3D& GLTransform::getScale() const
		{
			return scl;
		}

		const Quaternion& GLTransform::getRotation() const
		{
			return rot;
		}

		void GLTransform::setTranslation(Vector3D t)
		{
			dirty = true;
			tr = t;
		}

		void GLTransform::setScale(Vector3D s)
		{
			dirty = true;
			scl = s;
		}

		void GLTransform::setRotation(Quaternion r)
		{
			dirty = true;
			rot = r;
		}

		void GLTransform::identity()
		{
			dirty = false;
			tr.zero();
			scl.set(1.0f,1.0f,1.0f);
			rot.set(0.0f,0.0f,0.0f,1.0f);
			total.identity();
			invert.identity();
		}

		void GLTransform::build()
		{
			Vector3D invscl(1.0f / scl.x, 1.0f / scl.y, 1.0f / scl.z);
			Vector3D invtr(-tr);

			// rotation part
			total = rot.toMatrix();

			invert = total.transpose();

			// scale part
			total[0][0] *= scl.x;	total[0][1] *= scl.y;	total[0][2] *= scl.z;
			total[1][0] *= scl.x;	total[1][1] *= scl.y;	total[1][2] *= scl.z;
			total[2][0] *= scl.x;	total[2][1] *= scl.y;	total[2][2] *= scl.z;

			invert[0][0] *= invscl.x;	invert[0][1] *= invscl.x;	invert[0][2] *= invscl.x;	
			invert[1][0] *= invscl.y;	invert[1][1] *= invscl.y;	invert[1][2] *= invscl.y;
			invert[2][0] *= invscl.z;	invert[2][1] *= invscl.z;	invert[2][2] *= invscl.z;

			// translation part
			total[0][3] = tr.x;
			total[1][3] = tr.y;
			total[2][3] = tr.z;

			invert[0][3] = invtr.x * invert[0][0] + invtr.y * invert[0][1] + invtr.z * invert[0][2];
			invert[1][3] = invtr.x * invert[1][0] + invtr.y * invert[1][1] + invtr.z * invert[1][2];
			invert[2][3] = invtr.x * invert[2][0] + invtr.y * invert[2][1] + invtr.z * invert[2][2];

			dirty = false;
		}

		void GLTransform::setOpengGLMatrix()
		{
			glMatrixMode(GL_MODELVIEW);
			if(dirty)
				build();

			// OpenGL regards array as a column-major matrix
			Matrix4x4 oglmat = total.transpose();
			glMultMatrixf(oglmat.toArray());
		}

		void GLTransform::projectVector(Vector4D& src, Vector4D& dst)
		{
			if(dirty)
				build();
			total.projectVector(src,dst);
		}

		void GLTransform::unprojectVector(Vector4D& src, Vector4D& dst)
		{
			if(dirty)
				build();
	
			invert.projectVector(src,dst);
		}
		
		const GLTransform &GLTransform::operator=(const GLTransform &t)
		{
			tr = t.getTranslation();
			scl = t.getScale();
			rot = t.getRotation();
			dirty = true;
			return *this;
		}
	}
}
