

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_VISUALGRIDWORLDDISPLAYCLIENT_H
#define ADORE_VISUALGRIDWORLDDISPLAYCLIENT_H

#include <visual/displayclient.h>
#include <GL/glut.h>
#include <GL/gl.h>  
#include <GL/glu.h> 
#include <adrcore/rl/rl.h>
#include <adrcore/rl/bestqpath.h>
#include <adrcore/util/debug.h>
#include <adrcore/types/aarray.h>
#include <set>

namespace adore {

namespace visual {
    
/**
 * Visualizes a qtable of a Gridworld.
 * the states' ids must correspond to the pixel positions
	 
*/
template<class GridT, class QTableT>
class GridWorldDisplayClient : public DisplayClient, public core::BestQPath<QTableT, GridT>
{
    typedef typename GridT::StateT StateT;
    typedef typename GridT::ActionT ActionT;
	typedef typename core::BestQPath<QTableT, GridT> BaseT;
    
    public:
        GridWorldDisplayClient(GridT *g, QTableT *t):
            core::BestQPath<QTableT, GridT>(t, g),
            table(t),
            grid(g)
        {
        }
    
        ~GridWorldDisplayClient(){;}

    private:
        
        void drawSquare(float x, float y, float r, float g, float b, float a = 1.0)
        {
                int h = getResHeight();
                glColor4f(r, g, b, a);
                glVertex2f(x, h - y);
                glVertex2f(x, h - y - dy);
                glVertex2f(x + dx, h - y - dy);
                glVertex2f(x + dx, h - y);
        }
        
        void drawArrow(float fromX, float fromY, float toX, float toY, 
                       float r, float g, float b, float a = 1.0)
        {
            int h = getResHeight();
            glColor4f(r, g, b, a);
            
            glVertex2f(fromX, h - fromY);
            glVertex2f(toX, h - toY);
            
            const float scale = 0.1;
            float relX = toX - fromX;
            float relY = toY - fromY;
            float orthoX = relY;
            float orthoY = -relX;
            
            glVertex2f(toX, h - toY);
            glVertex2f(toX + scale * (orthoX - relX), h - (toY + scale * (orthoY - relY)));
            
            orthoX = -orthoX;
            orthoY = -orthoY;
            
            glVertex2f(toX, h - toY);
            glVertex2f(toX + scale * (orthoX - relX), h - (toY + scale * (orthoY - relY)));
        }
        
        void transition(StateT s1, StateT s2, ActionT a, core::QValueT reward, int depth)
        {
            ADORE_UNUSED(a);
            ADORE_UNUSED(depth);
            float ratio = (reward < 0 ? 0 : reward) / grid->getMaxReward();
            
            int row, col;
            grid->toRowCol(int(s1), row, col);
            float lastFx = (col + 0.5) * dx; 
            float lastFy = (row + 0.5) * dy;
            
            grid->toRowCol(int(s2), row, col);
            float fx = (col + 0.5) * dx; 
            float fy = (row + 0.5) * dy;
                    
            drawArrow(lastFx, lastFy, fx, fy, (1.0 - ratio), ratio, 0.0);
        }
        
        void drawTransitions(StateT s, StateT goal)
        {
            if(int(s) == int(goal))
            {
                printf("goal reached\n");
                return;
            }
            std::set<int>::iterator it = visited.find(int(s));
            if(it != visited.end()) return;
            
            int row, col;
            grid->toRowCol(int(s), row, col);
            float lastFx = (col + 0.5) * dx; 
            float lastFy = (row + 0.5) * dy;

            std::vector<ActionT> bestA;
            visited.insert(int(s));
                    
//             printf("find action of maximum reward\n");
                
            core::QValueT reward = core::getMaxRewardActions(table, grid, s, bestA);
//             printf("find action of maximum reward - done\n");
            if(bestA.size() == 0)
            {
                printf("dead end\n");
                return;
            }
            else
            {
                float ratio = (reward < 0 ? 0 : reward) / grid->getMaxReward();//red/green color ratio
                    
                for(typename std::vector<ActionT>::iterator it = bestA.begin(); it != bestA.end(); ++it)
                {
                    s = grid->perform(s, *it);
                    
                    grid->toRowCol(int(s), row, col);
                    float fx = (col + 0.5) * dx; 
                    float fy = (row + 0.5) * dy;
                    
                    drawArrow(lastFx, lastFy, fx, fy, (1.0 - ratio), ratio, 0.0);
                    
                    drawTransitions(s, goal);
                }
            }
        }
        
        void drawSceneImpl()
        {
            glDisable (GL_DEPTH_TEST);
            
            assert(table);
            assert(grid);
            visited.clear();
            
            core::AArray<int, 2> board = grid->getBoard();
            prepareRemap(board.cols(), board.rows());
            
            float fx = 0, fy = 0;
            dx = getScale(), dy = getScale();
            
            enable2D();
            
            //the faces of the grid
            fy = 0;
            glBegin(GL_QUADS);
            for(int y = 0; y < board.rows(); ++y, fy += dy)
            {
                fx = 0;
                for(int x =0; x < board.cols(); ++x, fx += dx)
                {
                    switch(board(y, x))
                    {
                        case 0:
                            drawSquare(fx, fy, 0.0, 0.0, 1.0);
                            break;
                        case GridT::S:
                            drawSquare(fx, fy, 1.0, 0.0, 0.0);
                            break;
                        case GridT::G:
                            drawSquare(fx, fy, 0.0, 1.0, 0.0);
                            break;
                        case GridT::C:
                            drawSquare(fx, fy, 0.0, 0.0, 0.0);
                            break;
                        case GridT::B:
                            drawSquare(fx, fy, 1.0, 1.0, 0.0);
                            break;
                    }
                    
                }
            }
            glEnd();
//             glClear(GL_DEPTH_BUFFER_BIT);
            
            //the lines of the grid
            fx = 0, fy = 0;
            float bottom = board.rows() * dy;
            float right = board.cols() * dx;
            glBegin(GL_LINES);
            
            glColor4f(1.0, 1.0, 1.0, 1.0);
            for(int y = 0; y <= board.rows(); ++y, fy += dy)
            {
                glVertex2f(0.0F, fy);
                glVertex2f(right, fy);
            }
            for(int x = 0; x <= board.cols(); ++x, fx += dx)
            {
                glVertex2f(fx, 0.0F);
                glVertex2f(fx, bottom);
            }
            
            //the most probable path
            StateT s(grid->getStart());
            StateT goal(grid->getGoal());
			this->dfs(s);
//             drawTransitions(s, goal);
            
            glEnd();
            
            disable2D();
            glEnable (GL_DEPTH_TEST);
        }
         
        QTableT *table;
        GridT *grid;
        float dx, dy;
        std::set<int> visited;

};

}

}

#endif
