

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_OPENGLITEMRENDERER_H
#define ADORE_OPENGLITEMRENDERER_H

#include "glnavigator.h"
#include "glcamera.h"
#include "renderitem.h"
#include <adrcore/util/debug.h>
#include <adrcore/types/aarray.h>
#include <vector>

namespace adore {
namespace visual {

/**
	 
*/
class ItemRenderer:public GLNavigator{
    public:
        ItemRenderer();
    
        virtual ~ItemRenderer();
        
        //navigator funtions:
        void init();
        void focus();
        void unfocus();
                
        void paint();
        void resize(int width, int height);
        void pick(int x, int y);
                
        void perspective(float fov, float aspect, float nplane, float fplane);
        void oblique(const float dir[3], float width, float height, float nplane, float fplane)
        {
            ADORE_UNUSED(dir);
            ADORE_UNUSED(width);
            ADORE_UNUSED(height);
            ADORE_UNUSED(nplane);
            ADORE_UNUSED(fplane);
        }
                
        void lookAt(const float eye[3], const float aim[3], const float up[3]);
        void lookAt(const adore::core::AVector &eye, const adore::core::AVector &aim, const adore::core::AVector &up);
        void roll(float angle);
        void yaw(float angle);
        void pitch(float angle);
        void slide(const float delta[3]);
        void turn(float angle);
        void tilt(float angle, float gap);
                
        //scene trafos are not supported
        void translate(const float delta[3], bool absolute)
        {
            ADORE_UNUSED(delta);
            ADORE_UNUSED(absolute);
        }
                
        void rotate(const float axis[3], float angle, bool absolute)
        {
            ADORE_UNUSED(axis);
            ADORE_UNUSED(angle);
            ADORE_UNUSED(absolute);
        }
                
        void scale(const float factor[3], bool absolute)
        {
            ADORE_UNUSED(factor);
            ADORE_UNUSED(absolute);
        }
                
        void identity(){;}
        
        //navigator functions - end

        void addRenderItem(RenderItem *it)
        {
            items.push_back(it);
        }
        
        void clearItems(bool del)
        {
            if(del)
            {
                for(uint i = 0; i < items.size(); ++i)
                {
                    delete items[i];
                }
            }
            items.clear();
        }
        
    private:

        std::vector<RenderItem *> items;
        GLCamera camera;
};

}}

#endif
