

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_VISUALVECTORDISPLAYCLIENT_H
#define ADORE_VISUALVECTORDISPLAYCLIENT_H

#include <visual/displayclient.h>
#include <adrcore/imgprocessing/channel.h>

namespace adore {

namespace visual {

/**
	 
*/
template<class imgType1, class imgType2 = imgType1>
class VectorDisplayClient : public DisplayClient
{
    typedef core::Channel<imgType1> ChanT1;
    typedef core::Channel<imgType2> ChanT2;
    
    public:
        VectorDisplayClient(ChanT1 *ch1, ChanT2 *ch2 = NULL, int stp = 5):
        chan1(ch1),
        chan2(ch2),
        step(stp)
        {
            
        }
    
        ~VectorDisplayClient(){;}
        
    private:
        
        void drawSceneImpl()
        {
            prepareRemap(chan1->getWidth(), chan1->getHeight());
                
            enable2D();
            drawXY();
            disable2D();
        }
        
        void idleFuncImpl(){/*drawScene();*/}
        
        void drawXY()
        {
            typename ChanT1::Iterator2D itC1 = chan1->getIteratorBegin(step, step);
            typename ChanT2::Iterator2D itC2 = chan2->getIteratorBegin(step, step);
            typename ChanT1::Iterator2D end = chan1->getIteratorEnd(step, step);
            for(; itC1 != end; ++itC1, ++itC2)
            {
                uint sx, sy;
                chan1->getCoors(itC1.getPointer(), sx, sy);

                float x = float(sx), y = float(sy);
                float dx = 0.5 * signum(*itC1) * max(0.0F, log2f(float(signum(*itC1) * *itC1)));
                float dy = 0.5 * signum(*itC2) * max(0.0F, log2f(float(signum(*itC2) * *itC2)));
//                 printf("(%d, %d): (%d, %d) - (%f, %f)\n", sx, sy, *itC1, *itC2, dx, dy);
                float x2 = x + dx;
                float y2 = y + dy;
                float x1 = x - dx;
                float y1 = y - dy;
                imageToWindow(x, y);
                imageToWindow(x1, y1);
                imageToWindow(x2, y2);
                
                glBegin(GL_LINE_STRIP);
                glColor4f(0.0, 1.0, 0.0, 1.0);
                glVertex2f(x1, y1);
                glColor4f(1.0, 0.0, 0.0, 1.0);
                glVertex2f(x, y);
                glColor4f(0.0, 1.0, 0.0, 1.0);
                glVertex2f(x2, y2);
                glEnd();
            }
        }
        
        ChanT1 *chan1;
        ChanT2 *chan2;
        int step;
};

}

}

#endif
