

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef CALCTHREAD_H
#define CALCTHREAD_H

#include <QApplication>
#include <QThread>
#include <QMutex>
#include <QMutexLocker>

#include <vector>
#include <memory>

#include <adrcore/types/nviewmap.h>
#include <adrcore/viewmatch/viewknitter.h>
#include <adrcore/nview/modifier.h>
#include <adrcore/nview/parallelmodifier.h>

#include "config.h"
#include "constants.h"

namespace adore
{
	namespace core
	{
		class NViewMap;
	}
}

class CalcThread : public QThread
{
		Q_OBJECT
	public:
		CalcThread( const adore::core::NViewMap& nv, const std::vector<ViewT*>& sel,
		            const batch::Config& cfg, bool multithreaded, QObject* parent = 0 );

		~CalcThread() { }

		std::size_t stepsCount()const
		{
			QMutexLocker( &this->mutex );
			return config.getCmdCount();
		}

		void stopDelayed()
		{
			QMutexLocker( &this->mutex );
			stopFlag = true;
		}

		bool stopped() const
		{
			QMutexLocker( &this->mutex );
			return stopFlag;
		}

		const adore::core::NViewMap& getNViewMap();

	signals:
		void sigOpBegin();
		void sigOpEnd();

	private:
		void run();

		bool stopFlag;
		const adore::core::NViewMap& nviewmap_ref;
		adore::core::NViewMap nviewmap;

		const std::vector<ViewT*>& selection;
		batch::Config config;
		bool try_multithreaded;
		std::auto_ptr<adore::core::ViewKnitter> knitter;
		adore::core::Modifier modifier;
		adore::core::ParallelModifier pmodifier;

		mutable QMutex mutex;

		void createViewRelations( const batch::Config& cfg, size_t idx );
		void makePairs();
		void calcFeatures( const batch::Config& cfg, size_t idx );
		void findLines( const batch::Config& cfg, size_t idx );
		void calcDescriptors( const batch::Config& cfg, size_t idx );
		void calcCorrs( const batch::Config& cfg, size_t idx );
		void track( const batch::Config& cfg, size_t idx );
		void normalize( bool norm_denorm );
		void filter( const batch::Config& cfg, size_t idx );
		void filterAndMergeLines( const batch::Config& cfg, size_t idx );
		void calcTensor();
		void projectReconst();
		void metricReconst( const batch::Config& cfg, size_t idx );
		void defaultReconst( const batch::Config & cfg, size_t idx );
		void globalAdjustment( const batch::Config & cfg, size_t idx );

		void mergeIterative();

		void maxWH( int& w, int& h )const;
};

#endif
