

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "cameratourist.h"
#include <iostream>
#include <fstream>
#include <string>
#include <adrvisual/glrenderer.h>
#include <adrvisual/glcamera.h>

using adore::visual::GLCamera;
using adore::visual::GLRenderer;
using adore::visual::Vector3D;
using adore::visual::GLTransform;
using adore::visual::Quaternion;
using std::ifstream;
using std::string;
using std::cout;
using std::endl;

const float MovementAccu::rotFactor ( 0.0005f );
const float MovementAccu::slideFactor ( 0.005f );

CameraTourist::CameraTourist() : startCam(), turnSpine ( 0, 1, 0 ), curStep ( 0 ), subStep ( 0 ), intSpeed ( 10 ), doRepeat ( false ), doReturn ( false ), dotTicks ( 0 ), triTicks ( 1000 )
{
	script.reserve ( 50 );
	//skript actions
	actionIds["ROT_X+"] = 0;
	actionIds["ROT_X-"] = 1;
	actionIds["ROT_Y+"] = 2;
	actionIds["ROT_Y-"] = 3;
	actionIds["SLIDE+"] = 4;
	actionIds["SLIDE-"] = 5;
	actionIds["SPEED"]  = 6;
	actionIds["DOTS+"]  = 34;
	actionIds["DOTS-"]  = 35;
	actionIds["TRIS+"]  = 36;
	actionIds["TRIS-"]  = 37;
	actionIds["NOP"]  = 41;

	//init actions
	actionIds["REPEAT"]      = 7;
	actionIds["CAM_EYE_X"]   = 8;
	actionIds["CAM_EYE_Y"]   = 9;
	actionIds["CAM_EYE_Z"]   = 10;
	actionIds["CAM_SPINE_X"] = 11;
	actionIds["CAM_SPINE_Y"] = 12;
	actionIds["CAM_SPINE_Z"] = 13;
	actionIds["CAM_U_X"]     = 14;
	actionIds["CAM_U_Y"]     = 15;
	actionIds["CAM_U_Z"]     = 16;
	actionIds["CAM_V_X"]     = 17;
	actionIds["CAM_V_Y"]     = 18;
	actionIds["CAM_V_Z"]     = 19;
	actionIds["CAM_N_X"]     = 20;
	actionIds["CAM_N_Y"]     = 21;
	actionIds["CAM_N_Z"]     = 22;
	actionIds["RETURN"]      = 23;//return to start camera
	actionIds["WORLD_ROT_X"] = 24;
	actionIds["WORLD_ROT_Y"] = 25;
	actionIds["WORLD_ROT_Z"] = 26;
	actionIds["WORLD_ROT_W"] = 27;
	actionIds["WORLD_POS_X"] = 28;
	actionIds["WORLD_POS_Y"] = 29;
	actionIds["WORLD_POS_Z"] = 30;
	actionIds["WORLD_SCALE_X"] = 31;
	actionIds["WORLD_SCALE_Y"] = 32;
	actionIds["WORLD_SCALE_Z"] = 33;
	actionIds["WORLD_TURN_X"] = 38;
	actionIds["WORLD_TURN_Y"] = 39;
	actionIds["WORLD_TURN_Z"] = 40;
	actionIds["TRI_TICKS"] = 42;
}


CameraTourist::~CameraTourist()
{
}

//returns true if more to come
bool cutFirstWord ( const string &str, string &cut, string &remains )
{
	size_t found1 = str.find_first_not_of ( " \t" );
	if ( found1 != string::npos )
	{
		size_t found2 = str.find_first_of ( " \t", found1 );
		cut = str.substr ( found1, found2 - found1 );
		if ( found2 == string::npos ) return false;
		else
		{
			remains = str.substr ( found2 );
			return true;
		}
	}
	else
	{
		COUT ( DBGNORMAL, "empty line\n" );
	}
	return false;
}

void CameraTourist::reset()
{
	curStep = 0;
	subStep = 0;
	intSpeed = 10;
	doRepeat = false;
	doReturn = false;
	dotTicks = 0;
	triTicks = 1000;
}

void CameraTourist::readScript ( const char *filename )
{
	COUT ( DBGNORMAL, "tourist reading guide\n" );
	char order[256];
	string what, numRepeats;
	ifstream ifs ( filename, ifstream::in );
	if ( ifs.is_open() )
	{
		script.clear();
		while ( ifs.good() )
		{
			ifs.getline ( order, 256 );
//			printf( "%s\n", order );
			string str ( order );
			if ( cutFirstWord ( str, what, str ) )
			{
				StringIdIter iter = actionIds.find ( what );
				if ( iter != actionIds.end() )
				{
					int id = ( *iter ).second;
					float num = 0.0f;
					cutFirstWord ( str, numRepeats, str );
					num = atof ( numRepeats.c_str() );
//					cout << what << " --- " << numRepeats << "#" << endl;
//					printf( "%d --- %.2f\n", id, num );

					//handle the init commands
					Quaternion rot; Vector3D pos; Vector3D scale;
					switch ( id )
					{
						case 7:
							doRepeat = num != 0 ? true : false;
							break;
						case 8:
							startCam.getEyeLoc() [0] = num;
							break;
						case 9:
							startCam.getEyeLoc() [1] = num;
							break;
						case 10:
							startCam.getEyeLoc() [2] = num;
							break;
						case 11:
							startCam.getSpine() [0] = num;
							break;
						case 12:
							startCam.getSpine() [1] = num;
							break;
						case 13:
							startCam.getSpine() [2] = num;
							break;
						case 14:
							startCam.getU() [0] = num;
							break;
						case 15:
							startCam.getU() [1] = num;
							break;
						case 16:
							startCam.getU() [2] = num;
							break;
						case 17:
							startCam.getV() [0] = num;
							break;
						case 18:
							startCam.getV() [1] = num;
							break;
						case 19:
							startCam.getV() [2] = num;
							break;
						case 20:
							startCam.getN() [0] = num;
							break;
						case 21:
							startCam.getN() [1] = num;
							break;
						case 22:
							startCam.getN() [2] = num;
							break;
						case 23:
							doReturn = num != 0 ? true : false;
							break;
						case 24:
							rot[0] = num;
							break;
						case 25:
							rot[1] = num;
							break;
						case 26:
							rot[2] = num;
							break;
						case 27:
							rot[3] = num;
							break;
						case 28:
							pos[0] = num;
							break;
						case 29:
							pos[1] = num;
							break;
						case 30:
							pos[2] = num;
							break;
						case 31:
							scale[0] = num;
							break;
						case 32:
							scale[1] = num;
							break;
						case 33:
							scale[2] = num;
							break;
						case 38:
							turnSpine[0] = num;
							break;
						case 39:
							turnSpine[1] = num;
							break;
						case 40:
							turnSpine[2] = num;
							break;
						case 42:
							triTicks = int ( num );
							break;
						default:
							script.push_back ( CameraAction ( id, int ( num ) ) );
							break;
					}
					worldTrafo.setRotation ( rot );
					worldTrafo.setTranslation ( pos );
					worldTrafo.setScale ( scale );
				}
				else
				{
					COUT ( DBGNORMAL, "unknown order\n" );
				}
			}
		}
		ifs.close();
	}
	else
	{
		cout << "failed to open " << filename << endl;
	}
	printf ( "found %zd orders\n", script.size() );
	curStep = 0;
	subStep = 0;
}

bool CameraTourist::step ( GLRenderer *visu )
{
	if ( curStep >= ( int ) script.size() )
	{
		if ( doRepeat == true )
		{
			curStep = 0;
			subStep = 0;
		}
		else return false;
	}

	if ( curStep == 0 && subStep == 0 )
	{
		visu->deviationTicks ( triTicks, 1000 );

		if ( doReturn )
		{
			visu->setCamera ( startCam );
			visu->getEntity()->world = worldTrafo;
			visu->getEntity()->setObjUp ( turnSpine );
		}
	}

	CameraAction &action = script[curStep];

	if ( action.what == 6 )
	{
		intSpeed = action.numRepeats;
		subStep = action.numRepeats;
	}

	if ( subStep >= action.numRepeats )
	{
//		printf( "%d >= %d: inc curStep\n", subStep, action.numRepeats );
		++curStep;
		subStep = 0;
		return step ( visu );
	}
	++subStep;

	//do the action
	int what = action.what;
//	printf( "what, substep; %d, %d\n", what, subStep );
	const float x_axis[3] = {1.0f, 0.0f, 0.0f};
//	const float y_axis[3] = {0.0f, 1.0f, 0.0f};
	float delta[3] = {0.0f,0.0f,0.0f};
	float rspeed = MovementAccu::rotFactor * float ( intSpeed );
	float sspeed = MovementAccu::slideFactor * float ( intSpeed );

	//for dots
	static unsigned char penclr[] = { 0xff, 0xff, 0xff, 0xff };

	switch ( what )
	{
		case 0:
			//if (rotateUp)
			visu->rotate ( x_axis, rspeed,false );
			break;
		case 1:
			//if (rotateDown)
			visu->rotate ( x_axis, -rspeed,false );
			break;
		case 2:
			//if (rotateLeft)
//			visu->rotate ( y_axis, rspeed,false );
			visu->turnTable( rspeed, -false );
			break;
		case 3:
			//if (rotateRight)
//			visu->rotate ( y_axis, -rspeed,false );
			visu->turnTable( -rspeed, false );
			break;
		case 4:
			//if (slideForward)
			delta[2] = -10.0f * sspeed;
			visu->slide ( delta );
			break;
		case 5:
			//if (slideBack)
			delta[2] = 10.0f * sspeed;
			visu->slide ( delta );
			break;
		case 34:
			//dots+
			dotTicks += intSpeed;
			visu->penTicks ( penclr, dotTicks, 1000 );
			break;
		case 35:
			//dots-
			dotTicks -= intSpeed;
			visu->penTicks ( penclr, dotTicks, 1000 );
			break;
		case 36:
			//tris+
			triTicks += intSpeed;
//			printf( "+triticks: %d\n", triTicks );
			visu->deviationTicks ( triTicks, 1000 );
			break;
		case 37:
			//tris-
			triTicks -= intSpeed;
//			printf( "-triticks: %d\n", triTicks );
			visu->deviationTicks ( triTicks, 1000 );
			break;
		case 41:
			break;
		default:
			assert ( false );
			break;
	};
	return true;
}


void CameraTourist::printTurnTableSpine ( adore::visual::Vector3D spine )
{
	printf ( "WORLD_TURN_X %.4f\n", spine[0] );
	printf ( "WORLD_TURN_Y %.4f\n", spine[1] );
	printf ( "WORLD_TURN_Z %.4f\n", spine[2] );
}

void CameraTourist::printWorldTrafo ( adore::visual::GLTransform trafo )
{
	Quaternion rot = trafo.getRotation();
	Vector3D pos   = trafo.getTranslation();
	Vector3D scale   = trafo.getScale();
	printf ( "WORLD_POS_X %.4f\n", pos[0] );
	printf ( "WORLD_POS_Y %.4f\n", pos[1] );
	printf ( "WORLD_POS_Z %.4f\n", pos[2] );
	printf ( "WORLD_SCALE_X %.4f\n", scale[0] );
	printf ( "WORLD_SCALE_Y %.4f\n", scale[1] );
	printf ( "WORLD_SCALE_Z %.4f\n", scale[2] );
	printf ( "WORLD_ROT_X %.4f\n", rot[0] );
	printf ( "WORLD_ROT_Y %.4f\n", rot[1] );
	printf ( "WORLD_ROT_Z %.4f\n", rot[2] );
	printf ( "WORLD_ROT_W %.4f\n", rot[3] );
}

void CameraTourist::printCamera ( adore::visual::GLCamera cam )
{
	Vector3D &v = cam.getEyeLoc();
	printf ( "CAM_EYE_X %.4f\n", v[0] );
	printf ( "CAM_EYE_Y %.4f\n", v[1] );
	printf ( "CAM_EYE_Z %.4f\n", v[2] );
	v = cam.getSpine();
	printf ( "CAM_SPINE_X %.4f\n", v[0] );
	printf ( "CAM_SPINE_Y %.4f\n", v[1] );
	printf ( "CAM_SPINE_Z %.4f\n", v[2] );
	v = cam.getU();
	printf ( "CAM_U_X %.4f\n", v[0] );
	printf ( "CAM_U_Y %.4f\n", v[1] );
	printf ( "CAM_U_Z %.4f\n", v[2] );
	v = cam.getV();
	printf ( "CAM_V_X %.4f\n", v[0] );
	printf ( "CAM_V_Y %.4f\n", v[1] );
	printf ( "CAM_V_Z %.4f\n", v[2] );
	v = cam.getN();
	printf ( "CAM_N_X %.4f\n", v[0] );
	printf ( "CAM_N_Y %.4f\n", v[1] );
	printf ( "CAM_N_Z %.4f\n", v[2] );
}
