

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <iostream>
#include <sstream>
#include <algorithm>
#include <cctype>

#include "config.h"

namespace batch
{

	using namespace std;

	Config::Config( const string & text )
	{
		setText( text );
	}

	void Config::setText( const string & text )
	{
		if ( text.empty() ) return;

		Parser parser;
		parser.makeConform( text, program );
		uint cmds = program.cmds.size();
		cout << "(Config::setText): steps count - " << cmds << "\n";
	}

	static string def_str = "Error";

	bool Config::checkCmdIdx( uint idx ) const
	{
		if ( 0 > idx || idx >= program.cmds.size() )
			return false;
		return true;
	}

	const string& Config::getCmd( uint idx )const
	{
		if ( !checkCmdIdx( idx ) ) return def_str;
		return program.cmds[idx].name;
	}

	const string & Config::getAlgorithm( uint idx ) const
	{
		if ( !checkCmdIdx( idx ) || program.cmds[idx].algs.empty() )
			return def_str;
		return program.cmds[idx].algs[0].name;
	}

	int Config::optIdx( uint idx, const string & opt ) const
	{
		if ( !checkCmdIdx( idx ) ) return -1;
		if ( !program.cmds[idx].algs.empty() )
		{
			uint o_cnt = program.cmds[idx].algs[0].opts.size();
			for ( uint i = 0; i < o_cnt; ++i )
			{
				if ( opt == program.cmds[idx].algs[0].opts[i].name )
					return i;
			}
		}
		else
		{
			uint o_cnt = program.cmds[idx].opts.size();
			for ( uint i = 0; i < o_cnt; ++i )
			{
				if ( opt == program.cmds[idx].opts[i].name )
					return i;
			}
		}
		return -1;
	}

	Config::WHAT Config::checkOpt( uint idx, uint o_idx, uint v_idx, const string& type ) const
	{
		Config::WHAT result = Config::ERR;
		if ( !checkCmdIdx( idx ) ) return result;

		if ( !program.cmds[idx].algs.empty() )
		{
			if ( o_idx >= program.cmds[idx].algs[0].opts.size() )
				return result;
			if ( type != program.cmds[idx].algs[0].opts[o_idx].type )
				return result;
			if ( v_idx >= program.cmds[idx].algs[0].opts[o_idx].values.size() )
				return result;
			result = Config::ALG;
		}
		else
		{
			if ( v_idx > program.cmds[idx].opts.size() - 1 )
				return result;
			if ( type != program.cmds[idx].opts[o_idx].type )
				return result;
			if ( v_idx >= program.cmds[idx].opts[o_idx].values.size() )
				return result;
			result = Config::OPT;
		}
		return result;
	}

	bool Config::getOptionInt( uint idx, const string& opt, int & retval, uint v_idx )const
	{
		int o_idx = optIdx( idx, opt );
		if ( 0 > o_idx ) { retval = int(); return false; }

		Config::WHAT what = checkOpt( idx, ( uint )o_idx, v_idx, "int" );

		istringstream istr;
		if ( Config::ALG == what )
			istr.str( program.cmds[idx].algs[0].getValue( opt, v_idx ) );
		else if ( Config::OPT == what )
			istr.str( program.cmds[idx].getValue( opt, v_idx ) );
		else
		{
			retval = int();
			return false;
		}

		int _integer;
		if ( ( istr >> _integer ) )
		{
			retval = _integer;
			return true;
		}
		retval = int();
		return false;
	}

	bool Config::getOptionDouble( uint idx, const string& opt, double & retval, uint v_idx )const
	{
		int o_idx = optIdx( idx, opt );
		if ( 0 > o_idx ) { retval = double(); return false; }

		Config::WHAT what = checkOpt( idx, ( uint )o_idx, v_idx, "double" );

		istringstream istr;
		if ( Config::ALG == what )
			istr.str( program.cmds[idx].algs[0].getValue( opt, v_idx ) );
		else if ( Config::OPT == what )
			istr.str( program.cmds[idx].getValue( opt, v_idx ) );
		else
		{
			retval = double();
			return false;
		}

		double _double;
		if ( ( istr >> _double ) )
		{
			retval = _double;
			return true;
		}
		retval = double();
		return false;
	}

	bool Config::getOptionStr( uint idx, const string & opt, string & retval, uint v_idx ) const
	{
		int o_idx = optIdx( idx, opt );
		if ( 0 > o_idx ) { retval = string(); return false; }

		Config::WHAT what = checkOpt( idx, ( uint )o_idx, v_idx, "enum" );
		if ( Config::ALG == what )
			retval = program.cmds[idx].algs[0].getValue( opt, v_idx );
		else if ( Config::OPT == what )
			retval = program.cmds[idx].getValue( opt, v_idx );
		else
		{
			retval = string();
			return false;
		}
		return true;
	}

	bool Config::getOptionBool( uint idx, const std::string& opt, bool& retval, uint v_idx )const
	{
		int o_idx = optIdx( idx, opt );
		if ( 0 > o_idx ) { retval = false; return false; }

		string result;
		Config::WHAT what = checkOpt( idx, ( uint )o_idx, v_idx, "bool" );
		if ( Config::ALG == what )
			result = program.cmds[idx].algs[0].getValue( opt, v_idx );
		else if ( Config::OPT == what )
			result = program.cmds[idx].getValue( opt, v_idx );
		transform( result.begin(), result.end(), result.begin(), ::tolower );
		retval = ( "yes" == result || "true" == result ) ? true : false;
		return true;
	}

} // namespace batch
