

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <QtCore/QTimer>

#include "controlgl.h"

#include <cmath>

ControlGL::ControlGL() :
		m_is_pressed(false),
		m_is_dragging(false)
{

}

void ControlGL::pressed(double x, double y)
{
	m_is_dragging = false;
	m_is_pressed = true;
	m_last_point = QPointF(x, y);
}

void ControlGL::released(double x, double y)
{
	m_is_pressed = false;
	double dx = x - m_last_point.x();
	double dy = y - m_last_point.y();
	if(!m_is_dragging)
		click(x, y);
	else {
		m_is_dragging = false;
		drag(dx, dy);
	}
}

void ControlGL::moved(double x, double y)
{
	if(!m_is_pressed) return;
	QPointF temp(m_last_point);
	double dx = x - m_last_point.x();
	double dy = y - m_last_point.y();
	m_last_point = QPointF(x, y);
	if(!m_is_dragging) {
		double dev = sqrt(dx * dx + dy * dy);
		if(dev > MaxClickDeviation) {
			m_is_dragging = true;
			beginDragging(temp.x(), temp.y());
		} else {
			m_last_point = temp;
		}
	}
	if(m_is_dragging)
		drag(dx, dy);
}
