

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <QHeaderView>
#include <QTimer>

#include "dirtreewidget.h"

DirTreeWidget::DirTreeWidget( QWidget * parent,DirModel::Mode mode ) :
		QTreeView( parent ),
		model(mode)
{
	setModel( &model );
	hideColumn( 2 ); hideColumn( 3 );
	header()->setVisible( false );
	header()->setResizeMode( 0, QHeaderView::ResizeToContents );
	header()->setResizeMode( 1, QHeaderView::ResizeToContents );
	header()->setStretchLastSection( false );

	connect( this, SIGNAL( activated( const QModelIndex & ) ), this, SLOT( slotItemActivated( const QModelIndex & ) ) );
//	QTimer* timer1 = new QTimer( this );
//	connect( timer1, SIGNAL( timeout() ), this, SLOT( slotRefreshRoot() ) );
//	timer1->start( 5000 );
}

void DirTreeWidget::setFilters( const QStringList & filters )
{
	model.setFileNameFilters( filters );
}

void DirTreeWidget::cd( const QString & path )
{
	QDir dir( path );
	if ( !dir.isAbsolute() || !dir.exists() ) return;
	QVector<QString> paths;
	do
	{
		paths.push_back( dir.canonicalPath() );
		dir.cdUp();
	}
	while ( !dir.isRoot() );
	paths.push_back( dir.canonicalPath() );

	QModelIndex invalidIndex;
	QItemSelectionModel* sel_model = selectionModel();
	if ( !sel_model ) return;

	int ptr = paths.size() - 1;
	QModelIndex index;
	do
	{
		index = model.index( paths[ptr] );
		if ( index == invalidIndex ) return;
		expand( index );
		--ptr;
	}
	while ( 0 <= ptr );
	scrollTo( index, QAbstractItemView::PositionAtTop );
	sel_model->select( index, QItemSelectionModel::SelectCurrent | QItemSelectionModel::Rows );
	setCurrentIndex(index);
}

void DirTreeWidget::slotItemActivated( const QModelIndex & index ) const
{
	if ( model.isDir( index ) )
		emit signalDirActivated( model.toPath( index ) );
	else
		emit signalFileActivated( model.toPath( index ) );
}

void DirTreeWidget::slotRefreshRoot()
{
	model.refresh();
}

QString DirTreeWidget::currentSelection() const
{
	QItemSelectionModel* selection = selectionModel();
	if(!selection->hasSelection())
		return QString();
	QModelIndex idx(selection->currentIndex());
	if(!idx.isValid())
		return QString();
	return model.toPath(idx);
}

