

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "exclusiveoptionsgroup.h"
#include "checkbuttongl.h"

#include <cassert>

using namespace std;

void ExclusiveOptionsGroup::addOption(CheckButtonGL * option, const std::string& optionId)
{
	assert(option != NULL);

	if(m_options.find(optionId) != m_options.end())
		return;
	if(m_options_names.find(option) != m_options_names.end())
		return;

	m_options[optionId] = option;
	m_options_names[option] = optionId;
	option->setState(false);
	connect(option, SIGNAL(toggled(ButtonGL*,bool)), this, SLOT(optionActivated(ButtonGL*)));
}

const std::string& ExclusiveOptionsGroup::activeOption() const
{
	static string noneIsActive("");
	for(OptionsInvMapT::const_iterator it = m_options_names.begin(); it != m_options_names.end(); ++it)
	{
		if(it->first->getState() == true)
			return it->second;
	}
	return noneIsActive;
}

void ExclusiveOptionsGroup::activateOption(const std::string& name)
{
	OptionsT::iterator it = m_options.find(name);
	if(it == m_options.end())
		return;
	it->second->setState(true);
	optionActivated(it->second);
}

void ExclusiveOptionsGroup::optionActivated(ButtonGL * option)
{
	CheckButtonGL * check(static_cast<CheckButtonGL*>(option));
	OptionsInvMapT::iterator it = m_options_names.find(check);
	if(it == m_options_names.end())
		return;
	if(check->getState() != true) {
		check->setState(true);
		return;
	}

	const string& name = it->second;

	for(it = m_options_names.begin(); it != m_options_names.end(); ++it)
	{
		if(it->first == check)
			continue;
		it->first->setState(false);
	}

	emit sigOptionActivated(name);
}
