

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <QtCore/QDir>
#include <QtGui/QTreeView>
#include <QtGui/QVBoxLayout>
#include <QtGui/QShowEvent>
#include <QtGui/QHeaderView>
#include <QtGui/QDialogButtonBox>
#include <QtGui/QApplication>

#include <cassert>

#include "folderdialog.h"
#include "dirtreewidget.h"

QString FolderDialog::getExistingDirectory( QWidget * parent,
                const QString & caption, const QString & dir, const QString& filter )
{
	FolderDialog dlg( filter, parent );
	if ( caption.isEmpty() ) dlg.setWindowTitle( tr( "Select some folder" ) );
	else dlg.setWindowTitle( caption );
	QString currentDir;
	if ( dir.isEmpty() ) currentDir = QDir::homePath();
	else currentDir = dir;

	int width = qMax(dlg.width(), parent->width());
	int height = qMax(dlg.height(), parent->height());
	dlg.resize(width / 2, height / 2);
	dlg.show();
	dlg.cd( currentDir );

	int result = dlg.exec();
	if ( QDialog::Rejected == result ) return QString();
	return dlg.selectedPath;
}

FolderDialog::FolderDialog( const QString& filters, QWidget * parent, Qt::WindowFlags flags )
		: QDialog( parent, flags )
{

	view = new DirTreeWidget( this, DirModel::Dirs );
	view->setFilters(filters.split( ";", QString::SkipEmptyParts ));
	connect( view, SIGNAL( signalDirActivated( const QString & ) ),
			 this, SLOT( slotFolderSelected( const QString & ) ) );

	QVBoxLayout* layout = new QVBoxLayout;
	layout->addWidget( view );

	QDialogButtonBox* buttons = new QDialogButtonBox( QDialogButtonBox::Ok |
	                QDialogButtonBox::Cancel );
	connect( buttons, SIGNAL( rejected() ), this, SLOT( reject() ) );
	connect( buttons, SIGNAL( accepted() ), this, SLOT( accept() ) );
	layout->addWidget( buttons );
	setLayout( layout );

	view->setFocus( Qt::OtherFocusReason );
}

void FolderDialog::slotFolderSelected( const QString & path )
{
	selectedPath = path;
	QDialog::accept();
}

void FolderDialog::accept()
{
	selectedPath = view->currentSelection();
	QDialog::accept();
}

void FolderDialog::cd( const QString & path )
{
	view->cd( path );
}

void FolderDialog::showEvent( QShowEvent * event )
{
	if ( event->spontaneous() ) return QDialog::showEvent( event );
	QItemSelectionModel* selmodel = view->selectionModel();
	if ( selmodel )
	{
		QModelIndexList list( selmodel->selectedRows() );
		if ( !list.isEmpty() )
		{
			view->scrollTo( list.at( 0 ) );
			view->setCurrentIndex( list.at( 0 ) );
		}
	}
	return QDialog::showEvent( event );
}
