

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include <QScrollArea>

#include "inspector.h"
#include "nviewstatistics.h"

#include <adrcore/util/mathutil.h>
#include <adrcore/test/assessor.h>

#include <cassert>

Inspector::Inspector( NView *nv, QWidget* parent )
		: QDialog( parent ), nview( nv )
{
	assert(nview);

	ui.setupUi( this );
	cameraTab = ui.tabs->widget( 0 );
	QScrollArea* sa = new QScrollArea;
	QLayout* layout = new QHBoxLayout;
	layout->addWidget( sa );
	delete cameraTab->layout();
	cameraTab->setLayout( layout );

	cameraPanel = new QWidget;
	int cnt = 0;
	layout = new QHBoxLayout;
	for ( NView::CameraIterator it = nview->getCamerasBegin(); it != nview->getCamerasEnd(); ++it )
	{
		CameraWidget* w = new CameraWidget;
		layout->addWidget( w );
		cw.push_back( w ); cnt++;
	}
	if ( cnt )
	{
		cameraPanel->setLayout( layout );
		sa->setWidget( cameraPanel );
	}

	errorTab = ui.tabs->widget( 1 );
	sa = new QScrollArea;
	layout = new QHBoxLayout;
	layout->addWidget( sa );
	errorTab->setLayout( layout );

	errorPanel = new QWidget;
	layout = new QHBoxLayout;
	ErrorWidget* w = new ErrorWidget( "Summary" );
	layout->addWidget( w );
	ew.push_back( w );
	for ( NView::ViewIterator it = nview->getViewsBegin(); it != nview->getViewsEnd(); ++it )
	{
		ErrorWidget* w = new ErrorWidget( "View" );
		layout->addWidget( w );
		ew.push_back( w );
	}
	errorPanel->setLayout( layout );
	sa->setWidget( errorPanel );

	nviewStatistics = new NViewStatistics(this);
	ui.tabs->addTab(nviewStatistics, tr("&Statistics"));

	recalc();
}

Inspector::~Inspector() {}

void Inspector::fillCamViews()
{
	AVector2D axis( 3 );
	int i = 0;
	for ( NView::CameraIterator it = nview->getCamerasBegin(); it != nview->getCamerasEnd(); ++it, ++i )
	{
		Camera *c = *it;
		c->decompose();
		AVector3D loc = c->getLocation();

		cw[i]->setPosX( loc( 0 ) );
		cw[i]->setPosY( loc( 1 ) );
		cw[i]->setPosZ( loc( 2 ) );

		AMatrix R = c->getRotationMatrix();
		cw[i]->setR00( R( 0, 0 ) );
		cw[i]->setR01( R( 0, 1 ) );
		cw[i]->setR02( R( 0, 2 ) );
		cw[i]->setR10( R( 1, 0 ) );
		cw[i]->setR11( R( 1, 1 ) );
		cw[i]->setR12( R( 1, 2 ) );
		cw[i]->setR20( R( 2, 0 ) );
		cw[i]->setR21( R( 2, 1 ) );
		cw[i]->setR22( R( 2, 2 ) );

		if ( isRightHand( R ) )
		{
			double angle;
			axis = anax.matToAngleAxis( R, angle );
			cw[i]->setRotX( axis( 0 ) );
			cw[i]->setRotY( axis( 1 ) );
			cw[i]->setRotZ( axis( 2 ) );
			cw[i]->setAngle( angle );
		}
		else
		{
			cw[i]->setRotX( -1 );
			cw[i]->setRotY( -1 );
			cw[i]->setRotZ( -1 );
			cw[i]->setAngle( -1 );
		}

		AMatrix K = c->getCalibrationMatrix();
		assert( K( 2, 2 ) > 0.0 );
		cw[i]->setFlX( K( 0, 0 ) );
		cw[i]->setFlY( K( 1, 1 ) );
		cw[i]->setSkew( K( 0, 1 ) );
		cw[i]->setPX( K( 0, 2 ) );
		cw[i]->setPY( K( 1, 2 ) );
		cw[i]->show();
	}
}

void Inspector::fillErrorViews()
{
	Assessor assessor;
	assessor.assess( *nview );

	vector<double> vals;
	vals.reserve( 50 );

	ew[0]->setSum( assessor.getReproErrorSum() );
	ew[0]->setMin( assessor.getReproErrorMin() );
	ew[0]->setMax( assessor.getReproErrorMax() );
	ew[0]->setAvg( assessor.getReproErrorAvg() );
	ew[0]->setVar( assessor.getReproErrorVar() );
	assessor.getReproErrors( vals );
	ew[0]->drawErrors( vals );

	ViewErrors *ve = NULL;
	int i = 1;
	for ( NView::ViewIterator it = nview->getViewsBegin(); it != nview->getViewsEnd(); ++it, ++i )
	{
		vals.clear();
		ve = assessor.getErrorsOf( *it );
		ew[i]->setSum( ve->getReproErrorSum() );
		ew[i]->setMin( ve->getReproErrorMin() );
		ew[i]->setMax( ve->getReproErrorMax() );
		ew[i]->setAvg( ve->getReproErrorAvg() );
		ew[i]->setVar( ve->getReproErrorVar() );
		ve->getReproErrors( vals );
		ew[i]->drawErrors( vals );
	}
}

void Inspector::fillStatistics()
{
	nviewStatistics->setData(nview);
}

void Inspector::recalc()
{
	fillCamViews();
	fillErrorViews();
	fillStatistics();
}

void Inspector::maximizeHor()
{
	QRect rect = frameGeometry();
	int x = rect.x();
	int y = rect.y();
	int height = rect.height();
	showFullScreen();
	rect = frameGeometry();
	int width = rect.width();
	showNormal();
	printf( "%d, %d, %d, %d\n", x, y, width, height );
	setGeometry( x, y, width, height );
}
