

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin
/// Author: Klaus Haeming


#include <QApplication>
#include <QTranslator>
#include <QGLFormat>
#include <QLocale>

#include "mainwindow.h"
#include "controller.h"

#include <cstdio>
#include <cstdlib>
#include <getopt.h>
#include <string>
#include <iostream>

using namespace std;

enum
{
	ADR_LANG_DE, ADR_LANG_EN, ADR_LANG_INVAL
};

int main( int argc, char *argv[] )
{
	Q_INIT_RESOURCE( nview );
	QApplication app( argc, argv );

	if ( !QGLFormat::hasOpenGL() )
	{
		cout << "QGLFormat::hasOpenGL() returned FALSE." << endl;
		return -1;
	}

	QTranslator qt_translator;
	qt_translator.load( ":resources/qt_de.qm" );
	QTranslator translator;
	translator.load( ":resources/nview_de.qm" );

	std::string configFile( "" );

	bool hasRecipe = false;
	std::string recipeFile( "" );

	MainWindow::Mode work_mode = MainWindow::NonDemo;
	int ret;

	int langval = ADR_LANG_DE;

	while ( 1 )
	{
		int option_index = 0;
		static struct option long_options[] =
		{
			{"glfile", 1, 0, 'g'},
			{"recipe", 1, 0, 'r'},
			{"lang", 1, 0, 'l'},
			{"demo", 0, 0, 'd'},
			{"nondemo", 0, 0, 'n'},
			{0, 0, 0, 0}
		};

		ret = getopt_long( argc, argv, "g:r:l:sdn", long_options, &option_index );
		if ( ret == -1 )
			break;

		switch ( ret )
		{
			case 'd':
				work_mode = MainWindow::Demo;
				printf( "mode: DEMO\n" );
				break;

			case 'n':
				work_mode = MainWindow::NonDemo;
				printf( "mode: NON-DEMO\n" );
				break;

			case 'g':
				if ( optarg )
				{
					configFile = optarg;
					printf( "GLconfigFile is %s\n", configFile.c_str() );
				}
				break;

			case 'r':
				if ( optarg )
				{
					recipeFile = optarg;
					hasRecipe = true;
					printf( "recipeFile is %s\n", recipeFile.c_str() );
				}
				break;

			case 'l':
				if ( optarg )
				{
					if ( !strncmp( "en", optarg, 2 ) )
					{
						DBGOUT( DBGNORMAL, "using the English language\n" );
						langval = ADR_LANG_EN;
					}
					else if ( !strncmp( "de", optarg, 2 ) )
					{
						DBGOUT( DBGNORMAL, "nutze die Deutsche Sprache\n" );
						langval = ADR_LANG_DE;
					}
					else
					{
						langval = ADR_LANG_INVAL;
					}
				}
				break;

			case '?':
				break;

			case 0:
			default:
				printf ( "invalid option: 0%o\n", ret );
				exit( 1 );
		}
	}

	switch ( langval )
	{
		case ADR_LANG_DE:
			QLocale::setDefault(QLocale("de"));
			app.installTranslator( &qt_translator );
			app.installTranslator( &translator );
			break;
		case ADR_LANG_EN:
			QLocale::setDefault(QLocale("en"));
			break;
		default:
			printf( "invalid language choice\n" );
			break;
	}

	MainWindow w( work_mode );
	Controller c( &w, configFile );
	w.show();
	if ( hasRecipe ) w.loadRecipe( recipeFile );

	if ( optind < argc )
	{
		QString file( argv[optind] );
		w.load( file );
	}
	return app.exec();
}
