

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming
/// Author: Sergey Cheremukhin


#include <QtGui/QFileDialog>
#include <QtGui/QStatusBar>
#include <QtGui/QMouseEvent>
#include <QtGui/QGraphicsScene>
#include <QtGui/QMessageBox>
#include <QtGui/QSlider>
#include <QtCore/QSettings>
#include <QtGui/QToolBar>
#include <QtCore/QProcess>
#include <QtGui/QTabWidget>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QMenu>
#include <QtGui/QMenuBar>

#include "config.h"

#include "mainwindow.h"
#include "selector.h"
#include "glwidget.h"
#include "nviewlist.h"
#include "controller.h"
#include "demowidget.h"
#include "folderdialog.h"
#include "dirtreewidget.h"
#include "featuresview.h"
#include "splitter.h"
#include "aboutbox.h"

MainWindow::MainWindow( Mode mode, QWidget* parent )
		: QMainWindow( parent ), working_mode( mode ),
		selector( 0 ), fView( NULL ),
		glView( 0 ), controller( 0 ), aboutBox( 0 )
{
	setupUi( this );

	simpleSaving = embedImages = false;
	storeDescr = true;

	QSizePolicy p( QSizePolicy::Expanding, QSizePolicy::Expanding );
	fView = new FeaturesView;
	fView->setSizePolicy( p );

	glView = new GLWidget( QGLFormat( QGL::DoubleBuffer | QGL::DepthBuffer | QGL::Rgba ) );
	QGLFormat format = glView->format();
	bool ok = format.doubleBuffer();
	if ( !ok ) cerr << "I have no doubleBuffer!!!!!!!!!" << endl;

	glView->setSizePolicy( p );
	glView->setMouseTracking( true );
	setMouseTracking( true );

	m_splitter3d_2d = new Splitter(Qt::Vertical, this);
	m_splitter_3d_conf = new Splitter(Qt::Horizontal, m_splitter3d_2d);
	m_splitter_3d_conf->addWidget(glView);
	m_splitter3d_2d->addWidget(m_splitter_3d_conf);
	m_splitter3d_2d->addWidget(fView);

	setCentralWidget(m_splitter3d_2d);

	list = new NViewList;

	if ( NonDemo == working_mode ) setupNonDemo();
	else setupDemo();

	readSettings();
	visualNameLabel = 0;

	QFont ccfont; ccfont.setPointSize(9);
	QLabel * l1 = new QLabel(tr("This work is licensed under"));
	l1->setFont(ccfont);
	QLabel * l2 = new QLabel;
	l2->setPixmap(QPixmap(":/images/CC_BY-NC-SA_de_80x15.png"));
	QLabel * l3 = new QLabel(trUtf8("FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion"));
	l3->setFont(ccfont);
	statusBar()->insertWidget( 0, l1 );
	statusBar()->insertWidget( 1, l2 );
	statusBar()->insertWidget( 2, l3 );
}

MainWindow::~MainWindow() {}

void MainWindow::setupNonDemo()
{
	QTabWidget* tabs = new QTabWidget;
	m_splitter_3d_conf->addWidget(tabs);
	m_splitter_recept_tree = new Splitter(Qt::Vertical, tabs);
	tabs->addTab( m_splitter_recept_tree, tr( "Recipe and Tree" ) );

	selector = new Selector( this );
	m_splitter_recept_tree->addWidget( selector );
	m_splitter_recept_tree->addWidget( list );

	browser = new DirTreeWidget( tabs );
	browser->setFilters( QStringList( "*.che" ) );
	connect( browser, SIGNAL( signalFileActivated( const QString& ) ), this, SLOT( load( const QString& ) ) );
	tabs->addTab( browser, tr( "Browser" ) );

	createActions();
	createMenus();
	createToolbars();
}

void MainWindow::setupDemo()
{
	DemoWidget* w = new DemoWidget( this );
	m_splitter_3d_conf->addWidget( w );
	selector = new Selector( ":resources/default2.txt", this );
	selector->setVisible( false );
	glView->slotCamsOFF();
	fView->setShowPointCorrs(false);
}

void MainWindow::readSettings()
{
	QString set_name( "NVIEW_WORK" );
	if ( Demo == working_mode ) set_name = ( "NVIEW_DEMO" );

	QSettings* settings = new QSettings( "ADORE2", set_name );
	if ( !settings->contains( "Splitter_3d_2d_Data" ) ||
					!settings->contains( "Splitter_3d_Conf_Data" ) ||
					!settings->contains( "Splitter_recept_tree_Data" ) )
	{
		delete settings;
		if ( NonDemo == working_mode )
			settings = new QSettings( ":resources/NVIEW_WORK.conf", QSettings::IniFormat );
		else
			settings = new QSettings( ":resources/NVIEW_DEMO.conf", QSettings::IniFormat );
	}

	QPoint pos = settings->value( "pos", QPoint( 0, 0 ) ).toPoint();
	QSize size = settings->value( "size", QSize( 1024, 768 ) ).toSize();
	resize( size ); move( pos );
	m_splitter3d_2d->restoreState( settings->value( "Splitter_3d_2d_Data" ).toByteArray() );
	m_splitter_3d_conf->restoreState( settings->value( "Splitter_3d_Conf_Data" ).toByteArray() );
	if ( Demo != working_mode && m_splitter_recept_tree != NULL )
		m_splitter_recept_tree->restoreState( settings->value( "Splitter_recept_tree_Data" ).toByteArray() );

	lastDir = settings->value( "lastDir", QDir::homePath() ).toString();
	if ( Demo != working_mode && browser != NULL )
		browser->cd( lastDir );
	lastImgDir = settings->value( "lastImgDir", QDir::homePath() ).toString();

	delete settings;

	if( working_mode == Demo ) {
		QList<int> sizes;
		sizes.push_back(10000);
		sizes.push_back(0);
		m_splitter3d_2d->setSizes(sizes);
	}
}

void MainWindow::writeSettings()
{
	QString set_name( "NVIEW_WORK" );
	if ( Demo == working_mode ) set_name = ( "NVIEW_DEMO" );

	QSettings settings( "ADORE2", set_name );
	settings.setValue( "pos", pos() );
	settings.setValue( "size", size() );
	settings.setValue( "Splitter_3d_2d_Data", m_splitter3d_2d->saveState() );
	settings.setValue( "Splitter_3d_Conf_Data", m_splitter_3d_conf->saveState() );
	if ( Demo != working_mode && m_splitter_recept_tree != NULL )
		settings.setValue( "Splitter_recept_tree_Data", m_splitter_recept_tree->saveState() );
	settings.setValue( "lastDir", lastDir );
	settings.setValue( "lastImgDir", lastImgDir );
}

void MainWindow::closeEvent( QCloseEvent * event )
{
	writeSettings();
}

void MainWindow::setController( Controller *c )
{
	controller = c;
	if ( !c ) return;
}

void MainWindow::open()
{
	QString dir( lastDir );
	char* envDir = getenv( "NVIEW3D_MODEL_LOAD_DIR" );
	if ( envDir ) dir = envDir;

	QString fn( QFileDialog::getOpenFileName( this, tr( "Open from ..." ), dir, "serialized (*.che)", NULL, QFileDialog::DontUseNativeDialog ) );
	if ( fn.isEmpty() ) statusBar()->showMessage( tr( "Loading aborted" ), 2000 );
	else
	{
		lastCHE = fn;
		load( lastCHE );
	}
}

void MainWindow::findGoodZoom()
{
	fView->calculateGoodZoom();
}

void MainWindow::openImageDir()
{
	QString imgDir( lastImgDir );
	char* envDir = getenv( "NVIEW3D_IMAGE_LOAD_DIR" );
	if ( envDir ) imgDir = envDir;

	QString directory( FolderDialog::getExistingDirectory( this, tr( "Directory Chooser" ),
	                   imgDir, "*.png;*.jpg;*.PNG;*.JPG" ) );
	if ( directory.isEmpty() ) return;

	QDir dir( directory );
	QStringList fileList( dir.entryList( QStringList() << "*.png" << "*.jpg" << "*.PNG" << "*.JPG", QDir::Files, QDir::Name ) );
	if ( fileList.empty() ) return;

	controller->addViews( fileList, dir );
	if ( !envDir ) lastImgDir = directory;
	findGoodZoom();
}

void MainWindow::slotLoadFromCamera()
{
	if ( downloadImageScript() ) findGoodZoom();
}

bool MainWindow::downloadImageScript()
{
	QProcess proc( this );

	const char *defaultEnvDir = "/home/images";
	const char *defaultEnvCreate = "takenshots";

	const char *envDir = getenv( "NVIEW3D_IMAGE_SAVE_DIR" );
	if ( !envDir ) envDir = defaultEnvDir;
	const char *envCreate = getenv( "NVIEW3D_CREATE_DIR_PREFIX" );
	if ( !envCreate ) envCreate = defaultEnvCreate;

	QString directory( envDir );
	QDir dir( directory );
	QString filter( envCreate );
	filter.append( "???" );
	QStringList fileList = dir.entryList( QStringList() << filter, QDir::Dirs, QDir::Name );

	QString file( envCreate );
	file.append( "000" );
	if ( fileList.count() > 0 )
	{
		QString last = fileList.last();
		QString right = last.right( 3 );
		int count = right.toInt();
		++count;
		QString num;
		num.setNum( count );
		file.replace( file.length() - num.length(), num.length(), num );
	}

	directory += "/";
	directory += file;
	printf( "argument von shell-script: %s\n", directory.toAscii().constData() );

	proc.setProcessChannelMode( QProcess::ForwardedChannels );
	proc.start( "downloadAndDeleteImages.sh", QStringList() << directory );
	if ( !proc.waitForStarted( 1000 ) )
	{
		QMessageBox::information( this, tr( "Image loader" ), tr( "Loading of images has failed" ), QMessageBox::Ok );
		return false;
	}
	else
	{
		if ( !proc.waitForFinished ( 300000 ) )//wait 5 minutes
		{
			QMessageBox::information( this, tr( "Image loader" ), tr( "Loading of images has probably failed" ), QMessageBox::Ok );
			return false;
		}

		QDir dir( directory );
		QStringList fileList( dir.entryList( QStringList() << "*.png" << "*.jpg" << "*.PNG" << "*.JPG", QDir::Files, QDir::Name ) );
		if ( fileList.empty() )
		{
			printf( "filelist empty, path was %s\n", dir.path().toAscii().constData() );
			return false;
		}

		controller->addViews( fileList, dir );
	}
	return true;
}

void MainWindow::save()
{
	printf( "saving\n" );
	if ( lastCHE.isEmpty() ) saveAs();
	else controller->save( lastCHE, simpleSaving, embedImages, storeDescr );
}

void MainWindow::saveAs()
{
	static const QString ext( ".che" );

	QString dir( lastDir );
	char* envDir = getenv( "NVIEW3D_MODEL_SAVE_DIR" );
	if ( envDir ) dir = envDir;

	QString fn( QFileDialog::getSaveFileName( this, tr( "Save in ..." ), dir, "serialized (*.che)", NULL, QFileDialog::DontUseNativeDialog ) );
	if ( !fn.isEmpty() )
	{
		if ( !fn.endsWith( ext ) ) fn += ext;
		lastCHE = fn; save();
		if ( !envDir )
			lastDir = QFileInfo( lastCHE ).absolutePath();
	}
	else statusBar()->showMessage( tr( "Saving aborted" ), 2000 );
}

void MainWindow::to3D()
{
	controller->to3D();
}

void MainWindow::to3D_2()
{
	controller->to3D( true );
}

NViewList* MainWindow::getSetsView()
{
	return list;
}

batch::Config MainWindow::getConfig() const
{
	batch::Config cfg( selector->makeScript().toStdString() );
	return cfg;
}

void MainWindow::createActions()
{
	aboutAct = new QAction( tr( "&about Adore" ), this );
	connect( aboutAct, SIGNAL( triggered() ), this, SLOT( slotShowAboutBox()) );
	
	changeVisualAct = new QAction( tr( "change Visualiser" ), this );
	connect( changeVisualAct, SIGNAL( triggered() ), this, SLOT( slotChangeVisualizer() ) );
	toggleCamsAct = new QAction( tr( "toggle Cams" ), this );
	connect( toggleCamsAct, SIGNAL( triggered() ), glView, SLOT( slotToggleShowCams() ) );
	resetAllButViewsAct = new QAction( tr( "reset ALL but not Views" ), this );
	connect( resetAllButViewsAct, SIGNAL( triggered() ), this, SLOT( slotResetButViews() ) );
	embedImagesAct = new QAction( tr( "Embed images" ), this );
	embedImagesAct->setCheckable( true );
	connect( embedImagesAct, SIGNAL( toggled( bool ) ), this, SLOT( setEmbedImages( bool ) ) );
	connect( actionTo_3D_2, SIGNAL( triggered() ), this, SLOT( to3D_2() ) );
	storeDescriptorsAct = new QAction( tr( "Store descriptors" ), this );
	storeDescriptorsAct->setCheckable( true );
	storeDescriptorsAct->setChecked( true );
	connect( storeDescriptorsAct, SIGNAL( toggled( bool ) ), this, SLOT( setStoreDescrs( bool ) ) );
	connect( actionRemove_epipolar_lines, SIGNAL( triggered() ), fView, SLOT( removeEpipolarLines()) );

	m_showPointsAct = new QAction(tr("Show point features"), this);
	m_showPointsAct->setCheckable(true);
	m_showPointsAct->setChecked(true);
	connect(m_showPointsAct, SIGNAL(toggled(bool)), fView, SLOT(setShowFeats(bool)));

	m_showLinesAct = new QAction(tr("Show line features"), this);
	m_showLinesAct->setCheckable(true);
	m_showLinesAct->setChecked(true);
	connect(m_showLinesAct, SIGNAL(toggled(bool)), fView, SLOT(setShowLines(bool)));

	m_showPcorrsAct = new QAction(tr("Show point correspondences"), this);
	m_showPcorrsAct->setCheckable(true);
	m_showPcorrsAct->setChecked(true);
	connect(m_showPcorrsAct, SIGNAL(toggled(bool)), fView, SLOT(setShowPointCorrs(bool)));

	m_showLcorrsAct = new QAction(tr("Show line correspondences"), this);
	m_showLcorrsAct->setCheckable(true);
	m_showLcorrsAct->setChecked(true);
	connect(m_showLcorrsAct, SIGNAL(toggled(bool)), fView, SLOT(setShowLineCorrs(bool)));
}

void MainWindow::createMenus()
{
	QMenu* file_menu = menuBar()->addMenu( tr( "&File" ) );
	file_menu->addAction( action_Open );
	file_menu->addAction( action_Save );
	file_menu->addAction( actionSave_as );
	file_menu->addAction( action_Close );

	QMenu* view_menu = menuBar()->addMenu( tr("&View") );
	view_menu->addAction( m_showPointsAct );
	view_menu->addAction( m_showLinesAct );
	view_menu->addAction( m_showPcorrsAct );
	view_menu->addAction( m_showLcorrsAct );

	QMenu* act_menu = menuBar()->addMenu( tr( "&Actions" ) );
	act_menu->addAction( actionOpen_Images );
	act_menu->addAction( actionTo_3D );
	act_menu->addAction( actionTo_3D_2 );
	act_menu->addAction( actionRemove_epipolar_lines );

	QMenu* opt_menu = menuBar()->addMenu( tr( "&Options" ) );
	opt_menu->addAction( changeVisualAct );
	opt_menu->addAction( toggleCamsAct );
	opt_menu->addAction( resetAllButViewsAct );
	opt_menu->addAction( embedImagesAct );
	opt_menu->addAction( storeDescriptorsAct );
	
	QMenu* help_menu = menuBar()->addMenu( tr( "&Help" ) );
	help_menu->addAction( aboutAct );
}

void MainWindow::createToolbars()
{
	QToolBar* toolbar = addToolBar( tr( "Toolbar" ) );
	toolbar->addAction( action_Open );
	toolbar->addAction( action_Save );
	toolbar->addSeparator();
	toolbar->addAction( actionOpen_Images );
	toolbar->addAction( actionTo_3D );
	toolbar->addAction( actionTo_3D_2 );
}

void MainWindow::load( const QString & file )
{
	controller->load( file );
	char* envDir = getenv( "NVIEW3D_MODEL_LOAD_DIR" );
	if ( !envDir ) lastDir = QFileInfo( file ).absolutePath();

	fView->calculateGoodZoom();
}

void MainWindow::slotChangeVisualizer()
{
	controller->resetHighlights();
	glView->nextVisualizer();
	QString str( "GLVisualizer: " );
	str += glView->visualizerName();
	if ( !visualNameLabel )
	{
		visualNameLabel = new QLabel( str );
		statusBar()->addPermanentWidget( visualNameLabel );
	}
	else visualNameLabel->setText( str );
}

void MainWindow::slotShowAboutBox()
{
	if( !aboutBox )
	{
		aboutBox = new AboutBox(this);
	}
	aboutBox->resetLink();
	aboutBox->open();
}

void MainWindow::slotStandard()
{
	controller->toStandardCamera();
}

void MainWindow::slotReset()
{
	controller->resetAll();
}

void MainWindow::slotResetButViews()
{
	controller->resetAllButViews();
}

void MainWindow::setSimpleSaving( bool on )
{
	simpleSaving = on;
}

void MainWindow::setEmbedImages( bool on )
{
	embedImages = on;
}

void MainWindow::loadRecipe( std::string &r )
{
	selector->load( r.c_str() );
}

void MainWindow::setStoreDescrs( bool on )
{
	storeDescr = on;
}
