

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "nviewlist.h"
#include <QStandardItemModel>
#include <QHeaderView>

#include <adrcore/test/assessor.h>

NViewList::NViewList( QWidget * parent )
		: QTreeView( parent )
{
	QTreeView::header()->setResizeMode( QHeaderView::ResizeToContents );
	model = new QStandardItemModel;
	setModel( model );

	connect( this, SIGNAL( doubleClicked( const QModelIndex& ) ),
	         this, SLOT( slotItemDoubleClicked( const QModelIndex& ) ) );
}

void NViewList::setupHeader( int vcnt )
{
	int col = model->columnCount();
	if ( col < vcnt + 2 )
	{
		QStringList header;
		header << "structure" << "SqrtRESum";
		for ( int i = 0; i < vcnt; i++ )
			header << QString::number( i + 1 );
		model->setHorizontalHeaderLabels( header );
	}
}

QList< QStandardItem * > NViewList::makeRow( const QString & label1, NView * nv ) const
{
	Assessor assess;
	assess.assess( *nv );
	QList<QStandardItem*> items;
	items << new NViewListItem( label1, nv );
	QString s = QString::number( sqrt( assess.getReproErrorSum() ), 'f', 12 );
	items << new QStandardItem( s );
	for ( NView::ViewIterator vit = nv->getViewsBegin(); vit != nv->getViewsEnd(); ++vit )
	{
		QString s = QString::number( sqrt( assess.getErrorsOf( *vit )->getReproErrorSum() ), 'f', 12 );
		items << new QStandardItem( s );
	}
	for ( int i = 0; i < items.size(); i++ ) items[i]->setEditable( false );
	return items;
}

NView * NViewList::newRow( const QString & label1, NView * nv )
{
	if ( !nv ) return 0;
	DBGOUT( DBGNORMAL, "direct - toAdd: %p\n", nv );
	setupHeader( nv->getNumViews() );
	QList<QStandardItem*> row( makeRow( label1, nv ) );
	nvToItem[nv] = row.front();
	model->invisibleRootItem()->appendRow( row );
	return nv;
}

NView * NViewList::newRow( NView * parent, const QString & label1, NView * nv )
{
	DBGOUT( DBGNORMAL, "!below - parent: %p, toAdd: %p\n", parent, nv );
	QStandardItem* item = nvToItem[parent];
	assert( item );
	QList<QStandardItem*> row = makeRow( label1, nv );
	nvToItem[nv] = row.front();
	item->appendRow( row );
	return nv;
}

NView * NViewList::newRow( NView * parent, NView * below,
                           const QString & label1, NView * nv )
{
	DBGOUT( DBGNORMAL, "below - parent: %p, below:%p, toAdd: %p\n", parent, below, nv );
	QStandardItem* item = nvToItem[parent];
	assert( item );
	int chcnt = item->rowCount();
	for ( int j = 0; j < chcnt; j++ )
	{
		NViewListItem* child = dynamic_cast<NViewListItem*>( item->child( j ) );
		if ( child )
		{
			item->appendRow( makeRow( label1, nv ) );
			return nv;
		}
	}
	return 0;
}

void NViewList::clear()
{
	model->clear();
	nvToItem.clear();
}

void NViewList::slotItemDoubleClicked( const QModelIndex & index )
{
	QModelIndex idx = index.sibling( index.row(), 0 );
	NViewListItem* item = dynamic_cast<NViewListItem*>( model->itemFromIndex( idx ) );
	if ( !item )
	{
		printf( "(NViewList::slotItemDoubleClicked): model is broken!?" );
		return;
	}
	emit sigItemDoubleClicked( item->getNView() );
}

void NViewList::currentChanged( const QModelIndex & current, const QModelIndex & previous )
{
	if ( current == previous ) return;
	QModelIndex idx = current.sibling( current.row(), 0 );
	NViewListItem* item = dynamic_cast<NViewListItem*>( model->itemFromIndex( idx ) );
	if ( !item )
	{
		printf( "(NViewList::slotItemClicked): model is broken!?" );
		return;
	}
	emit sigItemSelected( item->getNView() );
}

NView * NViewList::selectFirst()
{
	QStandardItem* item = model->item( 0 );
	if ( !item ) return 0;
	NViewListItem* nitem = dynamic_cast<NViewListItem*>( item );
	if ( !nitem ) return 0;
	setCurrentIndex( item->index() );
	return nitem->getNView();
}

void NViewList::deselect()
{
	clearSelection();
}
