

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <QProgressBar>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTimeLine>
#include <QPushButton>
#include <QTextBrowser>
#include <QCloseEvent>
#include <QTimer>

#include "progressdialog.h"
#include "calcthread.h"

const int aniSteps = 40;
const int aniPause = 250;

ProgressDialog::ProgressDialog( CalcThread * thr, QWidget * parent, bool show_list )
		:QDialog( parent ), thread( thr ), timer( new QTimer( this ) ), aniPhase( 1 ), aniPos( 0 )
{
	setWindowFlags( Qt::Dialog | Qt::CustomizeWindowHint | Qt::WindowTitleHint );
	resize( int( 0.618f * float( parent->width() ) ), height() );
	setWindowTitle( tr( "Progress of reconstruction" ) );

	connect( thread, SIGNAL( finished() ), this, SLOT( accept() ) );
	connect( thread, SIGNAL( sigOpBegin() ), this, SLOT( slotThreadOpBegin() ) );
	connect( thread, SIGNAL( sigOpEnd() ), this, SLOT( slotThreadOpEnd() ) );

	progressBar = new QProgressBar( this );
	progressBar->setRange( 0, aniSteps * thread->stepsCount() );
	progressBar->setValue( 0 );
	progressBar->setTextVisible( false );

	QPushButton* b = new QPushButton( tr( "Abort after current" ), this );
	connect( b, SIGNAL( pressed() ), this, SLOT( slotAbortAfterNext() ) );

	QHBoxLayout* l_b = new QHBoxLayout;
	l_b->addWidget( b );

	log = new QTextBrowser( this );

	QVBoxLayout* l = new QVBoxLayout( this );

	if ( show_list ) l->addWidget( log );
	else log->setVisible( false );

	l->addWidget( progressBar );
	l->addLayout( l_b );
	setLayout( l );

	connect( timer, SIGNAL( timeout() ), this, SLOT( timetick() ) );
	timer->start( aniPause );

	thread->start();
}

void ProgressDialog::slotAbortAfterNext()
{
	thread->stopDelayed();
	QObject* obj = sender();
	if ( obj )
	{
		QPushButton* b = qobject_cast<QPushButton*>( obj );
		if ( b ) b->setEnabled( false );
	}
}

void ProgressDialog::slotThreadOpBegin()
{
	int value = aniPhase;
	int end = thread->stepsCount();
	appendText( tr( "Step %1 of %2 ... " ).arg( value ).arg( end ) );
}

void ProgressDialog::slotThreadOpEnd()
{
	appendText( tr( "SUCCEEDED" ) + "\n" );
	//progressBar->setValue(progressBar->value() + 1);
	aniPos = aniSteps * aniPhase;
	++aniPhase;
}

void ProgressDialog::appendText( const QString & text )
{
	QTextDocument* doc = log->document();
	QString content( doc->toPlainText() );
	doc->setPlainText( content + text );
}

void ProgressDialog::closeEvent( QCloseEvent * event )
{
	event->ignore();
}

void ProgressDialog::timetick()
{
	float arg = -( aniPos/float( aniSteps ) - aniPhase + 1.0f );
	aniPos += expf( arg );
	if ( aniPos > aniSteps * aniPhase ) aniPos = aniSteps * aniPhase;
	progressBar->setValue( int( aniPos ) );
}

